@@ -58,7 +58,7 @@ public class CTabFolderRenderer {
 	 */
 	Color lastBorderColor = null;
 
-	Font chevronFont = null;
+	private Font chevronFont = null;
 
 	//TOP_LEFT_CORNER_HILITE is laid out in reverse (ie. top to bottom)
 	//so can fade in same direction as right swoop curve
@@ -542,33 +542,21 @@ public class CTabFolderRenderer {
 	protected void dispose() {
 		disposeAntialiasColors();
 		disposeSelectionHighlightGradientColors();
-		if (fillColor != null) {
-			fillColor.dispose();
-			fillColor = null;
-		}
+
+		fillColor = null;
+		minMaxBorderColor = null;
+
 		if (chevronFont != null) {
 			chevronFont.dispose();
 			chevronFont = null;
 		}
-		if (minMaxBorderColor != null) {
-			minMaxBorderColor.dispose();
-			minMaxBorderColor = null;
-		}
 	}
 
 	void disposeAntialiasColors() {
-		if (tabAreaColor != null) tabAreaColor.dispose();
-		if (selectedInnerColor != null) selectedInnerColor.dispose();
-		if (selectedOuterColor != null) selectedOuterColor.dispose();
 		tabAreaColor = selectedInnerColor = selectedOuterColor = null;
 	}
 
 	void disposeSelectionHighlightGradientColors() {
-		if(selectionHighlightGradientColorsCache == null)
-			return;
-		for (Color element : selectionHighlightGradientColorsCache) {
-			element.dispose();
-		}
 		selectionHighlightGradientColorsCache = null;
 	}
 
@@ -1741,6 +1729,13 @@ public class CTabFolderRenderer {
 		return true;
 	}
 
+	void resetChevronFont() {
+		if (chevronFont != null) {
+			chevronFont.dispose();
+			chevronFont = null;
+		}
+	}
+
 	void setSelectionHighlightGradientColor(Color start) {
 		//
 		//Set to null to match all the early return cases.
