@@ -18,6 +18,8 @@ import org.eclipse.swt.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.gtk3.*;
+import org.eclipse.swt.internal.gtk4.*;
 import org.eclipse.swt.widgets.*;
 
 /**
@@ -33,7 +35,7 @@ import org.eclipse.swt.widgets.*;
  * The DropTarget can receive data from within the same application or from other applications
  * (such as text dragged from a text editor like Word).</p>
  *
- * <code><pre>
+ * <pre><code>
  *	int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK;
  *	Transfer[] types = new Transfer[] {TextTransfer.getInstance()};
  *	DropTarget target = new DropTarget(label, operations);
@@ -47,7 +49,7 @@ import org.eclipse.swt.widgets.*;
  * <code>event.currentDataType</code> field.  When the data is dropped, it is the responsibility of
  * the application to copy this data for its own purposes.
  *
- * <code><pre>
+ * <pre><code>
  *	target.addDropListener (new DropTargetListener() {
  *		public void dragEnter(DropTargetEvent event) {};
  *		public void dragOver(DropTargetEvent event) {};
@@ -63,7 +65,7 @@ import org.eclipse.swt.widgets.*;
  *			label.setText ((String) event.data); // data copied to label text
  *		}
  * 	});
- * </pre></code>
+ * </code></pre>
  *
  * <dl>
  *	<dt><b>Styles</b></dt> <dd>DND.DROP_NONE, DND.DROP_COPY, DND.DROP_MOVE, DND.DROP_LINK</dd>
@@ -109,17 +111,16 @@ public class DropTarget extends Widget {
 	static Callback Drag_Data_Received;
 	static Callback Drag_Drop;
 
-	 static {
+	static {
 		Drag_Motion = new Callback(DropTarget.class, "Drag_Motion", 5); //$NON-NLS-1$
-		if (Drag_Motion.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
 		Drag_Leave = new Callback(DropTarget.class, "Drag_Leave", 3); //$NON-NLS-1$
-		if (Drag_Leave.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
 		Drag_Data_Received = new Callback(DropTarget.class, "Drag_Data_Received", 7); //$NON-NLS-1$
-		if (Drag_Data_Received.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
 		Drag_Drop = new Callback(DropTarget.class, "Drag_Drop", 5); //$NON-NLS-1$
-		if (Drag_Drop.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
 	}
 
+	/* GTK4 specific */
+	long dropController;
+
 /**
  * Creates a new <code>DropTarget</code> to allow data to be dropped on the specified
  * <code>Control</code>.
@@ -154,85 +155,93 @@ public class DropTarget extends Widget {
 public DropTarget(Control control, int style) {
 	super(control, checkStyle(style));
 	this.control = control;
-	if (Drag_Motion == null || Drag_Leave == null || Drag_Data_Received == null || Drag_Drop == null) {
-		 DND.error(DND.ERROR_CANNOT_INIT_DROP);
-	}
-	if (control.getData(DND.DROP_TARGET_KEY) != null) {
-		DND.error(DND.ERROR_CANNOT_INIT_DROP);
-	}
-	control.setData(DND.DROP_TARGET_KEY, this);
 
-	drag_motion_handler = OS.g_signal_connect(control.handle, OS.drag_motion, Drag_Motion.getAddress(), 0);
-	drag_leave_handler = OS.g_signal_connect(control.handle, OS.drag_leave, Drag_Leave.getAddress(), 0);
-	drag_data_received_handler = OS.g_signal_connect(control.handle, OS.drag_data_received, Drag_Data_Received.getAddress(), 0);
-	drag_drop_handler = OS.g_signal_connect(control.handle, OS.drag_drop, Drag_Drop.getAddress(), 0);
+	if (GTK.GTK4) {
+		int actions = opToOsOp(style);
+		dropController = GTK4.gtk_drop_target_async_new(0, actions);
 
-	// Dispose listeners
-	controlListener = event -> {
-		if (!DropTarget.this.isDisposed()){
-			DropTarget.this.dispose();
+		GTK4.gtk_widget_add_controller(control.handle, dropController);
+	} else {
+		if (Drag_Motion == null || Drag_Leave == null || Drag_Data_Received == null || Drag_Drop == null) {
+			DND.error(DND.ERROR_CANNOT_INIT_DROP);
 		}
-	};
-	control.addListener(SWT.Dispose, controlListener);
+		if (control.getData(DND.DROP_TARGET_KEY) != null) {
+			DND.error(DND.ERROR_CANNOT_INIT_DROP);
+		}
+		control.setData(DND.DROP_TARGET_KEY, this);
 
-	this.addListener(SWT.Dispose, event -> onDispose());
+		drag_motion_handler = OS.g_signal_connect(control.handle, OS.drag_motion, Drag_Motion.getAddress(), 0);
+		drag_leave_handler = OS.g_signal_connect(control.handle, OS.drag_leave, Drag_Leave.getAddress(), 0);
+		drag_data_received_handler = OS.g_signal_connect(control.handle, OS.drag_data_received, Drag_Data_Received.getAddress(), 0);
+		drag_drop_handler = OS.g_signal_connect(control.handle, OS.drag_drop, Drag_Drop.getAddress(), 0);
 
-	Object effect = control.getData(DEFAULT_DROP_TARGET_EFFECT);
-	if (effect instanceof DropTargetEffect) {
-		dropEffect = (DropTargetEffect) effect;
-	} else if (control instanceof Table) {
-		dropEffect = new TableDropTargetEffect((Table) control);
-	} else if (control instanceof Tree) {
-		dropEffect = new TreeDropTargetEffect((Tree) control);
-	}
-
-	dragOverHeartbeat = () -> {
-		Control control1 = DropTarget.this.control;
-		if (control1 == null || control1.isDisposed() || dragOverStart == 0) return;
-		long time = System.currentTimeMillis();
-		int delay = DRAGOVER_HYSTERESIS;
-		if (time < dragOverStart) {
-			delay = (int)(dragOverStart - time);
-		} else {
-			dragOverEvent.time += DRAGOVER_HYSTERESIS;
-			int allowedOperations = dragOverEvent.operations;
-			TransferData[] allowedTypes = dragOverEvent.dataTypes;
-			//pass a copy of data types in to listeners in case application modifies it
-			TransferData[] dataTypes = new TransferData[allowedTypes.length];
-			System.arraycopy(allowedTypes, 0, dataTypes, 0, dataTypes.length);
-
-			DNDEvent event = new DNDEvent();
-			event.widget = dragOverEvent.widget;
-			event.x = dragOverEvent.x;
-			event.y = dragOverEvent.y;
-			event.time = dragOverEvent.time;
-			event.feedback = DND.FEEDBACK_SELECT;
-			event.dataTypes = dataTypes;
-			event.dataType = selectedDataType;
-			event.operations = dragOverEvent.operations;
-			event.detail  = selectedOperation;
-			if (dropEffect != null) {
-				event.item = dropEffect.getItem(dragOverEvent.x, dragOverEvent.y);
+		// Dispose listeners
+		controlListener = event -> {
+			if (!DropTarget.this.isDisposed()){
+				DropTarget.this.dispose();
 			}
-			selectedDataType = null;
-			selectedOperation = DND.DROP_NONE;
-			notifyListeners(DND.DragOver, event);
-			if (event.dataType != null) {
-				for (int i = 0; i < allowedTypes.length; i++) {
-					if (allowedTypes[i].type == event.dataType.type) {
-						selectedDataType = event.dataType;
-						break;
+		};
+		control.addListener(SWT.Dispose, controlListener);
+
+		this.addListener(SWT.Dispose, event -> onDispose());
+
+		Object effect = control.getData(DEFAULT_DROP_TARGET_EFFECT);
+		if (effect instanceof DropTargetEffect) {
+			dropEffect = (DropTargetEffect) effect;
+		} else if (control instanceof Table) {
+			dropEffect = new TableDropTargetEffect((Table) control);
+		} else if (control instanceof Tree) {
+			dropEffect = new TreeDropTargetEffect((Tree) control);
+		}
+
+		dragOverHeartbeat = () -> {
+			Control control1 = DropTarget.this.control;
+			if (control1 == null || control1.isDisposed() || dragOverStart == 0) return;
+			long time = System.currentTimeMillis();
+			int delay = DRAGOVER_HYSTERESIS;
+			if (time < dragOverStart) {
+				delay = (int)(dragOverStart - time);
+			} else {
+				dragOverEvent.time += DRAGOVER_HYSTERESIS;
+				int allowedOperations = dragOverEvent.operations;
+				TransferData[] allowedTypes = dragOverEvent.dataTypes;
+				//pass a copy of data types in to listeners in case application modifies it
+				TransferData[] dataTypes = new TransferData[allowedTypes.length];
+				System.arraycopy(allowedTypes, 0, dataTypes, 0, dataTypes.length);
+
+				DNDEvent event = new DNDEvent();
+				event.widget = dragOverEvent.widget;
+				event.x = dragOverEvent.x;
+				event.y = dragOverEvent.y;
+				event.time = dragOverEvent.time;
+				event.feedback = DND.FEEDBACK_SELECT;
+				event.dataTypes = dataTypes;
+				event.dataType = selectedDataType;
+				event.operations = dragOverEvent.operations;
+				event.detail  = selectedOperation;
+				if (dropEffect != null) {
+					event.item = dropEffect.getItem(dragOverEvent.x, dragOverEvent.y);
+				}
+				selectedDataType = null;
+				selectedOperation = DND.DROP_NONE;
+				notifyListeners(DND.DragOver, event);
+				if (event.dataType != null) {
+					for (int i = 0; i < allowedTypes.length; i++) {
+						if (allowedTypes[i].type == event.dataType.type) {
+							selectedDataType = event.dataType;
+							break;
+						}
 					}
 				}
+				if (selectedDataType != null && (event.detail & allowedOperations) != 0) {
+					selectedOperation = event.detail;
+				}
 			}
-			if (selectedDataType != null && (event.detail & allowedOperations) != 0) {
-				selectedOperation = event.detail;
-			}
-		}
-		control1 = DropTarget.this.control;
-		if (control1 == null || control1.isDisposed()) return;
-		control1.getDisplay().timerExec(delay, dragOverHeartbeat);
-	};
+			control1 = DropTarget.this.control;
+			if (control1 == null || control1.isDisposed()) return;
+			control1.getDisplay().timerExec(delay, dragOverHeartbeat);
+		};
+	}
 }
 
 static int checkStyle (int style) {
@@ -240,33 +249,33 @@ static int checkStyle (int style) {
 	return style;
 }
 
-static long /*int*/ Drag_Data_Received ( long /*int*/ widget, long /*int*/ context, long /*int*/ x, long /*int*/ y, long /*int*/ data, long /*int*/ info, long /*int*/ time){
+static long Drag_Data_Received ( long widget, long context, long x, long y, long data, long info, long time){
 	DropTarget target = FindDropTarget(widget);
 	if (target == null) return 0;
-	target.drag_data_received (widget, context, (int)/*64*/x, (int)/*64*/y, data, (int)/*64*/info, (int)/*64*/time);
+	target.drag_data_received (widget, context, (int)x, (int)y, data, (int)info, (int)time);
 	return 0;
 }
 
-static long /*int*/ Drag_Drop(long /*int*/ widget, long /*int*/ context, long /*int*/ x, long /*int*/ y, long /*int*/ time) {
+static long Drag_Drop(long widget, long context, long x, long y, long time) {
 	DropTarget target = FindDropTarget(widget);
 	if (target == null) return 0;
-	return target.drag_drop (widget, context, (int)/*64*/x, (int)/*64*/y, (int)/*64*/time) ? 1 : 0;
+	return target.drag_drop (widget, context, (int)x, (int)y, (int)time) ? 1 : 0;
 }
 
-static long /*int*/ Drag_Leave ( long /*int*/ widget, long /*int*/ context, long /*int*/ time){
+static long Drag_Leave ( long widget, long context, long time){
 	DropTarget target = FindDropTarget(widget);
 	if (target == null) return 0;
-	target.drag_leave (widget, context, (int)/*64*/time);
+	target.drag_leave (widget, context, (int)time);
 	return 0;
 }
 
-static long /*int*/ Drag_Motion ( long /*int*/ widget, long /*int*/ context, long /*int*/ x, long /*int*/ y, long /*int*/ time){
+static long Drag_Motion ( long widget, long context, long x, long y, long time){
 	DropTarget target = FindDropTarget(widget);
 	if (target == null) return 0;
-	return target.drag_motion (widget, context, (int)/*64*/x, (int)/*64*/y, (int)/*64*/time) ? 1 : 0;
+	return target.drag_motion (widget, context, (int)x, (int)y, (int)time) ? 1 : 0;
 }
 
-static DropTarget FindDropTarget(long /*int*/ handle) {
+static DropTarget FindDropTarget(long handle) {
 	Display display = Display.findDisplay(Thread.currentThread());
 	if (display == null || display.isDisposed()) return null;
 	Widget widget = display.findWidget(handle);
@@ -280,7 +289,7 @@ static DropTarget FindDropTarget(long /*int*/ handle) {
  * it one of the messages defined in the <code>DropTargetListener</code>
  * interface.
  *
- * <p><ul>
+ * <ul>
  * <li><code>dragEnter</code> is called when the cursor has entered the drop target boundaries
  * <li><code>dragLeave</code> is called when the cursor has left the drop target boundaries and just before
  * the drop occurs or is cancelled.
@@ -290,7 +299,7 @@ static DropTarget FindDropTarget(long /*int*/ handle) {
  * <li><code>dropAccept</code> is called just before the drop is performed.  The drop target is given
  * the chance to change the nature of the drop or veto the drop by setting the <code>event.detail</code> field
  * <li><code>drop</code> is called when the data is being dropped
- * </ul></p>
+ * </ul>
  *
  * @param listener the listener which should be notified
  *
@@ -328,7 +337,7 @@ protected void checkSubclass () {
 	}
 }
 
-void drag_data_received ( long /*int*/ widget, long /*int*/ context, int x, int y, long /*int*/ selection_data, int info, int time){
+void drag_data_received ( long widget, long context, int x, int y, long selection_data, int info, int time){
 	DNDEvent event = new DNDEvent();
 	if (selection_data == 0 || !setEventData(context, x, y, time, event)) {
 		keyOperation = -1;
@@ -341,10 +350,10 @@ void drag_data_received ( long /*int*/ widget, long /*int*/ context, int x, int
 	// Get data in a Java format
 	Object object = null;
 	TransferData transferData = new TransferData();
-	int length = GTK.gtk_selection_data_get_length(selection_data);
-	int format = GTK.gtk_selection_data_get_format(selection_data);
-	long /*int*/ data = GTK.gtk_selection_data_get_data(selection_data);
-	long /*int*/ type = GTK.gtk_selection_data_get_data_type(selection_data);
+	int length = GTK3.gtk_selection_data_get_length(selection_data);
+	int format = GTK3.gtk_selection_data_get_format(selection_data);
+	long data = GTK3.gtk_selection_data_get_data(selection_data);
+	long type = GTK3.gtk_selection_data_get_data_type(selection_data);
 	if (data != 0) {
 		transferData.type = type;
 		transferData.length = length;
@@ -374,11 +383,11 @@ void drag_data_received ( long /*int*/ widget, long /*int*/ context, int x, int
 	OS.g_signal_stop_emission_by_name(widget, OS.drag_data_received);
 
 	//notify source of action taken
-	GTK.gtk_drag_finish(context, selectedOperation != DND.DROP_NONE, selectedOperation== DND.DROP_MOVE, time);
+	GTK3.gtk_drag_finish(context, selectedOperation != DND.DROP_NONE, selectedOperation== DND.DROP_MOVE, time);
 	return;
 }
 
-boolean drag_drop(long /*int*/ widget, long /*int*/ context, int x, int y, int time) {
+boolean drag_drop(long widget, long context, int x, int y, int time) {
 	DNDEvent event = new DNDEvent();
 	if (!setEventData(context, x, y, time, event)) {
 		keyOperation = -1;
@@ -411,11 +420,11 @@ boolean drag_drop(long /*int*/ widget, long /*int*/ context, int x, int y, int t
 		return false;
 	}
 	// ask drag source for dropped data
-	GTK.gtk_drag_get_data(widget, context, selectedDataType.type, time);
+	GTK3.gtk_drag_get_data(widget, context, selectedDataType.type, time);
 	return true;
 }
 
-void drag_leave ( long /*int*/ widget, long /*int*/ context, int time){
+void drag_leave ( long widget, long context, int time){
 	updateDragOverHover(0, null);
 
 	if (keyOperation == -1) return;
@@ -428,7 +437,7 @@ void drag_leave ( long /*int*/ widget, long /*int*/ context, int time){
 	notifyListeners(DND.DragLeave, event);
 }
 
-boolean drag_motion ( long /*int*/ widget, long /*int*/ context, int x, int y, int time){
+boolean drag_motion ( long widget, long context, int x, int y, int time){
 	int oldKeyOperation = keyOperation;
 
 	/*
@@ -567,12 +576,15 @@ public DropTargetEffect getDropTargetEffect() {
 
 int getOperationFromKeyState() {
 	int[] state = new int[1];
-	if (GTK.GTK3) {
-		long /*int*/ root = GDK.gdk_get_default_root_window ();
-		long /*int*/ pointer = GDK.gdk_get_pointer (GDK.gdk_window_get_display (root));
-		GDK.gdk_window_get_device_position(root, pointer, null, null, state);
+	long pointer = GDK.gdk_get_pointer (GDK.gdk_display_get_default());
+	if (GTK.GTK4) {
+		/*
+		 * TODO: calling gdk_window_get_device_position() here
+		 * uses gdk_get_default_root_window(), which doesn't exist on GTK4.
+		 */
 	} else {
-		GDK.gdk_window_get_pointer(0, null, null, state);
+		long root = GDK.gdk_get_default_root_window ();
+		GDK.gdk_window_get_device_position(root, pointer, null, null, state);
 	}
 	boolean ctrl = (state[0] & GDK.GDK_CONTROL_MASK) != 0;
 	boolean shift = (state[0] & GDK.GDK_SHIFT_MASK) != 0;
@@ -598,7 +610,7 @@ void onDispose(){
 	OS.g_signal_handler_disconnect(control.handle, drag_data_received_handler);
 	OS.g_signal_handler_disconnect(control.handle, drag_drop_handler);
 	if (transferAgents.length != 0)
-		GTK.gtk_drag_dest_unset(control.handle);
+		GTK3.gtk_drag_dest_unset(control.handle);
 	transferAgents = null;
 	if (controlListener != null)
 		control.removeListener(SWT.Dispose, controlListener);
@@ -671,51 +683,63 @@ public void removeDropListener(DropTargetListener listener) {
  * </ul>
  */
 public void setTransfer(Transfer... transferAgents){
-	if (transferAgents == null) DND.error(SWT.ERROR_NULL_ARGUMENT);
+	if (GTK.GTK4) {
+		this.transferAgents = transferAgents;
 
-	if (this.transferAgents.length != 0) {
-		GTK.gtk_drag_dest_unset(control.handle);
-	}
-	this.transferAgents = transferAgents;
-
-	GtkTargetEntry[] targets = new GtkTargetEntry[0];
-	for (int i = 0; i < transferAgents.length; i++) {
-		Transfer transfer = transferAgents[i];
-		if (transfer != null) {
-			int[] typeIds = transfer.getTypeIds();
-			String[] typeNames = transfer.getTypeNames();
-			for (int j = 0; j < typeIds.length; j++) {
-				GtkTargetEntry entry = new GtkTargetEntry();
-				byte[] buffer = Converter.wcsToMbcs(typeNames[j], true);
-				entry.target = OS.g_malloc(buffer.length);
-				C.memmove(entry.target, buffer, buffer.length);
-				entry.info = typeIds[j];
-				GtkTargetEntry[] newTargets = new GtkTargetEntry[targets.length + 1];
-				System.arraycopy(targets, 0, newTargets, 0, targets.length);
-				newTargets[targets.length] = entry;
-				targets = newTargets;
+		long contentFormatsBuilder = GTK4.gdk_content_formats_builder_new();
+		for (Transfer agent : transferAgents) {
+			for (String typeName : agent.getTypeNames()) {
+				GTK4.gdk_content_formats_builder_add_mime_type(contentFormatsBuilder, Converter.javaStringToCString(typeName));
 			}
 		}
-	}
+		GTK4.gtk_drop_target_async_set_formats(dropController, contentFormatsBuilder);
+	} else {
+		if (transferAgents == null) DND.error(SWT.ERROR_NULL_ARGUMENT);
 
-	long /*int*/ pTargets = OS.g_malloc(targets.length * GtkTargetEntry.sizeof);
-	for (int i = 0; i < targets.length; i++) {
-		OS.memmove(pTargets + i*GtkTargetEntry.sizeof, targets[i], GtkTargetEntry.sizeof);
-	}
+		if (this.transferAgents.length != 0) {
+			GTK3.gtk_drag_dest_unset(control.handle);
+		}
+		this.transferAgents = transferAgents;
 
-	int actions = opToOsOp(getStyle());
-	if (control instanceof Combo) {
-		if ((control.getStyle() & SWT.READ_ONLY) == 0) {
-			long /*int*/ entryHandle = GTK.gtk_bin_get_child (control.handle);
-			if (entryHandle != 0) {
-				GTK.gtk_drag_dest_unset(entryHandle);
+		GtkTargetEntry[] targets = new GtkTargetEntry[0];
+		for (int i = 0; i < transferAgents.length; i++) {
+			Transfer transfer = transferAgents[i];
+			if (transfer != null) {
+				int[] typeIds = transfer.getTypeIds();
+				String[] typeNames = transfer.getTypeNames();
+				for (int j = 0; j < typeIds.length; j++) {
+					GtkTargetEntry entry = new GtkTargetEntry();
+					byte[] buffer = Converter.wcsToMbcs(typeNames[j], true);
+					entry.target = OS.g_malloc(buffer.length);
+					C.memmove(entry.target, buffer, buffer.length);
+					entry.info = typeIds[j];
+					GtkTargetEntry[] newTargets = new GtkTargetEntry[targets.length + 1];
+					System.arraycopy(targets, 0, newTargets, 0, targets.length);
+					newTargets[targets.length] = entry;
+					targets = newTargets;
+				}
 			}
 		}
-	}
-	GTK.gtk_drag_dest_set(control.handle, 0, pTargets, targets.length, actions);
 
-	for (int i = 0; i < targets.length; i++) {
-		OS.g_free(targets[i].target);
+		long pTargets = OS.g_malloc(targets.length * GtkTargetEntry.sizeof);
+		for (int i = 0; i < targets.length; i++) {
+			OS.memmove(pTargets + i*GtkTargetEntry.sizeof, targets[i], GtkTargetEntry.sizeof);
+		}
+
+		int actions = opToOsOp(getStyle());
+		if (control instanceof Combo) {
+			if ((control.getStyle() & SWT.READ_ONLY) == 0) {
+				long entryHandle = GTK3.gtk_bin_get_child (control.handle);
+				if (entryHandle != 0) {
+					GTK3.gtk_drag_dest_unset(entryHandle);
+				}
+			}
+		}
+		GTK3.gtk_drag_dest_set(control.handle, 0, pTargets, targets.length, actions);
+
+		for (int i = 0; i < targets.length; i++) {
+			OS.g_free(targets[i].target);
+		}
 	}
 }
 
@@ -732,19 +756,10 @@ public void setDropTargetEffect(DropTargetEffect effect) {
 	dropEffect = effect;
 }
 
-boolean setEventData(long /*int*/ context, int x, int y, int time, DNDEvent event) {
+boolean setEventData(long context, int x, int y, int time, DNDEvent event) {
 	if (context == 0) return false;
-	long /*int*/ targets = 0;
-	int actions = 0;
-	if (GTK.GTK3) {
-		targets = GDK.gdk_drag_context_list_targets(context);
-		actions = GDK.gdk_drag_context_get_actions(context);
-	} else {
-		GdkDragContext dragContext = new GdkDragContext();
-		OS.memmove(dragContext, context, GdkDragContext.sizeof);
-		targets = dragContext.targets;
-		actions = dragContext.actions;
-	}
+	long targets = GDK.gdk_drag_context_list_targets(context);
+	int actions = GDK.gdk_drag_context_get_actions(context);
 	if (targets == 0) return false;
 
 	// get allowed operations
@@ -766,7 +781,7 @@ boolean setEventData(long /*int*/ context, int x, int y, int time, DNDEvent even
 	// Get allowed transfer types
 	TransferData[] dataTypes = new TransferData[0];
 	while (targets != 0) {
-		long /*int*/ pData = OS.g_list_data(targets);
+		long pData = OS.g_list_data(targets);
 		TransferData data = new TransferData();
 		data.type = pData;
 		for (int j = 0; j < transferAgents.length; j++) {
@@ -782,9 +797,13 @@ boolean setEventData(long /*int*/ context, int x, int y, int time, DNDEvent even
 		targets = OS.g_list_next (targets);
 	}
 	if (dataTypes.length == 0) return false;
-	long /*int*/ window = GTK.gtk_widget_get_window (control.handle);
 	int [] origin_x = new int[1], origin_y = new int[1];
-	GDK.gdk_window_get_origin(window, origin_x, origin_y);
+	if (GTK.GTK4) {
+		// TODO: GTK4 no gdk_surface_get_origin
+	} else {
+		long window = GTK3.gtk_widget_get_window (control.handle);
+		GDK.gdk_window_get_origin(window, origin_x, origin_y);
+	}
 	Point coordinates = DPIUtil.autoScaleDown(new Point(origin_x[0] + x, origin_y[0] + y));
 
 	event.widget = this;
