@@ -24,7 +24,7 @@ import org.eclipse.swt.internal.win32.*;
  * <p>An example of a java <code>String</code> containing RTF text is shown
  * below:</p>
  *
- * <code><pre>
+ * <pre><code>
  *     String rtfData = "{\\rtf1{\\colortbl;\\red255\\green0\\blue0;}\\uc1\\b\\i Hello World}";
  * </code></pre>
  *
@@ -74,7 +74,7 @@ public void javaToNative (Object object, TransferData transferData){
 		transferData.result = COM.DV_E_STGMEDIUM;
 		return;
 	}
-	long /*int*/ lpMultiByteStr = OS.GlobalAlloc(COM.GMEM_FIXED | COM.GMEM_ZEROINIT, cchMultiByte);
+	long lpMultiByteStr = OS.GlobalAlloc(COM.GMEM_FIXED | COM.GMEM_ZEROINIT, cchMultiByte);
 	OS.WideCharToMultiByte(codePage, 0, chars, -1, lpMultiByteStr, cchMultiByte, null, null);
 	transferData.stgmedium = new STGMEDIUM();
 	transferData.stgmedium.tymed = COM.TYMED_HGLOBAL;
@@ -105,9 +105,9 @@ public Object nativeToJava(TransferData transferData){
 	transferData.result = getData(data, formatetc, stgmedium);
 	data.Release();
 	if (transferData.result != COM.S_OK) return null;
-	long /*int*/ hMem = stgmedium.unionField;
+	long hMem = stgmedium.unionField;
 	try {
-		long /*int*/ lpMultiByteStr = OS.GlobalLock(hMem);
+		long lpMultiByteStr = OS.GlobalLock(hMem);
 		if (lpMultiByteStr == 0) return null;
 		try {
 			int codePage = OS.GetACP();
@@ -135,7 +135,7 @@ protected String[] getTypeNames(){
 }
 
 boolean checkRTF(Object object) {
-	return (object != null  && object instanceof String && ((String)object).length() > 0);
+	return (object instanceof String && ((String)object).length() > 0);
 }
 
 @Override
