@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -35,7 +38,7 @@ import org.eclipse.swt.widgets.*;
  * <p>Note that although this class is a subclass of <code>Composite</code>,
  * it does not make sense to add <code>Control</code> children to it,
  * or set a layout on it.
- * </p><p>
+ * </p>
  * <dl>
  *	<dt><b>Styles</b> <dd>BORDER
  *	<dt><b>Events</b> <dd>Dispose, Move, Resize
@@ -47,7 +50,6 @@ import org.eclipse.swt.widgets.*;
 public class OleClientSite extends Composite {
 
 	// Interfaces for this Ole Client Container
-	private COMObject  iUnknown;
 	COMObject  iOleClientSite;
 	private COMObject  iAdviseSink;
 	private COMObject  iOleInPlaceSite;
@@ -128,9 +130,14 @@ protected OleClientSite(Composite parent, int style) {
 			case SWT.Dispose : onDispose(e); break;
 			case SWT.FocusIn:
 				nestedFocusEvents++;
+				boolean hasFocus = isFocusControl();
 				onFocusIn(e);
 				nestedFocusEvents--;
-				if (nestedFocusEvents == 0)
+				/*
+				 * Added additional check below to avoid calling OleFrame#onFocusIn() twice,
+				 * which other wise lead to Main Menu refresh problem as seen in bug 527268
+				 */
+				if (nestedFocusEvents == 0 && hasFocus == isFocusControl())
 					frame.onFocusIn(e);
 				break;
 			case SWT.FocusOut:
@@ -188,7 +195,7 @@ public OleClientSite(Composite parent, int style, File file) {
 	try {
 
 		if (file == null || file.isDirectory() || !file.exists())
-			OLE.error(OLE.ERROR_INVALID_ARGUMENT);
+			OLE.error(SWT.ERROR_INVALID_ARGUMENT);
 
 		// Is there an associated CLSID?
 		GUID fileClsid = new GUID();
@@ -240,14 +247,14 @@ public OleClientSite(Composite parent, int style, String progId) {
 		tempStorage = createTempStorage();
 
 		// Create ole object with storage object
-		long /*int*/[] address = new long /*int*/[1];
+		long[] address = new long[1];
 		/*
 		* Bug in ICA Client 2.7. The creation of the IOleObject fails if the client
 		* site is provided to OleCreate().  The fix is to detect that the program
 		* id is an ICA Client and do not pass a client site to OleCreate().
 		* IOleObject.SetClientSite() is called later on.
 		*/
-		long /*int*/ clientSite = isICAClient() ? 0 : iOleClientSite.getAddress();
+		long clientSite = isICAClient() ? 0 : iOleClientSite.getAddress();
 		int result = COM.OleCreate(appClsid, COM.IIDIUnknown, COM.OLERENDER_DRAW, null, clientSite, tempStorage.getAddress(), address);
 		if (result != COM.S_OK)
 			OLE.error(OLE.ERROR_CANNOT_CREATE_OBJECT, result);
@@ -298,7 +305,7 @@ public OleClientSite(Composite parent, int style, String progId) {
 public OleClientSite(Composite parent, int style, String progId, File file) {
 	this(parent, style);
 	try {
-		if (file == null || file.isDirectory() || !file.exists()) OLE.error(OLE.ERROR_INVALID_ARGUMENT);
+		if (file == null || file.isDirectory() || !file.exists()) OLE.error(SWT.ERROR_INVALID_ARGUMENT);
 		appClsid = getClassID(progId);
 		if (appClsid == null) OLE.error(OLE.ERROR_INVALID_CLASSID);
 
@@ -328,7 +335,7 @@ void OleCreate(GUID appClsid, GUID fileClsid, char[] fileName, File file) {
 		// Using the same application that created file, therefore, use default mechanism.
 		tempStorage = createTempStorage();
 		// Create ole object with storage object
-		long /*int*/[] address = new long /*int*/[1];
+		long[] address = new long[1];
 		int result = COM.OleCreateFromFile(appClsid, fileName, COM.IIDIUnknown, COM.OLERENDER_DRAW, null, iOleClientSite.getAddress(), tempStorage.getAddress(), address);
 		if (result != COM.S_OK) OLE.error(OLE.ERROR_CANNOT_CREATE_OBJECT, result);
 		objIUnknown = new IUnknown(address[0]);
@@ -336,14 +343,14 @@ void OleCreate(GUID appClsid, GUID fileClsid, char[] fileName, File file) {
 		// Not using the same application that created file, therefore, copy from original file to a new storage file
 		IStorage storage = null;
 		if (COM.StgIsStorageFile(fileName) == COM.S_OK) {
-			long /*int*/[] address = new long /*int*/[1];
+			long[] address = new long[1];
 			int mode = COM.STGM_READ | COM.STGM_TRANSACTED | COM.STGM_SHARE_EXCLUSIVE;
 			int result = COM.StgOpenStorage(fileName, 0, mode, 0, 0, address); //Does an AddRef if successful
 			if (result != COM.S_OK) OLE.error(OLE.ERROR_CANNOT_OPEN_FILE, result);
 			storage = new IStorage(address[0]);
 		} else {
 			// Original file is not a Storage file so copy contents to a stream in a new storage file
-			long /*int*/[] address = new long /*int*/[1];
+			long[] address = new long[1];
 			int mode = COM.STGM_READWRITE | COM.STGM_DIRECT | COM.STGM_SHARE_EXCLUSIVE | COM.STGM_CREATE;
 			int result = COM.StgCreateDocfile(null, mode | COM.STGM_DELETEONRELEASE, 0, address); // Increments ref count if successful
 			if (result != COM.S_OK) OLE.error(OLE.ERROR_CANNOT_OPEN_FILE, result);
@@ -355,7 +362,7 @@ void OleCreate(GUID appClsid, GUID fileClsid, char[] fileName, File file) {
 			GUID wordGUID = getClassID(WORDPROGID);
 			if (wordGUID != null && COM.IsEqualGUID(appClsid, wordGUID)) streamName = "WordDocument"; //$NON-NLS-1$
 			if (isOffice2007) streamName = "Package"; //$NON-NLS-1$
-			address = new long /*int*/[1];
+			address = new long[1];
 			result = storage.CreateStream(streamName, mode, 0, 0, address); // Increments ref count if successful
 			if (result != COM.S_OK) {
 				storage.Release();
@@ -369,10 +376,10 @@ void OleCreate(GUID appClsid, GUID fileClsid, char[] fileName, File file) {
 				byte[] buffer = new byte[increment];
 				int count = 0;
 				while((count = fileInput.read(buffer)) > 0){
-					long /*int*/ pv = COM.CoTaskMemAlloc(count);
+					long pv = OS.CoTaskMemAlloc(count);
 					OS.MoveMemory(pv, buffer, count);
 					result = stream.Write(pv, count, null) ;
-					COM.CoTaskMemFree(pv);
+					OS.CoTaskMemFree(pv);
 					if (result != COM.S_OK) {
 						fileInput.close();
 						stream.Release();
@@ -398,12 +405,12 @@ void OleCreate(GUID appClsid, GUID fileClsid, char[] fileName, File file) {
 		if (result != COM.S_OK) OLE.error(OLE.ERROR_CANNOT_OPEN_FILE, result);
 
 		// create ole client
-		long /*int*/[] ppv = new long /*int*/[1];
+		long[] ppv = new long[1];
 		result = COM.CoCreateInstance(appClsid, 0, COM.CLSCTX_INPROC_HANDLER | COM.CLSCTX_INPROC_SERVER, COM.IIDIUnknown, ppv);
 		if (result != COM.S_OK) OLE.error(OLE.ERROR_CANNOT_CREATE_OBJECT, result);
 		objIUnknown = new IUnknown(ppv[0]);
 		// get the persistent storage of the ole client
-		ppv = new long /*int*/[1];
+		ppv = new long[1];
 		result = objIUnknown.QueryInterface(COM.IIDIPersistStorage, ppv);
 		if (result != COM.S_OK) OLE.error(OLE.ERROR_CANNOT_CREATE_OBJECT, result);
 		IPersistStorage iPersistStorage = new IPersistStorage(ppv[0]);
@@ -420,7 +427,7 @@ void OleCreate(GUID appClsid, GUID fileClsid, char[] fileName, File file) {
 }
 protected void addObjectReferences() {
 	//
-	long /*int*/[] ppvObject = new long /*int*/[1];
+	long[] ppvObject = new long[1];
 	if (objIUnknown.QueryInterface(COM.IIDIPersist, ppvObject) == COM.S_OK) {
 		IPersist objIPersist = new IPersist(ppvObject[0]);
 		GUID tempid = new GUID();
@@ -430,7 +437,7 @@ protected void addObjectReferences() {
 	}
 
 	//
-	ppvObject = new long /*int*/[1];
+	ppvObject = new long[1];
 	int result = objIUnknown.QueryInterface(COM.IIDIViewObject2, ppvObject);
 	if (result != COM.S_OK)
 		OLE.error(OLE.ERROR_INTERFACE_NOT_FOUND, result);
@@ -438,7 +445,7 @@ protected void addObjectReferences() {
 	objIViewObject2.SetAdvise(aspect, 0, iAdviseSink.getAddress());
 
 	//
-	ppvObject = new long /*int*/[1];
+	ppvObject = new long[1];
 	result = objIUnknown.QueryInterface(COM.IIDIOleObject, ppvObject);
 	if (result != COM.S_OK)
 		OLE.error(OLE.ERROR_INTERFACE_NOT_FOUND, result);
@@ -450,7 +457,7 @@ protected void addObjectReferences() {
 	 * during OleCreate. The fix is to check whether the clientSite has already been set
 	 * and set it. Note that setting it twice can result in assert failures.
 	 */
-	long /*int*/[] ppvClientSite = new long /*int*/[1];
+	long[] ppvClientSite = new long[1];
 	result = objIOleObject.GetClientSite(ppvClientSite);
 	if (ppvClientSite[0] == 0) {
 		objIOleObject.SetClientSite(iOleClientSite.getAddress());
@@ -465,13 +472,12 @@ protected void addObjectReferences() {
 	COM.OleSetContainedObject(objIUnknown.getAddress(), true);
 
 	// Is OLE object linked or embedded?
-	ppvObject = new long /*int*/[1];
+	ppvObject = new long[1];
 	if (objIUnknown.QueryInterface(COM.IIDIOleLink, ppvObject) == COM.S_OK) {
 		IOleLink objIOleLink = new IOleLink(ppvObject[0]);
-		long /*int*/[] ppmk = new long /*int*/[1];
+		long[] ppmk = new long[1];
 		if (objIOleLink.GetSourceMoniker(ppmk) == COM.S_OK) {
-			IMoniker objIMoniker = new IMoniker(ppmk[0]);
-			objIMoniker.Release();
+			new IUnknown(ppmk[0]).Release();
 			type = COM.OLELINKED;
 			objIOleLink.BindIfRunning();
 		} else {
@@ -495,102 +501,87 @@ private int ContextSensitiveHelp(int fEnterMode) {
 }
 protected void createCOMInterfaces() {
 
-	iUnknown = new COMObject(new int[]{2, 0, 0}){
-		@Override
-		public long /*int*/ method0(long /*int*/[] args) {return QueryInterface(args[0], args[1]);}
-		@Override
-		public long /*int*/ method1(long /*int*/[] args) {return AddRef();}
-		@Override
-		public long /*int*/ method2(long /*int*/[] args) {return Release();}
-	};
-
 	iOleClientSite = new COMObject(new int[]{2, 0, 0, 0, 3, 1, 0, 1, 0}){
 		@Override
-		public long /*int*/ method0(long /*int*/[] args) {return QueryInterface(args[0], args[1]);}
+		public long method0(long[] args) {return QueryInterface(args[0], args[1]);}
 		@Override
-		public long /*int*/ method1(long /*int*/[] args) {return AddRef();}
+		public long method1(long[] args) {return AddRef();}
 		@Override
-		public long /*int*/ method2(long /*int*/[] args) {return Release();}
+		public long method2(long[] args) {return Release();}
 		@Override
-		public long /*int*/ method3(long /*int*/[] args) {return SaveObject();}
+		public long method3(long[] args) {return SaveObject();}
 		// method4 GetMoniker - not implemented
 		@Override
-		public long /*int*/ method5(long /*int*/[] args) {return GetContainer(args[0]);}
+		public long method5(long[] args) {return GetContainer(args[0]);}
 		@Override
-		public long /*int*/ method6(long /*int*/[] args) {return ShowObject();}
+		public long method6(long[] args) {return ShowObject();}
 		@Override
-		public long /*int*/ method7(long /*int*/[] args) {return OnShowWindow((int)/*64*/args[0]);}
+		public long method7(long[] args) {return OnShowWindow((int)args[0]);}
 		// method8 RequestNewObjectLayout - not implemented
 	};
 
 	iAdviseSink = new COMObject(new int[]{2, 0, 0, 2, 2, 1, 0, 0}){
 		@Override
-		public long /*int*/ method0(long /*int*/[] args) {return QueryInterface(args[0], args[1]);}
+		public long method0(long[] args) {return QueryInterface(args[0], args[1]);}
 		@Override
-		public long /*int*/ method1(long /*int*/[] args) {return AddRef();}
+		public long method1(long[] args) {return AddRef();}
 		@Override
-		public long /*int*/ method2(long /*int*/[] args) {return Release();}
+		public long method2(long[] args) {return Release();}
 		@Override
-		public long /*int*/ method3(long /*int*/[] args) {return OnDataChange(args[0], args[1]);}
+		public long method3(long[] args) {return OnDataChange(args[0], args[1]);}
 		@Override
-		public long /*int*/ method4(long /*int*/[] args) {return OnViewChange((int)/*64*/args[0], (int)/*64*/args[1]);}
+		public long method4(long[] args) {return OnViewChange((int)args[0], (int)args[1]);}
 		//method5 OnRename - not implemented
 		@Override
-		public long /*int*/ method6(long /*int*/[] args) {OnSave();return 0;}
+		public long method6(long[] args) {OnSave();return 0;}
 		@Override
-		public long /*int*/ method7(long /*int*/[] args) {return OnClose();}
+		public long method7(long[] args) {return OnClose();}
 	};
 
-	iOleInPlaceSite = new COMObject(new int[]{2, 0, 0, 1, 1, 0, 0, 0, 5, C.PTR_SIZEOF == 4 ? 2 : 1, 1, 0, 0, 0, 1}){
+	iOleInPlaceSite = new COMObject(new int[]{2, 0, 0, 1, 1, 0, 0, 0, 5, 1, 1, 0, 0, 0, 1}){
 		@Override
-		public long /*int*/ method0(long /*int*/[] args) {return QueryInterface(args[0], args[1]);}
+		public long method0(long[] args) {return QueryInterface(args[0], args[1]);}
 		@Override
-		public long /*int*/ method1(long /*int*/[] args) {return AddRef();}
+		public long method1(long[] args) {return AddRef();}
 		@Override
-		public long /*int*/ method2(long /*int*/[] args) {return Release();}
+		public long method2(long[] args) {return Release();}
 		@Override
-		public long /*int*/ method3(long /*int*/[] args) {return GetWindow(args[0]);}
+		public long method3(long[] args) {return GetWindow(args[0]);}
 		@Override
-		public long /*int*/ method4(long /*int*/[] args) {return ContextSensitiveHelp((int)/*64*/args[0]);}
+		public long method4(long[] args) {return ContextSensitiveHelp((int)args[0]);}
 		@Override
-		public long /*int*/ method5(long /*int*/[] args) {return CanInPlaceActivate();}
+		public long method5(long[] args) {return CanInPlaceActivate();}
 		@Override
-		public long /*int*/ method6(long /*int*/[] args) {return OnInPlaceActivate();}
+		public long method6(long[] args) {return OnInPlaceActivate();}
 		@Override
-		public long /*int*/ method7(long /*int*/[] args) {return OnUIActivate();}
+		public long method7(long[] args) {return OnUIActivate();}
 		@Override
-		public long /*int*/ method8(long /*int*/[] args) {return GetWindowContext(args[0], args[1], args[2], args[3], args[4]);}
+		public long method8(long[] args) {return GetWindowContext(args[0], args[1], args[2], args[3], args[4]);}
 		@Override
-		public long /*int*/ method9(long /*int*/[] args) {
-			if (args.length == 2) {
-				return Scroll((int)/*64*/args[0], (int)/*64*/args[1]);
-			} else {
-				return Scroll_64(args[0]);
-			}
-		}
+		public long method9(long[] args) {return Scroll(args[0]);}
 		@Override
-		public long /*int*/ method10(long /*int*/[] args) {return OnUIDeactivate((int)/*64*/args[0]);}
+		public long method10(long[] args) {return OnUIDeactivate((int)args[0]);}
 		@Override
-		public long /*int*/ method11(long /*int*/[] args) {return OnInPlaceDeactivate();}
+		public long method11(long[] args) {return OnInPlaceDeactivate();}
 		// method12 DiscardUndoState - not implemented
 		// method13 DeactivateAndUndoChange - not implemented
 		@Override
-		public long /*int*/ method14(long /*int*/[] args) {return OnPosRectChange(args[0]);}
+		public long method14(long[] args) {return OnPosRectChange(args[0]);}
 	};
 
 	iOleDocumentSite = new COMObject(new int[]{2, 0, 0, 1}){
 		@Override
-		public long /*int*/ method0(long /*int*/[] args) {return QueryInterface(args[0], args[1]);}
+		public long method0(long[] args) {return QueryInterface(args[0], args[1]);}
 		@Override
-		public long /*int*/ method1(long /*int*/[] args) {return AddRef();}
+		public long method1(long[] args) {return AddRef();}
 		@Override
-		public long /*int*/ method2(long /*int*/[] args) {return Release();}
+		public long method2(long[] args) {return Release();}
 		@Override
-		public long /*int*/ method3(long /*int*/[] args) {return ActivateMe(args[0]);}
+		public long method3(long[] args) {return ActivateMe(args[0]);}
 	};
 }
 protected IStorage createTempStorage() {
-	long /*int*/[] tempStorage = new long /*int*/[1];
+	long[] tempStorage = new long[1];
 	int grfMode = COM.STGM_READWRITE | COM.STGM_SHARE_EXCLUSIVE | COM.STGM_DELETEONRELEASE;
 	int result = COM.StgCreateDocfile(null, grfMode, 0, tempStorage);
 	if (result != COM.S_OK) OLE.error(OLE.ERROR_CANNOT_CREATE_FILE, result);
@@ -612,12 +603,8 @@ private void deleteTempStorage() {
 	tempStorage = null;
 }
 protected void disposeCOMInterfaces() {
-	if (iUnknown != null)
-		iUnknown.dispose();
-	iUnknown = null;
-
 	if (iOleClientSite != null)
-	iOleClientSite.dispose();
+		iOleClientSite.dispose();
 	iOleClientSite = null;
 
 	if (iAdviseSink != null)
@@ -680,18 +667,18 @@ public int doVerb(int verb) {
 public int exec(int cmdID, int options, Variant in, Variant out) {
 
 	if (objIOleCommandTarget == null) {
-		long /*int*/[] address = new long /*int*/[1];
+		long[] address = new long[1];
 		if (objIUnknown.QueryInterface(COM.IIDIOleCommandTarget, address) != COM.S_OK)
 			return OLE.ERROR_INTERFACE_NOT_FOUND;
 		objIOleCommandTarget = new IOleCommandTarget(address[0]);
 	}
 
-	long /*int*/ inAddress = 0;
+	long inAddress = 0;
 	if (in != null){
 		inAddress = OS.GlobalAlloc(OS.GMEM_FIXED | OS.GMEM_ZEROINIT, VARIANT.sizeof);
 		in.getData(inAddress);
 	}
-	long /*int*/ outAddress = 0;
+	long outAddress = 0;
 	if (out != null){
 		outAddress = OS.GlobalAlloc(OS.GMEM_FIXED | OS.GMEM_ZEROINIT, VARIANT.sizeof);
 		out.getData(outAddress);
@@ -712,7 +699,7 @@ public int exec(int cmdID, int options, Variant in, Variant out) {
 	return result;
 }
 IDispatch getAutomationObject() {
-	long /*int*/[] ppvObject = new long /*int*/[1];
+	long[] ppvObject = new long[1];
 	if (objIUnknown.QueryInterface(COM.IIDIDispatch, ppvObject) != COM.S_OK)
 		return null;
 	return new IDispatch(ppvObject[0]);
@@ -734,13 +721,13 @@ protected GUID getClassID(String clientName) {
 	}
 	return guid;
 }
-private int GetContainer(long /*int*/ ppContainer) {
+private int GetContainer(long ppContainer) {
 	/* Simple containers that do not support links to their embedded
 	 * objects probably do not need to implement this method. Instead,
 	 * they can return E_NOINTERFACE and set ppContainer to NULL.
 	 */
 	if (ppContainer != 0)
-		COM.MoveMemory(ppContainer, new long /*int*/[]{0}, OS.PTR_SIZEOF);
+		OS.MoveMemory(ppContainer, new long[]{0}, C.PTR_SIZEOF);
 	return COM.E_NOINTERFACE;
 }
 private SIZE getExtent() {
@@ -748,7 +735,7 @@ private SIZE getExtent() {
 	// get the current size of the embedded OLENatives object
 	if (objIOleObject != null) {
 		if ( objIViewObject2 != null && !COM.OleIsRunning(objIOleObject.getAddress())) {
-			objIViewObject2.GetExtent(aspect, -1, null, sizel);
+			objIViewObject2.GetExtent(aspect, -1, 0, sizel);
 		} else {
 			objIOleObject.GetExtent(aspect, sizel);
 		}
@@ -776,13 +763,13 @@ public String getProgramID(){
 }
 String getProgID(GUID clsid) {
 	if (clsid != null){
-		long /*int*/[] lplpszProgID = new long /*int*/[1];
+		long[] lplpszProgID = new long[1];
 		if (COM.ProgIDFromCLSID(clsid, lplpszProgID) == COM.S_OK) {
-			long /*int*/ hMem = lplpszProgID[0];
+			long hMem = lplpszProgID[0];
 			int length = OS.GlobalSize(hMem);
-			long /*int*/ ptr = OS.GlobalLock(hMem);
+			long ptr = OS.GlobalLock(hMem);
 			char[] buffer = new char[length];
-			COM.MoveMemory(buffer, ptr, length);
+			OS.MoveMemory(buffer, ptr, length);
 			OS.GlobalUnlock(hMem);
 			OS.GlobalFree(hMem);
 
@@ -794,9 +781,9 @@ String getProgID(GUID clsid) {
 	}
 	return null;
 }
-int ActivateMe(long /*int*/ pViewToActivate) {
+int ActivateMe(long pViewToActivate) {
 	if (pViewToActivate == 0) {
-		long /*int*/[] ppvObject = new long /*int*/[1];
+		long[] ppvObject = new long[1];
 		if (objIUnknown.QueryInterface(COM.IIDIOleDocument, ppvObject) != COM.S_OK) return COM.E_FAIL;
 		IOleDocument objOleDocument = new IOleDocument(ppvObject[0]);
 		if (objOleDocument.CreateView(iOleInPlaceSite.getAddress(), 0, 0, ppvObject) != COM.S_OK) return COM.E_FAIL;
@@ -813,16 +800,16 @@ int ActivateMe(long /*int*/ pViewToActivate) {
 	objDocumentView.Show(1);//TRUE
 	return COM.S_OK;
 }
-protected int GetWindow(long /*int*/ phwnd) {
+protected int GetWindow(long phwnd) {
 	if (phwnd == 0)
 		return COM.E_INVALIDARG;
 	if (frame == null) {
-		COM.MoveMemory(phwnd, new long /*int*/[] {0}, OS.PTR_SIZEOF);
+		OS.MoveMemory(phwnd, new long[] {0}, C.PTR_SIZEOF);
 		return COM.E_NOTIMPL;
 	}
 
 	// Copy the Window's handle into the memory passed in
-	COM.MoveMemory(phwnd, new long /*int*/[] {handle}, OS.PTR_SIZEOF);
+	OS.MoveMemory(phwnd, new long[] {handle}, C.PTR_SIZEOF);
 	return COM.S_OK;
 }
 RECT getRect() {
@@ -834,17 +821,17 @@ RECT getRect() {
 	rect.bottom = area.y + area.height;
 	return rect;
 }
-private int GetWindowContext(long /*int*/ ppFrame, long /*int*/ ppDoc, long /*int*/ lprcPosRect, long /*int*/ lprcClipRect, long /*int*/ lpFrameInfo) {
+private int GetWindowContext(long ppFrame, long ppDoc, long lprcPosRect, long lprcClipRect, long lpFrameInfo) {
 	if (frame == null || ppFrame == 0)
 		return COM.E_NOTIMPL;
 
 	// fill in frame handle
-	long /*int*/ iOleInPlaceFrame = frame.getIOleInPlaceFrame();
-	COM.MoveMemory(ppFrame, new long /*int*/[] {iOleInPlaceFrame}, OS.PTR_SIZEOF);
+	long iOleInPlaceFrame = frame.getIOleInPlaceFrame();
+	OS.MoveMemory(ppFrame, new long[] {iOleInPlaceFrame}, C.PTR_SIZEOF);
 	frame.AddRef();
 
 	// null out document handle
-	if (ppDoc != 0) COM.MoveMemory(ppDoc, new long /*int*/[] {0}, OS.PTR_SIZEOF);
+	if (ppDoc != 0) OS.MoveMemory(ppDoc, new long[] {0}, C.PTR_SIZEOF);
 
 	// fill in position and clipping info
 	RECT rect = getRect();
@@ -859,10 +846,10 @@ private int GetWindowContext(long /*int*/ ppFrame, long /*int*/ ppDoc, long /*in
 	Shell shell = getShell();
 	Menu menubar = shell.getMenuBar();
 	if (menubar != null && !menubar.isDisposed()) {
-		long /*int*/ hwnd = shell.handle;
-		int cAccel = (int)/*64*/OS.SendMessage(hwnd, OS.WM_APP, 0, 0);
+		long hwnd = shell.handle;
+		int cAccel = (int)OS.SendMessage(hwnd, OS.WM_APP, 0, 0);
 		if (cAccel != 0) {
-			long /*int*/ hAccel = OS.SendMessage(hwnd, OS.WM_APP+1, 0, 0);
+			long hAccel = OS.SendMessage(hwnd, OS.WM_APP+1, 0, 0);
 			if (hAccel != 0) {
 				frameInfo.cAccelEntries = cAccel;
 				frameInfo.haccel = hAccel;
@@ -892,7 +879,7 @@ public boolean isDirty() {
 	 */
 
 	// Get access to the persistent storage mechanism
-	long /*int*/[] address = new long /*int*/[1];
+	long[] address = new long[1];
 	if (objIOleObject.QueryInterface(COM.IIDIPersistFile, address) != COM.S_OK)
 		return true;
 	IPersistFile permStorage = new IPersistFile(address[0]);
@@ -905,9 +892,9 @@ public boolean isDirty() {
 @Override
 public boolean isFocusControl () {
 	checkWidget ();
-	long /*int*/ focusHwnd = OS.GetFocus();
+	long focusHwnd = OS.GetFocus();
 	if (objIOleInPlaceObject == null) return (handle == focusHwnd);
-	long /*int*/[] phwnd = new long /*int*/[1];
+	long[] phwnd = new long[1];
 	objIOleInPlaceObject.GetWindow(phwnd);
 	while (focusHwnd != 0) {
 		if (phwnd[0] == focusHwnd) return true;
@@ -935,7 +922,7 @@ private boolean isOffice2007(boolean program) {
 private int OnClose() {
 	return COM.S_OK;
 }
-private int OnDataChange(long /*int*/ pFormatetc, long /*int*/ pStgmed) {
+private int OnDataChange(long pFormatetc, long pStgmed) {
 	return COM.S_OK;
 }
 private void onDispose(Event e) {
@@ -964,7 +951,7 @@ private void onDispose(Event e) {
 void onFocusIn(Event e) {
 	if (inDispose) return;
 	if (state != STATE_UIACTIVE) {
-		long /*int*/[] ppvObject = new long /*int*/[1];
+		long[] ppvObject = new long[1];
 		if (objIUnknown.QueryInterface(COM.IIDIOleInPlaceObject, ppvObject) == COM.S_OK) {
 			IOleInPlaceObject objIOleInPlaceObject = new IOleInPlaceObject(ppvObject[0]);
 			objIOleInPlaceObject.Release();
@@ -973,7 +960,7 @@ void onFocusIn(Event e) {
 	}
 	if (objIOleInPlaceObject == null) return;
 	if (isFocusControl()) return;
-	long /*int*/[] phwnd = new long /*int*/[1];
+	long[] phwnd = new long[1];
 	objIOleInPlaceObject.GetWindow(phwnd);
 	if (phwnd[0] == 0) return;
 	OS.SetFocus(phwnd[0]);
@@ -985,7 +972,7 @@ private int OnInPlaceActivate() {
 	frame.setCurrentDocument(this);
 	if (objIOleObject == null)
 		return COM.S_OK;
-	long /*int*/[] ppvObject = new long /*int*/[1];
+	long[] ppvObject = new long[1];
 	if (objIOleObject.QueryInterface(COM.IIDIOleInPlaceObject, ppvObject) == COM.S_OK) {
 		objIOleInPlaceObject = new IOleInPlaceObject(ppvObject[0]);
 	}
@@ -1002,7 +989,7 @@ private int OnInPlaceDeactivate() {
 	}
 	return COM.S_OK;
 }
-private int OnPosRectChange(long /*int*/ lprcPosRect) {
+private int OnPosRectChange(long lprcPosRect) {
 	Point size = DPIUtil.autoScaleUp(getSize()); // To Pixels
 	setExtent(size.x, size.y);
 	return COM.S_OK;
@@ -1020,7 +1007,7 @@ private void onPaint(Event e) {
 			rect.top = area.y; rect.bottom = area.y + size.cy;
 		}
 
-		long /*int*/ pArea = OS.GlobalAlloc(COM.GMEM_FIXED | COM.GMEM_ZEROINIT, RECT.sizeof);
+		long pArea = OS.GlobalAlloc(COM.GMEM_FIXED | COM.GMEM_ZEROINIT, RECT.sizeof);
 		OS.MoveMemory(pArea, rect, RECT.sizeof);
 		COM.OleDraw(objIUnknown.getAddress(), aspect, e.gc.handle, pArea);
 		OS.GlobalFree(pArea);
@@ -1037,7 +1024,7 @@ private int OnShowWindow(int fShow) {
 private int OnUIActivate() {
 	if (objIOleInPlaceObject == null) return COM.E_FAIL;
 	state = STATE_UIACTIVE;
-	long /*int*/[] phwnd = new long /*int*/[1];
+	long[] phwnd = new long[1];
 	if (objIOleInPlaceObject.GetWindow(phwnd) == COM.S_OK) {
 		OS.SetWindowPos(phwnd[0], OS.HWND_TOP, 0, 0, 0, 0, OS.SWP_NOSIZE | OS.SWP_NOMOVE);
 	}
@@ -1057,7 +1044,7 @@ int OnUIDeactivate(int fUndoable) {
 	if (menubar == null || menubar.isDisposed())
 		return COM.S_OK;
 
-	long /*int*/ shellHandle = shell.handle;
+	long shellHandle = shell.handle;
 	OS.SetMenu(shellHandle, menubar.handle);
 	return COM.OleSetMenuDescriptor(0, shellHandle, 0, 0, 0);
 }
@@ -1077,42 +1064,37 @@ private void onTraverse(Event event) {
 private int OnViewChange(int dwAspect, int lindex) {
 	return COM.S_OK;
 }
-protected int QueryInterface(long /*int*/ riid, long /*int*/ ppvObject) {
+protected int QueryInterface(long riid, long ppvObject) {
 
 	if (riid == 0 || ppvObject == 0)
 		return COM.E_NOINTERFACE;
 	GUID guid = new GUID();
 	COM.MoveMemory(guid, riid, GUID.sizeof);
 
-	if (COM.IsEqualGUID(guid, COM.IIDIUnknown)) {
-		COM.MoveMemory(ppvObject, new long /*int*/[] {iUnknown.getAddress()}, OS.PTR_SIZEOF);
+	if (COM.IsEqualGUID(guid, COM.IIDIUnknown) || COM.IsEqualGUID(guid, COM.IIDIOleClientSite)) {
+		OS.MoveMemory(ppvObject, new long[] {iOleClientSite.getAddress()}, C.PTR_SIZEOF);
 		AddRef();
 		return COM.S_OK;
 	}
 	if (COM.IsEqualGUID(guid, COM.IIDIAdviseSink)) {
-		COM.MoveMemory(ppvObject, new long /*int*/[] {iAdviseSink.getAddress()}, OS.PTR_SIZEOF);
-		AddRef();
-		return COM.S_OK;
-	}
-	if (COM.IsEqualGUID(guid, COM.IIDIOleClientSite)) {
-		COM.MoveMemory(ppvObject, new long /*int*/[] {iOleClientSite.getAddress()}, OS.PTR_SIZEOF);
+		OS.MoveMemory(ppvObject, new long[] {iAdviseSink.getAddress()}, C.PTR_SIZEOF);
 		AddRef();
 		return COM.S_OK;
 	}
 	if (COM.IsEqualGUID(guid, COM.IIDIOleInPlaceSite)) {
-		COM.MoveMemory(ppvObject, new long /*int*/[] {iOleInPlaceSite.getAddress()}, OS.PTR_SIZEOF);
+		OS.MoveMemory(ppvObject, new long[] {iOleInPlaceSite.getAddress()}, C.PTR_SIZEOF);
 		AddRef();
 		return COM.S_OK;
 	}
 	if (COM.IsEqualGUID(guid, COM.IIDIOleDocumentSite )) {
 		String progID = getProgramID();
 		if (!progID.startsWith("PowerPoint")) { //$NON-NLS-1$
-			COM.MoveMemory(ppvObject, new long /*int*/[] {iOleDocumentSite.getAddress()}, OS.PTR_SIZEOF);
+			OS.MoveMemory(ppvObject, new long[] {iOleDocumentSite.getAddress()}, C.PTR_SIZEOF);
 			AddRef();
 			return COM.S_OK;
 		}
 	}
-	COM.MoveMemory(ppvObject, new long /*int*/[] {0}, OS.PTR_SIZEOF);
+	OS.MoveMemory(ppvObject, new long[] {0}, C.PTR_SIZEOF);
 	return COM.E_NOINTERFACE;
 }
 /**
@@ -1130,7 +1112,7 @@ protected int QueryInterface(long /*int*/ riid, long /*int*/ ppvObject) {
 public int queryStatus(int cmd) {
 
 	if (objIOleCommandTarget == null) {
-		long /*int*/[] address = new long /*int*/[1];
+		long[] address = new long[1];
 		if (objIUnknown.QueryInterface(COM.IIDIOleCommandTarget, address) != COM.S_OK)
 			return 0;
 		objIOleCommandTarget = new IOleCommandTarget(address[0]);
@@ -1139,7 +1121,7 @@ public int queryStatus(int cmd) {
 	OLECMD olecmd = new OLECMD();
 	olecmd.cmdID = cmd;
 
-	int result = objIOleCommandTarget.QueryStatus(null, 1, olecmd, null);
+	int result = objIOleCommandTarget.QueryStatus(null, 1, olecmd, 0);
 
 	if (result != COM.S_OK) return 0;
 
@@ -1213,7 +1195,7 @@ public boolean save(File file, boolean includeOleInfo) {
 		return saveToStorageFile(file);
 	return saveToTraditionalFile(file);
 }
-private boolean saveFromContents(long /*int*/ address, File file) {
+private boolean saveFromContents(long address, File file) {
 
 	boolean success = false;
 
@@ -1224,7 +1206,7 @@ private boolean saveFromContents(long /*int*/ address, File file) {
 		FileOutputStream writer = new FileOutputStream(file);
 
 		int increment = 1024 * 4;
-		long /*int*/ pv = COM.CoTaskMemAlloc(increment);
+		long pv = OS.CoTaskMemAlloc(increment);
 		int[] pcbWritten = new int[1];
 		while (tempContents.Read(pv, increment, pcbWritten) == COM.S_OK && pcbWritten[0] > 0) {
 			byte[] buffer = new byte[ pcbWritten[0]];
@@ -1233,7 +1215,7 @@ private boolean saveFromContents(long /*int*/ address, File file) {
 			                      // first time it is called
 			success = true;
 		}
-		COM.CoTaskMemFree(pv);
+		OS.CoTaskMemFree(pv);
 
 		writer.close();
 
@@ -1244,7 +1226,7 @@ private boolean saveFromContents(long /*int*/ address, File file) {
 
 	return success;
 }
-private boolean saveFromOle10Native(long /*int*/ address, File file) {
+private boolean saveFromOle10Native(long address, File file) {
 
 	boolean success = false;
 
@@ -1253,19 +1235,19 @@ private boolean saveFromOle10Native(long /*int*/ address, File file) {
 
 	// The "\1Ole10Native" stream contains a DWORD header whose value is the length
 	// of the native data that follows.
-	long /*int*/ pv = COM.CoTaskMemAlloc(4);
+	long pv = OS.CoTaskMemAlloc(4);
 	int[] size = new int[1];
 	int rc = tempContents.Read(pv, 4, null);
 	OS.MoveMemory(size, pv, 4);
-	COM.CoTaskMemFree(pv);
+	OS.CoTaskMemFree(pv);
 	if (rc == COM.S_OK && size[0] > 0) {
 
 		// Read the data
 		byte[] buffer = new byte[size[0]];
-		pv = COM.CoTaskMemAlloc(size[0]);
+		pv = OS.CoTaskMemAlloc(size[0]);
 		rc = tempContents.Read(pv, size[0], null);
 		OS.MoveMemory(buffer, pv, size[0]);
-		COM.CoTaskMemFree(pv);
+		OS.CoTaskMemFree(pv);
 
 		// open the file and write data into it
 		try {
@@ -1293,14 +1275,14 @@ private boolean saveOffice2007(File file) {
 	boolean result = false;
 
 	/* Excel fails to open the package stream when the PersistStorage is not in hands off mode */
-	long /*int*/[] ppv = new long /*int*/[1];
+	long[] ppv = new long[1];
 	IPersistStorage iPersistStorage = null;
 	if (objIUnknown.QueryInterface(COM.IIDIPersistStorage, ppv) == COM.S_OK) {
 		iPersistStorage = new IPersistStorage(ppv[0]);
 		tempStorage.AddRef();
 		iPersistStorage.HandsOffStorage();
 	}
-	long /*int*/[] address = new long /*int*/[1];
+	long[] address = new long[1];
 	int grfMode = COM.STGM_DIRECT | COM.STGM_READ | COM.STGM_SHARE_EXCLUSIVE;
 	if (tempStorage.OpenStream("Package", 0, grfMode, 0, address) == COM.S_OK) { //$NON-NLS-1$
 		result = saveFromContents(address[0], file);
@@ -1334,11 +1316,11 @@ private boolean saveToStorageFile(File file) {
 	if (!updateStorage()) return false;
 
 	// get access to the persistent storage mechanism
-	long /*int*/[] address = new long /*int*/[1];
+	long[] address = new long[1];
 	if (objIOleObject.QueryInterface(COM.IIDIPersistStorage, address) != COM.S_OK) return false;
 	IPersistStorage permStorage = new IPersistStorage(address[0]);
 	try {
-		address = new long /*int*/[1];
+		address = new long[1];
 		char[] path = (file.getAbsolutePath()+"\0").toCharArray();
 		int mode = COM.STGM_TRANSACTED | COM.STGM_READWRITE | COM.STGM_SHARE_EXCLUSIVE | COM.STGM_CREATE;
 		int result = COM.StgCreateDocfile(path, mode, 0, address); //Does an AddRef if successful
@@ -1377,7 +1359,7 @@ private boolean saveToTraditionalFile(File file) {
 	if (!updateStorage())
 		return false;
 
-	long /*int*/[] address = new long /*int*/[1];
+	long[] address = new long[1];
 	// Look for a CONTENTS stream
 	if (tempStorage.OpenStream("CONTENTS", 0, COM.STGM_DIRECT | COM.STGM_READ | COM.STGM_SHARE_EXCLUSIVE, 0, address) == COM.S_OK) //$NON-NLS-1$
 		return saveFromContents(address[0], file);
@@ -1388,10 +1370,7 @@ private boolean saveToTraditionalFile(File file) {
 
 	return false;
 }
-private int Scroll_64(long /*int*/ scrollExtent) {
-	return COM.S_OK;
-}
-private int Scroll(int scrollExtent_cx, int scrollExtent_cy) {
+private int Scroll(long scrollExtent) {
 	return COM.S_OK;
 }
 void setBorderSpace(RECT newBorderwidth) {
@@ -1402,7 +1381,7 @@ void setBorderSpace(RECT newBorderwidth) {
 void setBounds() {
 	Rectangle area = DPIUtil.autoScaleUp(frame.getClientArea()); // To Pixels
 	setBounds(DPIUtil.autoScaleDown(borderWidths.left),
-		      DPIUtil.autoScaleDown(borderWidths.top),
+			  DPIUtil.autoScaleDown(borderWidths.top),
 			  DPIUtil.autoScaleDown(area.width - borderWidths.left - borderWidths.right),
 			  DPIUtil.autoScaleDown(area.height - borderWidths.top - borderWidths.bottom));
 	setObjectRects();
@@ -1419,7 +1398,7 @@ private void setExtent(int width, int height){
 	newExtent.cx = width; newExtent.cy = height;
 	newExtent = xFormPixelsToHimetric(newExtent);
 
-   // Get the server running first, then do a SetExtent, then show it
+	// Get the server running first, then do a SetExtent, then show it
 	boolean alreadyRunning = COM.OleIsRunning(objIOleObject.getAddress());
 	if (!alreadyRunning)
 		COM.OleRun(objIOleObject.getAddress());
@@ -1470,7 +1449,7 @@ private int ShowObject() {
 public void showProperties(String title) {
 
 	// Get the Property Page information from the OLE Object
-	long /*int*/[] ppvObject = new long /*int*/[1];
+	long[] ppvObject = new long[1];
 	if (objIUnknown.QueryInterface(COM.IIDISpecifyPropertyPages, ppvObject) != COM.S_OK) return;
 	ISpecifyPropertyPages objISPP = new ISpecifyPropertyPages(ppvObject[0]);
 	CAUUID caGUID = new CAUUID();
@@ -1484,16 +1463,16 @@ public void showProperties(String title) {
 		chTitle = new char[title.length()];
 		title.getChars(0, title.length(), chTitle, 0);
 	}
-	result = COM.OleCreatePropertyFrame(frame.handle, 10, 10, chTitle, 1, new long /*int*/[] {objIUnknown.getAddress()}, caGUID.cElems, caGUID.pElems, COM.LOCALE_USER_DEFAULT, 0, 0);
+	result = COM.OleCreatePropertyFrame(frame.handle, 10, 10, chTitle, 1, new long[] {objIUnknown.getAddress()}, caGUID.cElems, caGUID.pElems, COM.LOCALE_USER_DEFAULT, 0, 0);
 
 	// free the property page information
-	COM.CoTaskMemFree(caGUID.pElems);
+	OS.CoTaskMemFree(caGUID.pElems);
 }
 private boolean updateStorage() {
 
 	if (tempStorage == null) return false;
 
-	long /*int*/[] ppv = new long /*int*/[1];
+	long[] ppv = new long[1];
 	if (objIUnknown.QueryInterface(COM.IIDIPersistStorage, ppv) != COM.S_OK) return false;
 	IPersistStorage iPersistStorage = new IPersistStorage(ppv[0]);
 
@@ -1515,7 +1494,7 @@ private SIZE xFormHimetricToPixels(SIZE aSize) {
 	// Return a new Size which is the pixel transformation of a
 	// size in HIMETRIC units.
 
-	long /*int*/ hDC = OS.GetDC(0);
+	long hDC = OS.GetDC(0);
 	int xppi = OS.GetDeviceCaps(hDC, 88); // logical pixels/inch in x
 	int yppi = OS.GetDeviceCaps(hDC, 90); // logical pixels/inch in y
 	OS.ReleaseDC(0, hDC);
@@ -1530,7 +1509,7 @@ private SIZE xFormPixelsToHimetric(SIZE aSize) {
 	// Return a new size which is the HIMETRIC transformation of a
 	// size in pixel units.
 
-	long /*int*/ hDC = OS.GetDC(0);
+	long hDC = OS.GetDC(0);
 	int xppi = OS.GetDeviceCaps(hDC, 88); // logical pixels/inch in x
 	int yppi = OS.GetDeviceCaps(hDC, 90); // logical pixels/inch in y
 	OS.ReleaseDC(0, hDC);
