@@ -94,7 +94,7 @@ public ColorDialog(Shell parent, int style) {
 	checkSubclass ();
 }
 
-void changeColor(long /*int*/ id, long /*int*/ sel, long /*int*/ sender) {
+void changeColor(long id, long sel, long sender) {
 	selected = true;
 }
 
@@ -153,7 +153,7 @@ public RGB open() {
 	}
 	if (rgbs != null) {
 		NSArray keys = colorList.allKeys();
-		int length = (int)/*64*/keys.count();
+		int length = (int)keys.count();
 		for (int i=length-1; i>=0; i--) {
 			colorList.removeColorWithKey(new NSString(keys.objectAtIndex(i)));
 		}
@@ -170,7 +170,7 @@ public RGB open() {
 		}
 	}
 	SWTPanelDelegate delegate = (SWTPanelDelegate)new SWTPanelDelegate().alloc().init();
-	long /*int*/ jniRef = OS.NewGlobalRef(this);
+	long jniRef = OS.NewGlobalRef(this);
 	if (jniRef == 0) error(SWT.ERROR_NO_HANDLES);
 	OS.object_setInstanceVariable(delegate.id, Display.SWT_OBJECT, jniRef);
 	panel.setDelegate(delegate);
@@ -186,18 +186,18 @@ public RGB open() {
 	if (selected) {
 		NSColor color = panel.color();
 		if (color != null) {
-			double /*float*/ [] handle = display.getNSColorRGB(color);
+			double [] handle = display.getNSColorRGB(color);
 			if (handle != null) {
 				rgb = new RGB((int)(handle[0] * 255), (int)(handle[1] * 255), (int)(handle[2] * 255));
 			}
 		}
 	}
 	NSArray keys = colorList.allKeys();
-	int length = (int)/*64*/keys.count();
+	int length = (int)keys.count();
 	rgbs = new RGB[length];
 	for (int i=0; i<length; i++) {
 		NSString key = new NSString(keys.objectAtIndex(i));
-		double /*float*/ [] handle = display.getNSColorRGB(colorList.colorWithKey(key));
+		double [] handle = display.getNSColorRGB(colorList.colorWithKey(key));
 		if (handle != null) {
 			rgbs[i] = new RGB((int)(handle[0] * 255), (int)(handle[1] * 255), (int)(handle[2] * 255));
 		}
@@ -235,7 +235,7 @@ public void setRGBs(RGB[] rgbs) {
 	this.rgbs = rgbs;
 }
 
-void windowWillClose(long /*int*/ id, long /*int*/ sel, long /*int*/ sender) {
+void windowWillClose(long id, long sel, long sender) {
 	NSApplication.sharedApplication().stop(null);
 }
 }
