@@ -356,6 +356,15 @@ Rectangle [] computeProportions (Rectangle [] rects) {
 
 void drawRectangles (NSWindow window, Rectangle [] rects, boolean erase) {
 	NSGraphicsContext context = window.graphicsContext();
+	if (context == null) {
+		long width = (long) window.frame().width;
+		long height = (long) window.frame().height;
+		NSBitmapImageRep rep = (NSBitmapImageRep) new NSBitmapImageRep().alloc();
+		rep = rep.initWithBitmapDataPlanes(0, width, height, 8, 3, false, false, OS.NSDeviceRGBColorSpace,
+				OS.NSAlphaFirstBitmapFormat, width * 4, 32);
+		context = NSGraphicsContext.graphicsContextWithBitmapImageRep(rep);
+		rep.release();
+	}
 	NSGraphicsContext.static_saveGraphicsState();
 	NSGraphicsContext.setCurrentContext(context);
 	context.saveGraphicsState();
@@ -792,6 +801,15 @@ public boolean open () {
 		window.setContentView(null);
 		window.setBackgroundColor(NSColor.clearColor());
 		NSGraphicsContext context = window.graphicsContext();
+		if (context == null) {
+			long width = (long) window.frame().width;
+			long height = (long) window.frame().height;
+			NSBitmapImageRep rep = (NSBitmapImageRep) new NSBitmapImageRep().alloc();
+			rep = rep.initWithBitmapDataPlanes(0, width, height, 8, 3, false, false, OS.NSDeviceRGBColorSpace,
+					OS.NSAlphaFirstBitmapFormat, width * 4, 32);
+			context = NSGraphicsContext.graphicsContextWithBitmapImageRep(rep);
+			rep.release();
+		}
 		NSGraphicsContext.static_saveGraphicsState();
 		NSGraphicsContext.setCurrentContext(context);
 		context.setCompositingOperation(OS.NSCompositeClear);
