@@ -60,32 +60,6 @@ public final class Cursor extends Resource {
 	 */
 	public long handle;
 
-	static final byte[] APPSTARTING_SRC = {
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00,
-		0x0c, 0x00, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0x00,
-		0x7c, 0x00, 0x00, 0x00, (byte)0xfc, 0x00, 0x00, 0x00, (byte)0xfc, 0x01, 0x00, 0x00,
-		(byte)0xfc, 0x3b, 0x00, 0x00, 0x7c, 0x38, 0x00, 0x00, 0x6c, 0x54, 0x00, 0x00,
-		(byte)0xc4, (byte)0xdc, 0x00, 0x00, (byte)0xc0, 0x44, 0x00, 0x00, (byte)0x80, 0x39, 0x00, 0x00,
-		(byte)0x80, 0x39, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
-
-	static final byte[] APPSTARTING_MASK = {
-		0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x0e, 0x00, 0x00, 0x00,
-		0x1e, 0x00, 0x00, 0x00, 0x3e, 0x00, 0x00, 0x00, 0x7e, 0x00, 0x00, 0x00,
-		(byte)0xfe, 0x00, 0x00, 0x00, (byte)0xfe, 0x01, 0x00, 0x00, (byte)0xfe, 0x3b, 0x00, 0x00,
-		(byte)0xfe, 0x7f, 0x00, 0x00, (byte)0xfe, 0x7f, 0x00, 0x00, (byte)0xfe, (byte)0xfe, 0x00, 0x00,
-		(byte)0xee, (byte)0xff, 0x01, 0x00, (byte)0xe4, (byte)0xff, 0x00, 0x00, (byte)0xc0, 0x7f, 0x00, 0x00,
-		(byte)0xc0, 0x7f, 0x00, 0x00, (byte)0x80, 0x39, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
-
 Cursor (Device device) {
 	super(device);
 }
@@ -138,81 +112,41 @@ Cursor (Device device) {
  */
 public Cursor(Device device, int style) {
 	super(device);
-	int shape = 0;
-	byte[] name = null;
-	if (GTK.GTK4) {
-		switch (style) {
-			case SWT.CURSOR_APPSTARTING:	name = Converter.wcsToMbcs("progress", true) ; break;
-			case SWT.CURSOR_ARROW:			name = Converter.wcsToMbcs("default", true) ; break;
-			case SWT.CURSOR_WAIT:			name = Converter.wcsToMbcs("wait", true) ; break;
-			case SWT.CURSOR_CROSS:			name = Converter.wcsToMbcs("cell", true) ; break;
-			case SWT.CURSOR_HAND:			name = Converter.wcsToMbcs("pointer", true) ; break;
-			case SWT.CURSOR_HELP:			name = Converter.wcsToMbcs("help", true) ; break;
-			case SWT.CURSOR_SIZEALL:		name = Converter.wcsToMbcs("all-scroll", true) ; break;
-			case SWT.CURSOR_SIZENESW:		name = Converter.wcsToMbcs("nesw-resize", true); break;
-			case SWT.CURSOR_SIZENS:			name = Converter.wcsToMbcs("row-resize", true) ; break;
-			case SWT.CURSOR_SIZENWSE:		name = Converter.wcsToMbcs("nwse-resize", true); break;
-			case SWT.CURSOR_SIZEWE:			name = Converter.wcsToMbcs("col-resize", true); break;
-			case SWT.CURSOR_SIZEN:			name = Converter.wcsToMbcs("n-resize", true); break;
-			case SWT.CURSOR_SIZES:			name = Converter.wcsToMbcs("s-resize", true); break;
-			case SWT.CURSOR_SIZEE:			name = Converter.wcsToMbcs("e-resize", true); break;
-			case SWT.CURSOR_SIZEW:			name = Converter.wcsToMbcs("w-resize", true); break;
-			case SWT.CURSOR_SIZENE:			name = Converter.wcsToMbcs("ne-resize", true); break;
-			case SWT.CURSOR_SIZESE:			name = Converter.wcsToMbcs("se-resize", true); break;
-			case SWT.CURSOR_SIZESW:			name = Converter.wcsToMbcs("sw-resize", true); break;
-			case SWT.CURSOR_SIZENW:			name = Converter.wcsToMbcs("nw-resize", true); break;
-			case SWT.CURSOR_UPARROW:		name = Converter.wcsToMbcs("ns-resize", true); break;
-			case SWT.CURSOR_IBEAM:			name = Converter.wcsToMbcs("text", true); break;
-			case SWT.CURSOR_NO:				name = Converter.wcsToMbcs("not-allowed", true) ; break;
-			default:
-				SWT.error(SWT.ERROR_INVALID_ARGUMENT);
-		}
-	} else {
-		switch (style) {
-			case SWT.CURSOR_APPSTARTING:	name = Converter.wcsToMbcs("progress", true) ; break;
-			case SWT.CURSOR_ARROW:			shape = GDK.GDK_LEFT_PTR; break;
-			case SWT.CURSOR_WAIT:			shape = GDK.GDK_WATCH; break;
-			case SWT.CURSOR_CROSS:			shape = GDK.GDK_CROSS; break;
-			case SWT.CURSOR_HAND:			shape = GDK.GDK_HAND2; break;
-			case SWT.CURSOR_HELP:			shape = GDK.GDK_QUESTION_ARROW; break;
-			case SWT.CURSOR_SIZEALL:		shape = GDK.GDK_FLEUR; break;
-			case SWT.CURSOR_SIZENESW:		shape = GDK.GDK_SIZING; name=Converter.wcsToMbcs("nesw-resize", true); break;
-			case SWT.CURSOR_SIZENS:			shape = GDK.GDK_DOUBLE_ARROW; break;
-			case SWT.CURSOR_SIZENWSE:		shape = GDK.GDK_SIZING; name=Converter.wcsToMbcs("nwse-resize", true); break;
-			case SWT.CURSOR_SIZEWE:			shape = GDK.GDK_SB_H_DOUBLE_ARROW; break;
-			case SWT.CURSOR_SIZEN:			shape = GDK.GDK_TOP_SIDE; break;
-			case SWT.CURSOR_SIZES:			shape = GDK.GDK_BOTTOM_SIDE; break;
-			case SWT.CURSOR_SIZEE:			shape = GDK.GDK_RIGHT_SIDE; break;
-			case SWT.CURSOR_SIZEW:			shape = GDK.GDK_LEFT_SIDE; break;
-			case SWT.CURSOR_SIZENE:			shape = GDK.GDK_TOP_RIGHT_CORNER; break;
-			case SWT.CURSOR_SIZESE:			shape = GDK.GDK_BOTTOM_RIGHT_CORNER; break;
-			case SWT.CURSOR_SIZESW:			shape = GDK.GDK_BOTTOM_LEFT_CORNER; break;
-			case SWT.CURSOR_SIZENW:			shape = GDK.GDK_TOP_LEFT_CORNER; break;
-			case SWT.CURSOR_UPARROW:		shape = GDK.GDK_SB_UP_ARROW; break;
-			case SWT.CURSOR_IBEAM:			shape = GDK.GDK_XTERM; break;
-			case SWT.CURSOR_NO:				shape = GDK.GDK_X_CURSOR; name=Converter.wcsToMbcs("not-allowed", true) ; break;
-			default:
-				SWT.error(SWT.ERROR_INVALID_ARGUMENT);
-		}
+	String name1 = null, name2 = null;
+	switch (style) {
+		case SWT.CURSOR_APPSTARTING:	name1 = "left_ptr_watch"; break;
+		case SWT.CURSOR_ARROW:			name1 = "left_ptr"; break;
+		case SWT.CURSOR_WAIT:			name1 = "watch"; break;
+		case SWT.CURSOR_CROSS:			name1 = "crosshair"; break;
+		case SWT.CURSOR_HAND:			name1 = "hand2"; break;
+		case SWT.CURSOR_HELP:			name1 = "question_arrow"; break;
+		case SWT.CURSOR_SIZEALL:		name1 = "fleur"; break;
+		case SWT.CURSOR_SIZENESW:		name1 = "size_bdiag"; break;
+		case SWT.CURSOR_SIZENS:			name1 = "sb_v_double_arrow"; break;
+		case SWT.CURSOR_SIZENWSE:		name1 = "size_fdiag"; break;
+		case SWT.CURSOR_SIZEWE:			name1 = "sb_h_double_arrow"; break;
+		case SWT.CURSOR_SIZEN:			name1 = "top_side"; break;
+		case SWT.CURSOR_SIZES:			name1 = "bottom_side"; break;
+		case SWT.CURSOR_SIZEE:			name1 = "right_side"; break;
+		case SWT.CURSOR_SIZEW:			name1 = "left_side"; break;
+		case SWT.CURSOR_SIZENE:			name1 = "top_right_corner"; break;
+		case SWT.CURSOR_SIZESE:			name1 = "bottom_right_corner"; break;
+		case SWT.CURSOR_SIZESW:			name1 = "bottom_left_corner"; break;
+		case SWT.CURSOR_SIZENW:			name1 = "top_left_corner"; break;
+		case SWT.CURSOR_UPARROW:		name1 = "sb_up_arrow"; name2 = "up-arrow"; break;
+		case SWT.CURSOR_IBEAM:			name1 = "xterm"; break;
+		case SWT.CURSOR_NO:				name1 = "crossed_circle"; name2 = "not-allowed"; break;
+		default:
+			SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	}
-	if (name != null) {
+	for (String name : new String[] { name1, name2, "left_ptr", "default" }) {
+		if (name == null) continue;
 		if (GTK.GTK4) {
-			handle = GDK.gdk_cursor_new_from_name(name, 0);
+			handle = GDK.gdk_cursor_new_from_name (name, 0);
 		} else {
 			handle = GDK.gdk_cursor_new_from_name (GDK.gdk_display_get_default(), name);
 		}
-	}
-	if (handle == 0) {
-		if (shape == 0 && style == SWT.CURSOR_APPSTARTING) {
-			byte[] src = new byte[APPSTARTING_SRC.length];
-			System.arraycopy(APPSTARTING_SRC, 0, src, 0, src.length);
-			byte[] mask = new byte[APPSTARTING_MASK.length];
-			System.arraycopy(APPSTARTING_MASK, 0, mask, 0, mask.length);
-			handle = createCursor(src, mask, 32, 32, 2, 2, true);
-		} else {
-			if (!GTK.GTK4) handle = GDK.gdk_cursor_new_for_display(GDK.gdk_display_get_default(), shape);
-		}
-
+		if (handle != 0) break;
 	}
 	if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	init();
@@ -343,8 +277,10 @@ public Cursor(Device device, ImageData source, int hotspotX, int hotspotY) {
 		hotspotY >= source.height || hotspotY < 0) {
 		SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	}
-	long display = 0;
-	if (GDK.gdk_display_supports_cursor_color(display = GDK.gdk_display_get_default ())) {
+
+	long display = GDK.gdk_display_get_default();
+	boolean supportsColorCursor = GTK.GTK4 || GDK.gdk_display_supports_cursor_color(display);
+	if (supportsColorCursor) {
 		int width = source.width;
 		int height = source.height;
 		PaletteData palette = source.palette;
@@ -409,7 +345,14 @@ public Cursor(Device device, ImageData source, int hotspotX, int hotspotY) {
 			}
 		}
 		C.memmove(data, buffer, stride * height);
-		handle = GDK.gdk_cursor_new_from_pixbuf(display, pixbuf, hotspotX, hotspotY);
+
+		if (GTK.GTK4) {
+			long texture = GDK.gdk_texture_new_for_pixbuf(pixbuf);
+			handle = GDK.gdk_cursor_new_from_texture(texture, hotspotX, hotspotY, 0);
+			OS.g_object_unref(texture);
+		} else {
+			handle = GDK.gdk_cursor_new_from_pixbuf(display, pixbuf, hotspotX, hotspotY);
+		}
 		OS.g_object_unref(pixbuf);
 	} else {
 
@@ -543,14 +486,17 @@ long createCursor(byte[] sourceData, byte[] maskData, int width, int height, int
 	int stride = GDK.gdk_pixbuf_get_rowstride(pixbuf);
 	long pixels = GDK.gdk_pixbuf_get_pixels(pixbuf);
 	C.memmove(pixels, data, stride * height);
+
 	long cursor;
 	if (GTK.GTK4) {
-		long texture = GDK.gdk_texture_new_for_pixbuf (pixbuf);
-		cursor = GDK.gdk_cursor_new_from_texture (texture, hotspotX, hotspotY, 0);
+		long texture = GDK.gdk_texture_new_for_pixbuf(pixbuf);
+		cursor = GDK.gdk_cursor_new_from_texture(texture, hotspotX, hotspotY, 0);
+		OS.g_object_unref(texture);
 	} else {
 		cursor = GDK.gdk_cursor_new_from_pixbuf(GDK.gdk_display_get_default(), pixbuf, hotspotX, hotspotY);
 	}
 	OS.g_object_unref(pixbuf);
+
 	return cursor;
 }
 
