@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2018 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -19,6 +19,8 @@ import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.gtk3.*;
+import org.eclipse.swt.internal.gtk4.*;
 
 /**
  * Instances of this class represent a selectable user interface object that
@@ -50,7 +52,8 @@ import org.eclipse.swt.internal.gtk.*;
 public class Button extends Control {
 	long boxHandle, labelHandle, imageHandle, arrowHandle, groupHandle;
 	boolean selected, grayed;
-	ImageList imageList;
+	/** True iff this toggle button requires special theme handling. See bug 546552.*/
+	boolean toggleButtonTheming;
 	Image image;
 	String text;
 	GdkRGBA background;
@@ -124,7 +127,7 @@ GtkBorder getBorder (byte[] border, long handle, int defaultBorder) {
 		gtk_style_context_get_border(context, stateFlag, gtkBorder);
 		return gtkBorder;
 	} else {
-		GTK.gtk_widget_style_get (handle, border, borderPtr,0);
+		GTK3.gtk_widget_style_get (handle, border, borderPtr,0);
 		if (borderPtr[0] != 0) {
 			OS.memmove (gtkBorder, borderPtr[0], GtkBorder.sizeof);
 			GTK.gtk_border_free(borderPtr[0]);
@@ -210,9 +213,9 @@ Point computeSizeInPixels (int wHint, int hHint, boolean changed) {
 		int trimWidth, trimHeight;
 		if (!GTK.GTK4) {
 			int[] focusWidth = new int[1];
-			GTK.gtk_widget_style_get (handle, OS.focus_line_width, focusWidth, 0);
+			GTK3.gtk_widget_style_get (handle, OS.focus_line_width, focusWidth, 0);
 			int[] focusPadding = new int[1];
-			GTK.gtk_widget_style_get (handle, OS.focus_padding, focusPadding, 0);
+			GTK3.gtk_widget_style_get (handle, OS.focus_padding, focusPadding, 0);
 			trimWidth = 2 * (borderWidth + focusWidth [0] + focusPadding [0]);
 		} else {
 			trimWidth = 2 * borderWidth;
@@ -221,7 +224,7 @@ Point computeSizeInPixels (int wHint, int hHint, boolean changed) {
 		int indicatorHeight = 0;
 		if ((style & (SWT.CHECK | SWT.RADIO)) != 0) {
 			if (GTK.GTK4) {
-				long icon = GTK.gtk_widget_get_first_child(handle);
+				long icon = GTK4.gtk_widget_get_first_child(handle);
 				GtkRequisition minimum = new GtkRequisition ();
 				GTK.gtk_widget_get_preferred_size(icon, minimum, null);
 				long context = GTK.gtk_widget_get_style_context(icon);
@@ -231,8 +234,8 @@ Point computeSizeInPixels (int wHint, int hHint, boolean changed) {
 			} else {
 				int[] indicatorSize = new int[1];
 				int[] indicatorSpacing = new int[1];
-				GTK.gtk_widget_style_get (handle, OS.indicator_size, indicatorSize, 0);
-				GTK.gtk_widget_style_get (handle, OS.indicator_spacing, indicatorSpacing, 0);
+				GTK3.gtk_widget_style_get (handle, OS.indicator_size, indicatorSize, 0);
+				GTK3.gtk_widget_style_get (handle, OS.indicator_spacing, indicatorSpacing, 0);
 				indicatorHeight = indicatorSize [0] + 2 * indicatorSpacing [0];
 				trimWidth += indicatorHeight + indicatorSpacing [0];
 			}
@@ -243,8 +246,9 @@ Point computeSizeInPixels (int wHint, int hHint, boolean changed) {
 			GtkBorder innerBorder = getBorder (OS.inner_border, handle, INNER_BORDER);
 			trimWidth += innerBorder.left + innerBorder.right;
 			trimHeight += innerBorder.top + innerBorder.bottom;
-			if (GTK.gtk_widget_get_can_default (handle)) {
-				GtkBorder defaultBorder = getBorder (OS.default_border, handle, DEFAULT_BORDER);
+			boolean canDefault = GTK.GTK4 ? GTK4.gtk_widget_get_receives_default(handle) : GTK3.gtk_widget_get_can_default(handle);
+			if (canDefault) {
+				GtkBorder defaultBorder = getBorder(OS.default_border, handle, DEFAULT_BORDER);
 				trimWidth += defaultBorder.left + defaultBorder.right;
 				trimHeight += defaultBorder.top + defaultBorder.bottom;
 			}
@@ -279,8 +283,9 @@ Point computeSizeInPixels (int wHint, int hHint, boolean changed) {
 		GTK.gtk_widget_set_size_request (boxHandle, reqWidth [0], reqHeight [0]);
 	}
 	if (wHint != SWT.DEFAULT || hHint != SWT.DEFAULT) {
-		if (GTK.gtk_widget_get_can_default (handle)) {
-			GtkBorder border = getBorder (OS.default_border, handle, DEFAULT_BORDER);
+		boolean canDefault = GTK.GTK4 ? GTK4.gtk_widget_get_receives_default(handle) : GTK3.gtk_widget_get_can_default(handle);
+		if (canDefault) {
+			GtkBorder border = getBorder(OS.default_border, handle, DEFAULT_BORDER);
 			if (wHint != SWT.DEFAULT) size.x += border.left + border.right;
 			if (hHint != SWT.DEFAULT) size.y += border.top + border.bottom;
 		}
@@ -293,9 +298,7 @@ void createHandle (int index) {
 	state |= HANDLE;
 	if ((style & (SWT.PUSH | SWT.TOGGLE)) == 0) state |= THEME_BACKGROUND;
 	int bits = SWT.ARROW | SWT.TOGGLE | SWT.CHECK | SWT.RADIO | SWT.PUSH;
-	fixedHandle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
-	if (fixedHandle == 0) error (SWT.ERROR_NO_HANDLES);
-	gtk_widget_set_has_surface_or_window (fixedHandle, true);
+
 	switch (style & bits) {
 		case SWT.ARROW:
 			byte arrowType [] = GTK.GTK_NAMED_ICON_GO_UP;
@@ -303,19 +306,38 @@ void createHandle (int index) {
 			if ((style & SWT.DOWN) != 0) arrowType = GTK.GTK_NAMED_ICON_GO_DOWN;
 			if ((style & SWT.LEFT) != 0) arrowType = GTK.GTK_NAMED_ICON_GO_PREVIOUS;
 			if ((style & SWT.RIGHT) != 0) arrowType = GTK.GTK_NAMED_ICON_GO_NEXT;
-			arrowHandle = GTK.gtk_image_new_from_icon_name (arrowType, GTK.GTK_ICON_SIZE_MENU);
-			if (arrowHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
-			handle = GTK.gtk_button_new ();
-			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+			if (GTK.GTK4) {
+				handle = GTK4.gtk_button_new_from_icon_name(arrowType);
+				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+				arrowHandle = GTK4.gtk_widget_get_first_child(handle);
+				if (arrowHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			} else {
+				arrowHandle = GTK3.gtk_image_new_from_icon_name(arrowType, GTK.GTK_ICON_SIZE_MENU);
+				if (arrowHandle == 0) error (SWT.ERROR_NO_HANDLES);
+				handle = GTK.gtk_button_new();
+				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+
+				// Use gtk_button_set_image() on GTK3 to prevent icons from being
+				// trimmed with smaller sized buttons; see bug 528284.
+				GTK3.gtk_button_set_image(handle, arrowHandle);
+			}
 			break;
 		case SWT.TOGGLE:
 			handle = GTK.gtk_toggle_button_new ();
 			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+			if (Display.themeName != null) {
+				toggleButtonTheming = (GTK.GTK_VERSION >= OS.VERSION(3, 24, 11) && Display.themeName.contains("Adwaita"))
+						|| Display.themeName.contains("Yaru");
+			}
 			break;
 		case SWT.CHECK:
 			handle = GTK.gtk_check_button_new ();
 			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+			if (Display.themeName != null) {
+				toggleButtonTheming = (GTK.GTK_VERSION >= OS.VERSION(3, 24, 11) && Display.themeName.contains("Adwaita"))
+						|| Display.themeName.contains("Yaru");
+			}
 			break;
 		case SWT.RADIO:
 			/*
@@ -329,39 +351,77 @@ void createHandle (int index) {
 			* to the same group.  This allows the visible button to be
 			* unselected.
 			*/
-			groupHandle = GTK.gtk_radio_button_new (0);
-			if (groupHandle == 0) error (SWT.ERROR_NO_HANDLES);
-			OS.g_object_ref_sink (groupHandle);
-			handle = GTK.gtk_radio_button_new (GTK.gtk_radio_button_get_group (groupHandle));
-			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+
+			if (GTK.GTK4) {
+				groupHandle = GTK.gtk_check_button_new();
+				if (groupHandle == 0) error(SWT.ERROR_NO_HANDLES);
+				OS.g_object_ref_sink(groupHandle);
+				handle = GTK.gtk_check_button_new();
+				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+				GTK4.gtk_check_button_set_group(handle, groupHandle);
+			} else {
+				groupHandle = GTK3.gtk_radio_button_new (0);
+				if (groupHandle == 0) error (SWT.ERROR_NO_HANDLES);
+				OS.g_object_ref_sink (groupHandle);
+				handle = GTK3.gtk_radio_button_new (GTK3.gtk_radio_button_get_group (groupHandle));
+				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+			}
+
+			if (Display.themeName != null) {
+				toggleButtonTheming = (GTK.GTK_VERSION >= OS.VERSION(3, 24, 11) && Display.themeName.contains("Adwaita"))
+						|| Display.themeName.contains("Yaru");
+			}
 			break;
 		case SWT.PUSH:
 		default:
 			handle = GTK.gtk_button_new ();
 			if (handle == 0) error (SWT.ERROR_NO_HANDLES);
-			GTK.gtk_widget_set_can_default (handle, true);
+			if (!GTK.GTK4) GTK3.gtk_widget_set_can_default (handle, true);
 			break;
 	}
-	if ((style & SWT.ARROW) != 0) {
-		// Use gtk_button_set_image() on GTK3 to prevent icons from being
-		// trimmed with smaller sized buttons; see bug 528284.
-		GTK.gtk_button_set_image(handle, arrowHandle);
-	} else {
-		boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 4);
-		if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
-		labelHandle = GTK.gtk_label_new_with_mnemonic (null);
-		if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
-		imageHandle = GTK.gtk_image_new ();
-		if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
-		GTK.gtk_container_add (handle, boxHandle);
-		GTK.gtk_container_add (boxHandle, imageHandle);
-		GTK.gtk_container_add (boxHandle, labelHandle);
+
+	if ((style & SWT.ARROW) == 0) {
+		boxHandle = gtk_box_new(GTK.GTK_ORIENTATION_HORIZONTAL, false, 4);
+		if (boxHandle == 0) error(SWT.ERROR_NO_HANDLES);
+		labelHandle = GTK.gtk_label_new_with_mnemonic(null);
+		if (labelHandle == 0) error(SWT.ERROR_NO_HANDLES);
+
+
+		if (GTK.GTK4) {
+			imageHandle = GTK4.gtk_picture_new();
+			if (imageHandle == 0) error(SWT.ERROR_NO_HANDLES);
+
+			GTK.gtk_widget_set_parent(boxHandle, handle);
+			GTK4.gtk_box_append(boxHandle, imageHandle);
+			GTK4.gtk_box_append(boxHandle, labelHandle);
+		} else {
+			imageHandle = GTK.gtk_image_new();
+			if (imageHandle == 0) error(SWT.ERROR_NO_HANDLES);
+
+			GTK3.gtk_container_add(handle, boxHandle);
+			GTK3.gtk_container_add(boxHandle, imageHandle);
+			GTK3.gtk_container_add(boxHandle, labelHandle);
+		}
+
 		if ((style & SWT.WRAP) != 0) {
-			GTK.gtk_label_set_line_wrap (labelHandle, true);
-			GTK.gtk_label_set_line_wrap_mode (labelHandle, OS.PANGO_WRAP_WORD_CHAR);
+			if (GTK.GTK4) {
+				GTK.gtk_label_set_wrap(labelHandle, true);
+				GTK.gtk_label_set_wrap_mode(labelHandle, OS.PANGO_WRAP_WORD_CHAR);
+			} else {
+				GTK.gtk_label_set_line_wrap (labelHandle, true);
+				GTK.gtk_label_set_line_wrap_mode (labelHandle, OS.PANGO_WRAP_WORD_CHAR);
+			}
 		}
 	}
-	GTK.gtk_container_add (fixedHandle, handle);
+
+	fixedHandle = OS.g_object_new(display.gtk_fixed_get_type(), 0);
+	if (fixedHandle == 0) error(SWT.ERROR_NO_HANDLES);
+	if (GTK.GTK4) {
+		OS.swt_fixed_add(fixedHandle, handle);
+	} else {
+		GTK3.gtk_widget_set_has_window(fixedHandle, true);
+		GTK3.gtk_container_add(fixedHandle, handle);
+	}
 
 	if ((style & SWT.ARROW) != 0) return;
 	// In GTK 3 font description is inherited from parent widget which is not how SWT has always worked,
@@ -376,6 +436,26 @@ void createWidget (int index) {
 	text = "";
 }
 
+@Override
+void checkBackground() {
+	/*
+	 * Feature in GTK: some toggle style (check, radio, and toggle) buttons
+	 * have inverted colors, meaning the background color when checked is
+	 * a dark color (like blue or green), and the checkmark/indicator is
+	 * white. To complicate matters, this background area is an image, and
+	 * overriding this with a color causes the checkmark to be invisible.
+	 * The new (GTK3 >= 3.24.11) Adwaita theme is affected, as well as the
+	 * default Yaru theme on Ubuntu.
+	 *
+	 * Part of the fix is to not inherit the parents background. See bug 553657.
+	 */
+	if (toggleButtonTheming && (style & (SWT.CHECK | SWT.RADIO)) != 0) {
+		state &= ~PARENT_BACKGROUND;
+	} else {
+		super.checkBackground();
+	}
+}
+
 @Override
 void deregister () {
 	super.deregister ();
@@ -482,7 +562,12 @@ String getNameText () {
 public boolean getSelection () {
 	checkWidget ();
 	if ((style & (SWT.CHECK | SWT.RADIO | SWT.TOGGLE)) == 0) return false;
-	return GTK.gtk_toggle_button_get_active (handle);
+
+	if (GTK.GTK4 && (style & (SWT.CHECK | SWT.RADIO)) != 0) {
+		return GTK4.gtk_check_button_get_active(handle);
+	} else {
+		return GTK.gtk_toggle_button_get_active(handle);
+	}
 }
 
 /**
@@ -522,12 +607,12 @@ long gtk_clicked (long widget) {
 		}
 	} else {
 		if ((style & SWT.CHECK) != 0) {
-			if (grayed) {
-				if (GTK.gtk_toggle_button_get_active (handle)) {
-					GTK.gtk_toggle_button_set_inconsistent (handle, true);
-				} else {
-					GTK.gtk_toggle_button_set_inconsistent (handle, false);
-				}
+			if (GTK.GTK4) {
+				boolean inconsistent = grayed && GTK4.gtk_check_button_get_active(handle);
+				GTK4.gtk_check_button_set_inconsistent(handle, inconsistent);
+			} else {
+				boolean inconsistent = grayed && GTK.gtk_toggle_button_get_active(handle);
+				GTK.gtk_toggle_button_set_inconsistent(handle, inconsistent);
 			}
 		}
 	}
@@ -543,7 +628,7 @@ long gtk_draw (long widget, long cairo) {
 	 * The fix is to make the widget invisible to the user. Resizing widget later on to a larger size
 	 * makes the widget visible again in setBounds. See Bug 533469, Bug 531120.
 	 */
-	if (GTK.GTK_VERSION >= OS.VERSION (3, 20, 0) && (state & ZERO_WIDTH) != 0 && (state & ZERO_HEIGHT) != 0) {
+	if ((state & ZERO_WIDTH) != 0 && (state & ZERO_HEIGHT) != 0) {
 		if (GTK.gtk_widget_get_visible(widget)) GTK.gtk_widget_set_visible(widget, false);
 		// Button and display should not be disposed after hiding widget
 		if (isDisposed() || display == null || display.isDisposed()) error (SWT.ERROR_DEVICE_DISPOSED);
@@ -594,7 +679,13 @@ long gtk_key_press_event (long widget, long event) {
 @Override
 void hookEvents () {
 	super.hookEvents();
-	OS.g_signal_connect_closure (handle, OS.clicked, display.getClosure (CLICKED), false);
+
+	if (GTK.GTK4 && ((style & (SWT.RADIO | SWT.CHECK)) != 0)) {
+		OS.g_signal_connect_closure (handle, OS.toggled, display.getClosure (CLICKED), false);
+	} else {
+		OS.g_signal_connect_closure (handle, OS.clicked, display.getClosure (CLICKED), false);
+	}
+
 	if (labelHandle != 0) {
 		OS.g_signal_connect_closure_by_id (labelHandle, display.signalIds [MNEMONIC_ACTIVATE], 0, display.getClosure (MNEMONIC_ACTIVATE), false);
 	}
@@ -635,12 +726,17 @@ void releaseHandle () {
 }
 
 @Override
-void releaseWidget () {
-	super.releaseWidget ();
-	if (groupHandle != 0) OS.g_object_unref (groupHandle);
+void releaseWidget() {
+	super.releaseWidget();
+
+	if (GTK.GTK4) {
+		if (boxHandle != 0) GTK.gtk_widget_unparent(boxHandle);
+	}
+
+	// Release reference to hidden GtkCheckButton that allows for SWT.RADIO behavior
+	if (groupHandle != 0) OS.g_object_unref(groupHandle);
 	groupHandle = 0;
-	if (imageList != null) imageList.dispose ();
-	imageList = null;
+
 	image = null;
 	text = null;
 }
@@ -738,7 +834,7 @@ void _setAlignment (int alignment) {
 			case SWT.LEFT: arrowType = isRTL ? GTK.GTK_NAMED_ICON_GO_NEXT : GTK.GTK_NAMED_ICON_GO_PREVIOUS; break;
 			case SWT.RIGHT: arrowType = isRTL ? GTK.GTK_NAMED_ICON_GO_PREVIOUS : GTK.GTK_NAMED_ICON_GO_NEXT; break;
 		}
-		GTK.gtk_image_set_from_icon_name (arrowHandle, arrowType, GTK.GTK_ICON_SIZE_MENU);
+		GTK3.gtk_image_set_from_icon_name (arrowHandle, arrowType, GTK.GTK_ICON_SIZE_MENU);
 		return;
 	}
 	if ((alignment & (SWT.LEFT | SWT.RIGHT | SWT.CENTER)) == 0) return;
@@ -759,13 +855,8 @@ void _setAlignment (int alignment) {
 			gtk_box_set_child_packing (boxHandle, imageHandle, true, true, 0, GTK.GTK_PACK_START);
 		}
 
-		if (GTK.GTK_VERSION >= OS.VERSION(3, 16, 0)) {
-			gtk_label_set_align(labelHandle,0.0f,0.5f);
-			gtk_widget_set_align(imageHandle,GTK.GTK_ALIGN_START, GTK.GTK_ALIGN_CENTER);
-		} else {
-			GTK.gtk_misc_set_alignment (labelHandle, 0.0f, 0.5f);
-			GTK.gtk_misc_set_alignment (imageHandle, 0.0f, 0.5f);
-		}
+		gtk_label_set_align(labelHandle,0.0f,0.5f);
+		gtk_widget_set_align(imageHandle,GTK.GTK_ALIGN_START, GTK.GTK_ALIGN_CENTER);
 
 		GTK.gtk_label_set_justify (labelHandle, GTK.GTK_JUSTIFY_LEFT);
 		return;
@@ -775,25 +866,15 @@ void _setAlignment (int alignment) {
 			gtk_box_set_child_packing (boxHandle, labelHandle, true, true, 0, GTK.GTK_PACK_END);
 			gtk_box_set_child_packing (boxHandle, imageHandle, true, true, 0, GTK.GTK_PACK_START);
 
-			if (GTK.GTK_VERSION >= OS.VERSION(3, 16, 0))  {
-				gtk_label_set_align(labelHandle,0.0f,0.5f);
-				gtk_widget_set_align(imageHandle,GTK.GTK_ALIGN_END, GTK.GTK_ALIGN_CENTER);
-			} else  {
-				GTK.gtk_misc_set_alignment (labelHandle, 0f, 0.5f);
-				GTK.gtk_misc_set_alignment (imageHandle, 1f, 0.5f);
-			}
+			gtk_label_set_align(labelHandle,0.0f,0.5f);
+			gtk_widget_set_align(imageHandle,GTK.GTK_ALIGN_END, GTK.GTK_ALIGN_CENTER);
 
 		} else {
 			gtk_box_set_child_packing (boxHandle, labelHandle, true, true, 0, GTK.GTK_PACK_END);
 			gtk_box_set_child_packing (boxHandle, imageHandle, true, true, 0, GTK.GTK_PACK_START);
 
-			if (GTK.GTK_VERSION >= OS.VERSION(3, 16, 0))  {
-				gtk_label_set_align(labelHandle,0.5f,0.5f);
-				gtk_widget_set_align(imageHandle,GTK.GTK_ALIGN_CENTER, GTK.GTK_ALIGN_CENTER);
-			} else {
-				GTK.gtk_misc_set_alignment (labelHandle, 0.5f, 0.5f);
-				GTK.gtk_misc_set_alignment (imageHandle, 0.5f, 0.5f);
-			}
+			gtk_label_set_align(labelHandle,0.5f,0.5f);
+			gtk_widget_set_align(imageHandle,GTK.GTK_ALIGN_CENTER, GTK.GTK_ALIGN_CENTER);
 			GTK.gtk_label_set_justify (labelHandle, GTK.GTK_JUSTIFY_CENTER);
 		}
 		return;
@@ -807,13 +888,8 @@ void _setAlignment (int alignment) {
 			gtk_box_set_child_packing (boxHandle, imageHandle, true, true, 0, GTK.GTK_PACK_START);
 		}
 
-		if (GTK.GTK_VERSION >= OS.VERSION(3, 16, 0))  {
-			gtk_label_set_align(labelHandle,1.0f,0.5f);
-			gtk_widget_set_align(imageHandle,GTK.GTK_ALIGN_END, GTK.GTK_ALIGN_CENTER);
-		} else {
-			GTK.gtk_misc_set_alignment (labelHandle, 1.0f, 0.5f);
-			GTK.gtk_misc_set_alignment (imageHandle, 1.0f, 0.5f);
-		}
+		gtk_label_set_align(labelHandle,1.0f,0.5f);
+		gtk_widget_set_align(imageHandle,GTK.GTK_ALIGN_END, GTK.GTK_ALIGN_CENTER);
 		GTK.gtk_label_set_justify (labelHandle, GTK.GTK_JUSTIFY_RIGHT);
 		return;
 	}
@@ -827,6 +903,23 @@ void setBackgroundGdkRGBA (long context, long handle, GdkRGBA rgba) {
 	String color = display.gtk_rgba_to_css_string (rgba);
 	css += color + ";}";
 
+	/*
+	 * Feature in GTK: some toggle style (check, radio, and toggle) buttons
+	 * have inverted colors, meaning the background color when checked is
+	 * a dark color (like blue or green), and the checkmark/indicator is
+	 * white. To complicate matters, this background area is an image, and
+	 * overriding this with a color causes the checkmark to be invisible.
+	 * The new (GTK3 >= 3.24.11) Adwaita theme is affected, as well as the
+	 * default Yaru theme on Ubuntu.
+	 *
+	 *  The fix is to set the default CSS for the default background case, as
+	 *  this doesn't use the universal (*) selector. See bug 546552.
+	 */
+	if (rgba == null && toggleButtonTheming) {
+		super.setBackgroundGdkRGBA(context, handle, rgba);
+		return;
+	}
+
 	// Cache background color
 	cssBackground = css;
 
@@ -907,8 +1000,20 @@ void setFontDescription (long fontDesc) {
 		return;
 	} else {
 		super.setFontDescription (fontDesc);
-		if (labelHandle != 0) setFontDescription (labelHandle, fontDesc);
-		if (imageHandle != 0) setFontDescription (imageHandle, fontDesc);
+
+		if (GTK.GTK4) {
+			if (labelHandle != 0) setFontDescription(labelHandle, fontDesc);
+		} else {
+			/*
+			 * GTK3 Workaround for bug which causes incorrect size
+			 * calculation when the button (radio/check) is set active
+			 * before setting font description.
+			 */
+			boolean selected = getSelection();
+			if (selected) setSelection(!selected);
+			if (labelHandle != 0) setFontDescription(labelHandle, fontDesc);
+			setSelection(selected);
+		}
 	}
 }
 
@@ -928,31 +1033,42 @@ void setForegroundGdkRGBA (GdkRGBA rgba) {
 	setForegroundGdkRGBA (fixedHandle, rgba);
 	if (labelHandle != 0) setForegroundGdkRGBA (labelHandle, rgba);
 	if (imageHandle != 0) setForegroundGdkRGBA (imageHandle, rgba);
-
-	/*
-	 * Feature in GTK3: GtkCheckButton & its descendant GtkRadioButton are often invisible or
-	 * very hard to see with certain themes that don't define an icon for Check/Radio buttons.
-	 * Giving them a border color that matches the text color ensures consistent visibility
-	 * across most themes. See bug 463733.
-	 */
-	if (GTK.GTK_VERSION < OS.VERSION (3, 16, 0) && (style & (SWT.CHECK | SWT.RADIO)) != 0) {
-		gtk_swt_set_border_color (rgba);
-	}
 }
 
 @Override
 void setForegroundGdkRGBA (long handle, GdkRGBA rgba) {
-	if (GTK.GTK_VERSION < OS.VERSION(3, 14, 0)) {
-		super.setForegroundGdkRGBA(handle, rgba);
-		return;
-	}
-	GdkRGBA toSet = rgba == null ? display.COLOR_WIDGET_FOREGROUND_RGBA : rgba;
 	long context = GTK.gtk_widget_get_style_context (handle);
 
-	// Form foreground string
+	GdkRGBA toSet = rgba == null ? display.COLOR_WIDGET_FOREGROUND_RGBA : rgba;
+	String css;
 	String color = display.gtk_rgba_to_css_string(toSet);
-	String name = GTK.GTK_VERSION >= OS.VERSION(3, 20, 0) ? "button" : "GtkButton";
-	String css = name + " {color: " + color + ";}";
+	int bits = SWT.CHECK | SWT.RADIO;
+	switch (style & bits) {
+		case SWT.CHECK:
+			css = "checkbutton {color: " + color + ";}";
+			String checkColor;
+			if (toggleButtonTheming) {
+				GdkRGBA checkRGBA = rgba == null ? display.COLOR_TOGGLE_BUTTON_FOREGROUND_RGBA : rgba;
+				checkColor = display.gtk_rgba_to_css_string(checkRGBA);
+			} else {
+				checkColor = display.gtk_rgba_to_css_string(toSet);
+			}
+			css += " check {color: " + checkColor + ";}";
+			break;
+		case SWT.RADIO:
+			css = "radiobutton {color: " + color + ";}";
+			String radioColor;
+			if (toggleButtonTheming) {
+				GdkRGBA radioRGBA = rgba == null ? display.COLOR_TOGGLE_BUTTON_FOREGROUND_RGBA : rgba;
+				radioColor = display.gtk_rgba_to_css_string(radioRGBA);
+			} else {
+				radioColor = display.gtk_rgba_to_css_string(toSet);
+			}
+			css += " radio {color: " + radioColor + ";}";
+			break;
+		default:
+			css = "button {color: " + color + ";}";
+	}
 
 	// Cache foreground color
 	cssForeground = css;
@@ -962,28 +1078,6 @@ void setForegroundGdkRGBA (long handle, GdkRGBA rgba) {
 	gtk_css_provider_load_from_css(context, finalCss);
 }
 
-private void gtk_swt_set_border_color (GdkRGBA rgba) {
-	String css_string = "* {\n";
-	if (rgba != null) {
-		String css_color = display.gtk_rgba_to_css_string (rgba);
-		css_string += "border-color: " + css_color + ";\n";
-	}
-	css_string += "}\n";
-
-	String finalCss;
-	if (GTK.GTK_VERSION >= OS.VERSION(3, 14, 0)) {
-		// Cache foreground color
-		cssForeground += "\n" + css_string;
-		finalCss = display.gtk_css_create_css_color_string (cssBackground, cssForeground, SWT.FOREGROUND);
-	} else {
-		finalCss = css_string;
-	}
-
-	// Apply the CSS
-	long context = GTK.gtk_widget_get_style_context (handle);
-	gtk_css_provider_load_from_css (context, finalCss);
-}
-
 /**
  * Sets the grayed state of the receiver.  This state change
  * only applies if the control was created with the SWT.CHECK
@@ -1002,10 +1096,13 @@ public void setGrayed (boolean grayed) {
 	checkWidget();
 	if ((style & SWT.CHECK) == 0) return;
 	this.grayed = grayed;
-	if (grayed && GTK.gtk_toggle_button_get_active (handle)) {
-		GTK.gtk_toggle_button_set_inconsistent (handle, true);
+
+	if (GTK.GTK4) {
+		boolean inconsistent = grayed && GTK4.gtk_check_button_get_active(handle);
+		GTK4.gtk_check_button_set_inconsistent(handle, inconsistent);
 	} else {
-		GTK.gtk_toggle_button_set_inconsistent (handle, false);
+		boolean inconsistent = grayed && GTK.gtk_toggle_button_get_active(handle);
+		GTK.gtk_toggle_button_set_inconsistent(handle, inconsistent);
 	}
 }
 
@@ -1028,19 +1125,26 @@ public void setGrayed (boolean grayed) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
-public void setImage (Image image) {
-	checkWidget ();
+public void setImage(Image image) {
+	checkWidget();
 	if ((style & SWT.ARROW) != 0) return;
-	if (imageList != null) imageList.dispose ();
-	imageList = null;
+
 	if (image != null) {
-		if (image.isDisposed()) error (SWT.ERROR_INVALID_ARGUMENT);
-		imageList = new ImageList ();
-		int imageIndex = imageList.add (image);
-		long pixbuf = imageList.getPixbuf (imageIndex);
-		gtk_image_set_from_gicon(imageHandle, pixbuf);
+		if (image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
+		if (GTK.GTK4) {
+			long pixbuf = ImageList.createPixbuf(image);
+			long texture = GDK.gdk_texture_new_for_pixbuf(pixbuf);
+			OS.g_object_unref(pixbuf);
+			GTK4.gtk_picture_set_paintable(imageHandle, texture);
+		} else {
+			GTK3.gtk_image_set_from_surface(imageHandle, image.surface);
+		}
 	} else {
-		gtk_image_set_from_gicon (imageHandle, 0);
+		if (GTK.GTK4) {
+			GTK4.gtk_picture_set_paintable(imageHandle, 0);
+		} else {
+			GTK3.gtk_image_set_from_surface(imageHandle, 0);
+		}
 	}
 	this.image = image;
 	updateWidgetsVisibility();
@@ -1058,8 +1162,8 @@ void setOrientation (boolean create) {
 		if (arrowHandle != 0) {
 			byte arrowType [] = (style & SWT.RIGHT_TO_LEFT) != 0 ? GTK.GTK_NAMED_ICON_GO_NEXT : GTK.GTK_NAMED_ICON_GO_PREVIOUS;
 			switch (style & (SWT.LEFT | SWT.RIGHT)) {
-				case SWT.LEFT: GTK.gtk_image_set_from_icon_name (arrowHandle, arrowType, GTK.GTK_ICON_SIZE_MENU); break;
-				case SWT.RIGHT: GTK.gtk_image_set_from_icon_name (arrowHandle, arrowType, GTK.GTK_ICON_SIZE_MENU); break;
+				case SWT.LEFT: GTK3.gtk_image_set_from_icon_name (arrowHandle, arrowType, GTK.GTK_ICON_SIZE_MENU); break;
+				case SWT.RIGHT: GTK3.gtk_image_set_from_icon_name (arrowHandle, arrowType, GTK.GTK_ICON_SIZE_MENU); break;
 			}
 		}
 	}
@@ -1085,15 +1189,30 @@ public void setSelection (boolean selected) {
 	checkWidget();
 	if ((style & (SWT.CHECK | SWT.RADIO | SWT.TOGGLE)) == 0) return;
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CLICKED);
-	GTK.gtk_toggle_button_set_active (handle, selected);
+
+	if (GTK.GTK4 && (style & (SWT.CHECK | SWT.RADIO)) != 0) {
+		GTK4.gtk_check_button_set_active(handle, selected);
+	} else {
+		GTK.gtk_toggle_button_set_active(handle, selected);
+	}
+
 	if ((style & SWT.CHECK) != 0) {
-		if (selected && grayed) {
-			GTK.gtk_toggle_button_set_inconsistent (handle, true);
+		boolean inconsistent = selected && grayed;
+
+		if (GTK.GTK4) {
+			GTK4.gtk_check_button_set_inconsistent(handle, inconsistent);
 		} else {
-			GTK.gtk_toggle_button_set_inconsistent (handle, false);
+			GTK.gtk_toggle_button_set_inconsistent(handle, inconsistent);
 		}
 	}
-	if ((style & SWT.RADIO) != 0) GTK.gtk_toggle_button_set_active (groupHandle, !selected);
+	if ((style & SWT.RADIO) != 0) {
+		if (GTK.GTK4) {
+			GTK4.gtk_check_button_set_active(groupHandle, !selected);
+		} else {
+			GTK.gtk_toggle_button_set_active (groupHandle, !selected);
+		}
+	}
+
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CLICKED);
 }
 
@@ -1200,4 +1319,15 @@ long windowProc (long handle, long arg0, long user_data) {
 	return super.windowProc(handle, arg0, user_data);
 }
 
+@Override
+long dpiChanged(long object, long arg0) {
+	super.dpiChanged(object, arg0);
+
+	if (image != null) {
+		image.internal_gtk_refreshImageForZoom();
+		setImage(image);
+	}
+
+	return 0;
+}
 }
