@@ -19,6 +19,8 @@ import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.gtk3.*;
+import org.eclipse.swt.internal.gtk4.*;
 
 /**
  * Instances of this class are controls that allow the user
@@ -189,7 +191,7 @@ public void add(String string, int index) {
 
 	gtk_combo_box_insert(string, index);
 	if ((style & SWT.RIGHT_TO_LEFT) != 0 && popupHandle != 0) {
-		GTK.gtk_container_forall (popupHandle, display.setDirectionProc, GTK.GTK_TEXT_DIR_RTL);
+		GTK3.gtk_container_forall (popupHandle, display.setDirectionProc, GTK.GTK_TEXT_DIR_RTL);
 	}
 }
 
@@ -231,8 +233,8 @@ private void gtk_combo_box_insert(String string, int index) {
 private void gtk_combo_box_toggle_wrap (boolean wrap) {
 	if (handle == 0 || GTK.GTK4) return;
 	if (!wrap) {
-		if (GTK.gtk_combo_box_get_wrap_width(handle) == 1) {
-			GTK.gtk_combo_box_set_wrap_width(handle, 0);
+		if (GTK3.gtk_combo_box_get_wrap_width(handle) == 1) {
+			GTK3.gtk_combo_box_set_wrap_width(handle, 0);
 		}
 	} else  {
 		if (delayedEnableWrap) {
@@ -241,7 +243,7 @@ private void gtk_combo_box_toggle_wrap (boolean wrap) {
 			delayedEnableWrap = true;
 			display.asyncExec(() -> {
 				if (!isDisposed() && handle != 0) {
-					GTK.gtk_combo_box_set_wrap_width(handle, 1);
+					GTK3.gtk_combo_box_set_wrap_width(handle, 1);
 					delayedEnableWrap = false;
 				}
 			});
@@ -449,10 +451,10 @@ void clearText () {
 		}
 	} else {
 		if (GTK.GTK4) {
-			long bufferHandle = GTK.gtk_entry_get_buffer(entryHandle);
+			long bufferHandle = GTK4.gtk_entry_get_buffer(entryHandle);
 			GTK.gtk_entry_buffer_delete_text(bufferHandle, 0, -1);
 		} else {
-			GTK.gtk_entry_set_text(entryHandle, new byte[1]);
+			GTK3.gtk_entry_set_text(entryHandle, new byte[1]);
 		}
 	}
 	GTK.gtk_combo_box_set_active (handle, -1);
@@ -518,10 +520,10 @@ public void copy () {
 	checkWidget ();
 	if (entryHandle != 0) {
 		if (GTK.GTK4) {
-			long textHandle = GTK.gtk_widget_get_first_child(entryHandle);
-			GTK.gtk_widget_activate_action(textHandle, OS.action_copy_clipboard, null);
+			long textHandle = GTK4.gtk_widget_get_first_child(entryHandle);
+			GTK4.gtk_widget_activate_action(textHandle, OS.action_copy_clipboard, null);
 		} else {
-			GTK.gtk_editable_copy_clipboard(entryHandle);
+			GTK3.gtk_editable_copy_clipboard(entryHandle);
 		}
 	}
 }
@@ -529,16 +531,17 @@ public void copy () {
 @Override
 void createHandle (int index) {
 	state |= HANDLE | MENU;
+
 	fixedHandle = OS.g_object_new(display.gtk_fixed_get_type(), 0);
 	if (fixedHandle == 0) error(SWT.ERROR_NO_HANDLES);
-	gtk_widget_set_has_surface_or_window(fixedHandle, true);
+	if (!GTK.GTK4) GTK3.gtk_widget_set_has_window(fixedHandle, true);
 
 	long oldList = GTK.gtk_window_list_toplevels();
 	if ((style & SWT.READ_ONLY) != 0) {
 		handle = GTK.gtk_combo_box_text_new();
 		if (handle == 0) error(SWT.ERROR_NO_HANDLES);
 
-		cellHandle = GTK.GTK4 ? GTK.gtk_combo_box_get_child(handle) : GTK.gtk_bin_get_child (handle);
+		cellHandle = GTK.GTK4 ? GTK4.gtk_combo_box_get_child(handle) : GTK3.gtk_bin_get_child (handle);
 		if (cellHandle == 0) error(SWT.ERROR_NO_HANDLES);
 
 		gtk_combo_box_toggle_wrap(true);
@@ -546,8 +549,11 @@ void createHandle (int index) {
 		handle = GTK.gtk_combo_box_text_new_with_entry();
 		if (handle == 0) error(SWT.ERROR_NO_HANDLES);
 
-		entryHandle = GTK.GTK4 ? GTK.gtk_combo_box_get_child(handle) : GTK.gtk_bin_get_child(handle);
+		entryHandle = GTK.GTK4 ? GTK4.gtk_combo_box_get_child(handle) : GTK3.gtk_bin_get_child(handle);
 		if (entryHandle == 0) error(SWT.ERROR_NO_HANDLES);
+		if (DISABLE_EMOJI && GTK.GTK_VERSION >= OS.VERSION(3, 22, 20)) {
+		    GTK.gtk_entry_set_input_hints(entryHandle, GTK.GTK_INPUT_HINT_NO_EMOJI);
+		}
 
 		imContext = OS.imContextLast();
 	}
@@ -556,7 +562,7 @@ void createHandle (int index) {
 		OS.swt_fixed_add(fixedHandle, handle);
 	} else {
 		popupHandle = findPopupHandle(oldList);
-		GTK.gtk_container_add (fixedHandle, handle);
+		GTK3.gtk_container_add (fixedHandle, handle);
 	}
 
 	textRenderer = GTK.gtk_cell_renderer_text_new();
@@ -617,10 +623,10 @@ public void cut () {
 	checkWidget ();
 	if (entryHandle != 0) {
 		if (GTK.GTK4) {
-			long textHandle = GTK.gtk_widget_get_first_child(entryHandle);
-			GTK.gtk_widget_activate_action(textHandle, OS.action_cut_clipboard, null);
+			long textHandle = GTK4.gtk_widget_get_first_child(entryHandle);
+			GTK4.gtk_widget_activate_action(textHandle, OS.action_cut_clipboard, null);
 		} else {
-			GTK.gtk_editable_cut_clipboard(entryHandle);
+			GTK3.gtk_editable_cut_clipboard(entryHandle);
 		}
 	}
 }
@@ -652,7 +658,10 @@ boolean filterKey (int keyval, long event) {
 		lastEventTime = time;
 		long imContext = imContext ();
 		if (imContext != 0) {
-			return GTK.gtk_im_context_filter_keypress (imContext, event);
+			if (GTK.GTK4)
+				return GTK4.gtk_im_context_filter_keypress (imContext, event);
+			else
+				return GTK3.gtk_im_context_filter_keypress (imContext, event);
 		}
 	}
 	gdkEventKey = event;
@@ -710,14 +719,14 @@ long findButtonHandle() {
 	long result = 0;
 
 	if (GTK.GTK4) {
-		for (long child = GTK.gtk_widget_get_first_child(handle); child != 0; child = GTK.gtk_widget_get_next_sibling(child)) {
+		for (long child = GTK4.gtk_widget_get_first_child(handle); child != 0; child = GTK4.gtk_widget_get_next_sibling(child)) {
 			if (GTK.GTK_IS_BOX(child)) {
 				buttonBoxHandle = child;
 				break;
 			}
 		}
 
-		for (long child = GTK.gtk_widget_get_first_child(buttonBoxHandle); child != 0; child = GTK.gtk_widget_get_next_sibling(child)) {
+		for (long child = GTK4.gtk_widget_get_first_child(buttonBoxHandle); child != 0; child = GTK4.gtk_widget_get_next_sibling(child)) {
 			if (GTK.GTK_IS_BUTTON(child)) {
 				result = child;
 				break;
@@ -731,7 +740,7 @@ long findButtonHandle() {
 		 * GtkBox has to be retrieved first.
 		 * As it's internal child one can't get it in other way.
 		 */
-		GTK.gtk_container_forall(handle, display.allChildrenProc, 0);
+		GTK3.gtk_container_forall(handle, display.allChildrenProc, 0);
 		if (display.allChildren != 0) {
 			long list = display.allChildren;
 			while (list != 0) {
@@ -747,7 +756,7 @@ long findButtonHandle() {
 		}
 		buttonBoxHandle = childHandle;
 
-		GTK.gtk_container_forall (childHandle, display.allChildrenProc, 0);
+		GTK3.gtk_container_forall (childHandle, display.allChildrenProc, 0);
 		if (display.allChildren != 0) {
 			long list = display.allChildren;
 			while (list != 0) {
@@ -771,7 +780,7 @@ long findArrowHandle() {
 
 	if (cellBoxHandle != 0) {
 		if (GTK.GTK4) {
-			for (long child = GTK.gtk_widget_get_first_child(cellBoxHandle); child != 0; child = GTK.gtk_widget_get_next_sibling(child)) {
+			for (long child = GTK4.gtk_widget_get_first_child(cellBoxHandle); child != 0; child = GTK4.gtk_widget_get_next_sibling(child)) {
 				String name = display.gtk_widget_get_name(child);
 				if (name != null && name.equals("GtkBuiltinIcon")) {
 					result = child;
@@ -779,7 +788,7 @@ long findArrowHandle() {
 				}
 			}
 		} else {
-			GTK.gtk_container_forall (cellBoxHandle, display.allChildrenProc, 0);
+			GTK3.gtk_container_forall (cellBoxHandle, display.allChildrenProc, 0);
 			if (display.allChildren != 0) {
 				long list = display.allChildren;
 				while (list != 0) {
@@ -812,7 +821,7 @@ long findMenuHandle() {
 	long result = 0;
 
 	if (popupHandle != 0) {
-		GTK.gtk_container_forall(popupHandle, display.allChildrenProc, 0);
+		GTK3.gtk_container_forall(popupHandle, display.allChildrenProc, 0);
 		if (display.allChildren != 0) {
 			long list = display.allChildren;
 			while (list != 0) {
@@ -857,7 +866,11 @@ void fixIM () {
 	if (gdkEventKey != 0 && gdkEventKey != -1) {
 		long imContext = imContext ();
 		if (imContext != 0) {
-			GTK.gtk_im_context_filter_keypress (imContext, gdkEventKey);
+			if (GTK.GTK4)
+				GTK4.gtk_im_context_filter_keypress (imContext, gdkEventKey);
+			else
+				GTK3.gtk_im_context_filter_keypress (imContext, gdkEventKey);
+
 			gdkEventKey = -1;
 			return;
 		}
@@ -913,20 +926,20 @@ void hookEvents(long [] handles) {
 		long eventHandle = handles[i];
 		if (eventHandle != 0) {
 			if (GTK.GTK4) {
-				long motionController = GTK.gtk_event_controller_motion_new();
-				GTK.gtk_widget_add_controller(eventHandle, motionController);
+				long motionController = GTK4.gtk_event_controller_motion_new();
+				GTK4.gtk_widget_add_controller(eventHandle, motionController);
 				OS.g_signal_connect (motionController, OS.motion, display.enterMotionScrollProc, MOTION);
 				OS.g_signal_connect (motionController, OS.motion, display.enterMotionScrollProc, MOTION_INVERSE);
 
-				long gestureController = GTK.gtk_gesture_click_new();
-				GTK.gtk_widget_add_controller(eventHandle, gestureController);
+				long gestureController = GTK4.gtk_gesture_click_new();
+				GTK4.gtk_widget_add_controller(eventHandle, gestureController);
 				OS.g_signal_connect(gestureController, OS.pressed, display.gesturePressReleaseProc, GESTURE_PRESSED);
 				OS.g_signal_connect(gestureController, OS.released, display.gesturePressReleaseProc, GESTURE_RELEASED);
 
 				//TODO: GTK4 event-after
 			} else {
 				int eventMask =	GDK.GDK_POINTER_MOTION_MASK | GDK.GDK_BUTTON_PRESS_MASK | GDK.GDK_BUTTON_RELEASE_MASK;
-				GTK.gtk_widget_add_events (eventHandle, eventMask);
+				GTK3.gtk_widget_add_events (eventHandle, eventMask);
 				OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [MOTION_NOTIFY_EVENT], 0, display.getClosure (MOTION_NOTIFY_EVENT), false);
 				OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [MOTION_NOTIFY_EVENT], 0, display.getClosure (MOTION_NOTIFY_EVENT_INVERSE), true);
 
@@ -948,7 +961,7 @@ void hookEvents(long [] handles) {
 					OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [EVENT_AFTER], 0, display.getClosure (EVENT_AFTER), false);
 				}
 
-				if (OS.G_OBJECT_TYPE (eventHandle) == GTK.GTK_TYPE_MENU ()) {
+				if (OS.G_OBJECT_TYPE (eventHandle) == GTK3.GTK_TYPE_MENU ()) {
 					OS.g_signal_connect_closure(eventHandle, OS.selection_done, display.getClosure (SELECTION_DONE), true);
 				}
 			}
@@ -1006,7 +1019,7 @@ boolean dragDetect(int x, int y, boolean filter, boolean dragOnTimeout, boolean[
 	if (filter && entryHandle != 0) {
 		int [] index = new int [1];
 		int [] trailing = new int [1];
-		long layout = GTK.gtk_entry_get_layout (entryHandle);
+		long layout = GTK3.gtk_entry_get_layout (entryHandle);
 		OS.pango_layout_xy_to_index (layout, x * OS.PANGO_SCALE, y * OS.PANGO_SCALE, index, trailing);
 		long ptr = OS.pango_layout_get_text (layout);
 		int position = (int)OS.g_utf8_pointer_to_offset (ptr, ptr + index[0]) + trailing[0];
@@ -1031,7 +1044,30 @@ long enterExitHandle () {
 
 @Override
 long eventWindow () {
-	return paintWindow ();
+	if ((style & SWT.READ_ONLY) != 0) {
+		GTK.gtk_widget_realize (handle);
+		return gtk_widget_get_window (handle);
+	}
+
+	/*
+	 * Single-line Text (GtkEntry in GTK) uses a GDK_INPUT_ONLY
+	 * internal window. This window can't be used for any kind
+	 * of painting, but this is the window to which functions
+	 * like Control.setCursor() should apply.
+	 */
+	GTK.gtk_widget_realize (entryHandle);
+	long window = gtk_widget_get_window (entryHandle);
+	// Find the internal GDK_INPUT_ONLY window
+	long children = GDK.gdk_window_get_children (window);
+	if (children != 0) {
+		long childrenIterator = children;
+		do {
+			window = OS.g_list_data (childrenIterator);
+		} while ((childrenIterator = OS.g_list_next (childrenIterator)) != 0);
+	}
+	OS.g_list_free (children);
+
+	return window;
 }
 
 @Override
@@ -1064,10 +1100,10 @@ Point getCaretLocationInPixels () {
 		return new Point (0, 0);
 	}
 	int index = GTK.gtk_editable_get_position (entryHandle);
-	index = GTK.gtk_entry_text_index_to_layout_index (entryHandle, index);
+	index = GTK3.gtk_entry_text_index_to_layout_index (entryHandle, index);
 	int [] offset_x = new int [1], offset_y = new int [1];
-	GTK.gtk_entry_get_layout_offsets (entryHandle, offset_x, offset_y);
-	long layout = GTK.gtk_entry_get_layout (entryHandle);
+	GTK3.gtk_entry_get_layout_offsets (entryHandle, offset_x, offset_y);
+	long layout = GTK3.gtk_entry_get_layout (entryHandle);
 	PangoRectangle pos = new PangoRectangle ();
 	OS.pango_layout_index_to_pos (layout, index, pos);
 	Point thickness = getThickness (entryHandle);
@@ -1096,7 +1132,7 @@ public int getCaretPosition () {
 	if ((style & SWT.READ_ONLY) != 0) {
 		return 0;
 	}
-	long ptr = GTK.gtk_entry_get_text (entryHandle);
+	long ptr = GTK3.gtk_entry_get_text (entryHandle);
 	return (int)OS.g_utf8_offset_to_utf16_offset (ptr, GTK.gtk_editable_get_position (entryHandle));
 }
 
@@ -1160,7 +1196,12 @@ public int getItemCount () {
  */
 public int getItemHeight () {
 	checkWidget();
-	return fontHeight (getFontDescription (), handle);
+
+	long fontDesc = getFontDescription ();
+	int result = fontHeight (fontDesc, handle);
+	OS.pango_font_description_free(fontDesc);
+
+	return result;
 }
 
 /**
@@ -1263,7 +1304,7 @@ public Point getSelection () {
 	int [] end = new int [1];
 	if (entryHandle != 0) {
 		GTK.gtk_editable_get_selection_bounds (entryHandle, start, end);
-		long ptr = GTK.gtk_entry_get_text (entryHandle);
+		long ptr = GTK3.gtk_entry_get_text (entryHandle);
 		start[0] = (int)OS.g_utf8_offset_to_utf16_offset (ptr, start[0]);
 		end[0] = (int)OS.g_utf8_offset_to_utf16_offset (ptr, end[0]);
 	}
@@ -1304,10 +1345,10 @@ public String getText () {
 	if (entryHandle != 0) {
 		long str = 0;
 		if (GTK.GTK4) {
-			long bufferHandle = GTK.gtk_entry_get_buffer(entryHandle);
+			long bufferHandle = GTK4.gtk_entry_get_buffer(entryHandle);
 			str = GTK.gtk_entry_buffer_get_text(bufferHandle);
 		} else {
-			str = GTK.gtk_entry_get_text(entryHandle);
+			str = GTK3.gtk_entry_get_text(entryHandle);
 		}
 
 		if (str == 0) {
@@ -1458,7 +1499,7 @@ long gtk_changed (long widget) {
 	* is to post the modify event when the user is typing.
 	*/
 	boolean keyPress = false;
-	long eventPtr = GTK.GTK4 ? 0 : GTK.gtk_get_current_event ();
+	long eventPtr = GTK.GTK4 ? 0 : GTK3.gtk_get_current_event ();
 	if (eventPtr != 0) {
 		int eventType = GDK.gdk_event_get_event_type(eventPtr);
 		eventType = fixGdkEventTypeValues(eventType);
@@ -1519,7 +1560,7 @@ long gtk_commit (long imContext, long text) {
 @Override
 long gtk_delete_text (long widget, long start_pos, long end_pos) {
 	if (!hooks (SWT.Verify) && !filters (SWT.Verify)) return 0;
-	long ptr = GTK.gtk_entry_get_text (entryHandle);
+	long ptr = GTK3.gtk_entry_get_text (entryHandle);
 	if (end_pos == -1) end_pos = OS.g_utf8_strlen (ptr, -1);
 	int start = (int)OS.g_utf8_offset_to_utf16_offset (ptr, start_pos);
 	int end = (int)OS.g_utf8_offset_to_utf16_offset (ptr, end_pos);
@@ -1691,7 +1732,7 @@ long gtk_insert_text (long widget, long new_text, long new_text_length, long pos
 	String oldText = new String (Converter.mbcsToWcs (buffer));
 	int [] pos = new int [1];
 	C.memmove (pos, position, 4);
-	long ptr = GTK.gtk_entry_get_text (entryHandle);
+	long ptr = GTK3.gtk_entry_get_text (entryHandle);
 	if (pos [0] == -1) pos [0] = (int)OS.g_utf8_strlen (ptr, -1);
 	int start = (int)OS.g_utf8_offset_to_utf16_offset (ptr, pos [0]);
 	String newText = verifyText (oldText, start, start);
@@ -1783,7 +1824,7 @@ long gtk_key_press_event (long widget, long event) {
 long gtk_populate_popup (long widget, long menu) {
 	if ((style & SWT.RIGHT_TO_LEFT) != 0) {
 		GTK.gtk_widget_set_direction (menu, GTK.GTK_TEXT_DIR_RTL);
-		GTK.gtk_container_forall (menu, display.setDirectionProc, GTK.GTK_TEXT_DIR_RTL);
+		GTK3.gtk_container_forall (menu, display.setDirectionProc, GTK.GTK_TEXT_DIR_RTL);
 	}
 	return 0;
 }
@@ -1866,34 +1907,17 @@ boolean isFocusHandle(long widget) {
 	return super.isFocusHandle (widget);
 }
 
-@Override
-long paintWindow () {
-	long childHandle =  entryHandle != 0 ? entryHandle : handle;
-	GTK.gtk_widget_realize (childHandle);
-	long window = gtk_widget_get_window (childHandle);
-	if ((style & SWT.READ_ONLY) != 0) return window;
-	long children = GDK.gdk_window_get_children (window);
-	if (children != 0) {
-		/*
-		 * The only direct child of GtkComboBox since 3.20 is GtkBox thus the children
-		 * have to be traversed to get to the entry one.
-		 */
-		do {
-			window = OS.g_list_data (children);
-		} while ((children = OS.g_list_next (children)) != 0);
-	}
-	OS.g_list_free (children);
-
-	return window;
-}
-
 @Override
 long paintSurface () {
 	long childHandle =  entryHandle != 0 ? entryHandle : handle;
 	GTK.gtk_widget_realize (childHandle);
 	long surface = gtk_widget_get_surface (childHandle);
 	if ((style & SWT.READ_ONLY) != 0) return surface;
-	/* TODO: GTK4 no access to children of the surface, for combobox may need to use gtk_combo_box_get_child () */
+	/*
+	 * TODO: GTK4 no access to children of the surface
+	 * for combobox may need to use gtk_combo_box_get_child ().
+	 * See also Bug 570331, maybe entire function just needs to be removed
+	 */
 
 	return surface;
 }
@@ -1914,7 +1938,7 @@ long paintSurface () {
  */
 public void paste () {
 	checkWidget ();
-	if (entryHandle != 0) GTK.gtk_editable_paste_clipboard (entryHandle);
+	if (entryHandle != 0) GTK3.gtk_editable_paste_clipboard (entryHandle);
 }
 
 @Override
@@ -2211,9 +2235,9 @@ void setButtonBackgroundGdkRGBA (GdkRGBA rgba) {
 		OS.g_object_unref(buttonProvider);
 	}
 	if (GTK.GTK4) {
-		GTK.gtk_css_provider_load_from_data (provider, Converter.wcsToMbcs (finalCss, true), -1);
+		GTK4.gtk_css_provider_load_from_data (buttonProvider, Converter.wcsToMbcs (finalCss, true), -1);
 	} else {
-		GTK.gtk_css_provider_load_from_data (provider, Converter.wcsToMbcs (finalCss, true), -1, null);
+		GTK3.gtk_css_provider_load_from_data (buttonProvider, Converter.wcsToMbcs (finalCss, true), -1, null);
 	}
 }
 
@@ -2235,9 +2259,9 @@ void setButtonForegroundGdkRGBA (GdkRGBA rgba) {
 		OS.g_object_unref(buttonProvider);
 	}
 	if (GTK.GTK4) {
-		GTK.gtk_css_provider_load_from_data (provider, Converter.wcsToMbcs (finalCss, true), -1);
+		GTK4.gtk_css_provider_load_from_data (provider, Converter.wcsToMbcs (finalCss, true), -1);
 	} else {
-		GTK.gtk_css_provider_load_from_data (provider, Converter.wcsToMbcs (finalCss, true), -1, null);
+		GTK3.gtk_css_provider_load_from_data (provider, Converter.wcsToMbcs (finalCss, true), -1, null);
 	}
 }
 
@@ -2248,8 +2272,7 @@ void setBackgroundGdkRGBA (long context, long handle, GdkRGBA rgba) {
 	} else {
 		background = rgba;
 	}
-	// CSS to be parsed for various widgets within Combo
-	String css = "* {";
+
 	String color, menuColor;
 	if (rgba != null) {
 		color = display.gtk_rgba_to_css_string (rgba);
@@ -2263,8 +2286,9 @@ void setBackgroundGdkRGBA (long context, long handle, GdkRGBA rgba) {
 
 		menuColor = display.gtk_rgba_to_css_string (display.COLOR_LIST_BACKGROUND_RGBA);
 	}
-	css += "background: " + color + ";}\n";
 
+	// CSS to be parsed for various widgets within Combo
+	String css = "* {background: " + color + ";}\n";
 	// Set the selected background color
 	GdkRGBA selectedBackground = display.getSystemColor(SWT.COLOR_LIST_SELECTION).handle;
 	GdkRGBA selectedForeground = display.getSystemColor(SWT.COLOR_LIST_SELECTION_TEXT).handle;
@@ -2285,7 +2309,12 @@ void setBackgroundGdkRGBA (long context, long handle, GdkRGBA rgba) {
 	}
 
 	String menuCss = "menu { background: " + menuColor + ";}";
-	GTK.gtk_css_provider_load_from_data (getComboProvider(), Converter.wcsToMbcs (menuCss, true), -1, null);
+	if (GTK.GTK4) {
+		GTK4.gtk_css_provider_load_from_data(getComboProvider(), Converter.wcsToMbcs(menuCss, true), -1);
+	} else {
+		GTK3.gtk_css_provider_load_from_data(getComboProvider(), Converter.wcsToMbcs(menuCss, true), -1, null);
+	}
+
 }
 
 long getComboProvider() {
@@ -2430,7 +2459,7 @@ public void setItem (int index, String string) {
 	gtk_combo_box_insert(string, index);
 
 	if ((style & SWT.RIGHT_TO_LEFT) != 0 && popupHandle != 0) {
-		GTK.gtk_container_forall (popupHandle, display.setDirectionProc, GTK.GTK_TEXT_DIR_RTL);
+		GTK3.gtk_container_forall (popupHandle, display.setDirectionProc, GTK.GTK_TEXT_DIR_RTL);
 	}
 }
 
@@ -2463,7 +2492,7 @@ public void setItems (String... items) {
 		String string = items [i];
 		gtk_combo_box_insert(string, i);
 		if ((style & SWT.RIGHT_TO_LEFT) != 0 && popupHandle != 0) {
-			GTK.gtk_container_forall (popupHandle, display.setDirectionProc, GTK.GTK_TEXT_DIR_RTL);
+			GTK3.gtk_container_forall (popupHandle, display.setDirectionProc, GTK.GTK_TEXT_DIR_RTL);
 		}
 	}
 }
@@ -2509,7 +2538,7 @@ void setOrientation (boolean create) {
 		if (entryHandle != 0) GTK.gtk_widget_set_direction (entryHandle, dir);
 		if (cellHandle != 0) GTK.gtk_widget_set_direction (cellHandle, dir);
 		if (!create) {
-			if (popupHandle != 0) GTK.gtk_container_forall (popupHandle, display.setDirectionProc, dir);
+			if (popupHandle != 0) GTK3.gtk_container_forall (popupHandle, display.setDirectionProc, dir);
 		}
 	}
 }
@@ -2557,10 +2586,10 @@ public void setSelection (Point selection) {
 	if (entryHandle != 0) {
 		long textPtr = 0;
 		if (GTK.GTK4) {
-			long bufferHandle = GTK.gtk_entry_get_buffer(entryHandle);
+			long bufferHandle = GTK4.gtk_entry_get_buffer(entryHandle);
 			textPtr = GTK.gtk_entry_buffer_get_text(bufferHandle);
 		} else {
-			textPtr = GTK.gtk_entry_get_text(entryHandle);
+			textPtr = GTK3.gtk_entry_get_text(entryHandle);
 		}
 
 		int start = (int)OS.g_utf16_offset_to_utf8_offset(textPtr, selection.x);
@@ -2613,7 +2642,7 @@ public void setText (String string) {
 	* fix is to block the firing of these events and fire them ourselves in a consistent manner.
 	*/
 	if (hooks (SWT.Verify) || filters (SWT.Verify)) {
-		long ptr = GTK.gtk_entry_get_text (entryHandle);
+		long ptr = GTK3.gtk_entry_get_text (entryHandle);
 		string = verifyText (string, 0, (int)OS.g_utf16_strlen (ptr, -1));
 		if (string == null) return;
 	}
@@ -2623,10 +2652,10 @@ public void setText (String string) {
 	OS.g_signal_handlers_block_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_TEXT);
 	OS.g_signal_handlers_block_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
 	if (GTK.GTK4) {
-		long bufferHandle = GTK.gtk_entry_get_buffer(entryHandle);
+		long bufferHandle = GTK4.gtk_entry_get_buffer(entryHandle);
 		GTK.gtk_entry_buffer_set_text(bufferHandle, buffer, string.length());
 	} else {
-		GTK.gtk_entry_set_text (entryHandle, buffer);
+		GTK3.gtk_entry_set_text (entryHandle, buffer);
 	}
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
 	OS.g_signal_handlers_unblock_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
@@ -2728,7 +2757,7 @@ String verifyText (String string, int start, int end) {
 	event.text = string;
 	event.start = start;
 	event.end = end;
-	long eventPtr = GTK.GTK4 ? 0 : GTK.gtk_get_current_event ();
+	long eventPtr = GTK.GTK4 ? 0 : GTK3.gtk_get_current_event ();
 	if (eventPtr != 0) {
 		int type = GDK.gdk_event_get_event_type(eventPtr);
 		type = fixGdkEventTypeValues(type);
