@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -21,6 +21,8 @@ import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.cairo.*;
 import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.gtk3.*;
+import org.eclipse.swt.internal.gtk4.*;
 
 /**
  * Instances of this class are controls which are capable
@@ -159,31 +161,44 @@ static int checkStyle (int style) {
 	return style;
 }
 
-Control [] _getChildren () {
-	long parentHandle = parentingHandle ();
-	long list = GTK.gtk_container_get_children (parentHandle);
-	if (list == 0) return new Control [0];
-	int count = OS.g_list_length (list);
-	Control [] children = new Control [count];
-	int i = 0;
-	long temp = list;
-	while (temp != 0) {
-		long handle = OS.g_list_data (temp);
-		if (handle != 0) {
-			Widget widget = display.getWidget (handle);
-			if (widget != null && widget != this) {
-				if (widget instanceof Control) {
-					children [i++] = (Control) widget;
-				}
+Control[] _getChildren () {
+	long parentHandle = parentingHandle();
+
+	if (GTK.GTK4) {
+		ArrayList<Control> childrenList = new ArrayList<>();
+		for (long child = GTK4.gtk_widget_get_first_child(parentHandle); child != 0; child = GTK4.gtk_widget_get_next_sibling(child)) {
+			Widget childWidget = display.getWidget(child);
+			if (childWidget != null && childWidget instanceof Control && childWidget != this) {
+				childrenList.add((Control)childWidget);
 			}
 		}
-		temp = OS.g_list_next (temp);
+
+		return childrenList.toArray(new Control[childrenList.size()]);
+	} else {
+		long list = GTK3.gtk_container_get_children (parentHandle);
+		if (list == 0) return new Control [0];
+		int count = OS.g_list_length (list);
+		Control [] children = new Control [count];
+		int i = 0;
+		long temp = list;
+		while (temp != 0) {
+			long handle = OS.g_list_data (temp);
+			if (handle != 0) {
+				Widget widget = display.getWidget (handle);
+				if (widget != null && widget != this) {
+					if (widget instanceof Control) {
+						children [i++] = (Control) widget;
+					}
+				}
+			}
+			temp = OS.g_list_next (temp);
+		}
+		OS.g_list_free (list);
+		if (i == count) return children;
+		Control [] newChildren = new Control [i];
+		System.arraycopy (children, 0, newChildren, 0, i);
+		return newChildren;
 	}
-	OS.g_list_free (list);
-	if (i == count) return children;
-	Control [] newChildren = new Control [i];
-	System.arraycopy (children, 0, newChildren, 0, i);
-	return newChildren;
 }
 
 Control [] _getTabList () {
@@ -303,19 +318,31 @@ void createHandle (int index, boolean fixed, boolean scrolled) {
 		if (fixed) {
 			fixedHandle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
 			if (fixedHandle == 0) error (SWT.ERROR_NO_HANDLES);
-			gtk_widget_set_has_surface_or_window (fixedHandle, true);
+			if (!GTK.GTK4) GTK3.gtk_widget_set_has_window(fixedHandle, true);
 		}
+
 		long vadj = GTK.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
 		if (vadj == 0) error (SWT.ERROR_NO_HANDLES);
 		long hadj = GTK.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
 		if (hadj == 0) error (SWT.ERROR_NO_HANDLES);
-		scrolledHandle = GTK.gtk_scrolled_window_new (hadj, vadj);
+
+		if (GTK.GTK4) {
+			scrolledHandle = GTK4.gtk_scrolled_window_new();
+			GTK.gtk_scrolled_window_set_hadjustment(scrolledHandle, hadj);
+			GTK.gtk_scrolled_window_set_vadjustment(scrolledHandle, vadj);
+			GTK.gtk_widget_set_hexpand(scrolledHandle, true);
+			GTK.gtk_widget_set_vexpand(scrolledHandle, true);
+		} else {
+			scrolledHandle = GTK3.gtk_scrolled_window_new (hadj, vadj);
+		}
 		if (scrolledHandle == 0) error (SWT.ERROR_NO_HANDLES);
 	}
-	handle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
-	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
-	gtk_widget_set_has_surface_or_window (handle, true);
+
+	handle = OS.g_object_new(display.gtk_fixed_get_type(), 0);
+	if (handle == 0) error(SWT.ERROR_NO_HANDLES);
+	if (!GTK.GTK4) GTK3.gtk_widget_set_has_window(handle, true);
 	GTK.gtk_widget_set_can_focus (handle, true);
+
 	if ((style & SWT.EMBEDDED) == 0) {
 		if ((state & CANVAS) != 0) {
 			/* Prevent an input method context from being created for the Browser widget */
@@ -326,22 +353,36 @@ void createHandle (int index, boolean fixed, boolean scrolled) {
 		}
 	}
 	if (scrolled) {
-		if (fixed) GTK.gtk_container_add (fixedHandle, scrolledHandle);
-		/*
-		* Force the scrolledWindow to have a single child that is
-		* not scrolled automatically.  Calling gtk_container_add()
-		* seems to add the child correctly but cause a warning.
-		*/
-		boolean warnings = display.getWarnings ();
-		display.setWarnings (false);
-		GTK.gtk_container_add (scrolledHandle, handle);
-		display.setWarnings (warnings);
+		if (fixed) {
+			if (GTK.GTK4) {
+				OS.swt_fixed_add(fixedHandle, scrolledHandle);
+			} else {
+				GTK3.gtk_container_add (fixedHandle, scrolledHandle);
+			}
+		}
+		if (GTK.GTK4) {
+			GTK4.gtk_scrolled_window_set_child(scrolledHandle, handle);
+		} else {
+			/*
+			* Force the scrolledWindow to have a single child that is
+			* not scrolled automatically.  Calling gtk_container_add()
+			* seems to add the child correctly but cause a warning.
+			*/
+			boolean warnings = display.getWarnings ();
+			display.setWarnings (false);
+			GTK3.gtk_container_add (scrolledHandle, handle);
+			display.setWarnings (warnings);
+		}
 
 		int hsp = (style & SWT.H_SCROLL) != 0 ? GTK.GTK_POLICY_ALWAYS : GTK.GTK_POLICY_NEVER;
 		int vsp = (style & SWT.V_SCROLL) != 0 ? GTK.GTK_POLICY_ALWAYS : GTK.GTK_POLICY_NEVER;
 		GTK.gtk_scrolled_window_set_policy (scrolledHandle, hsp, vsp);
 		if (hasBorder ()) {
-			GTK.gtk_scrolled_window_set_shadow_type (scrolledHandle, GTK.GTK_SHADOW_ETCHED_IN);
+			if (GTK.GTK4) {
+				GTK4.gtk_scrolled_window_set_has_frame(scrolledHandle, true);
+			} else {
+				GTK3.gtk_scrolled_window_set_shadow_type (scrolledHandle, GTK.GTK_SHADOW_ETCHED_IN);
+			}
 		}
 	}
 	if ((style & SWT.EMBEDDED) != 0) {
@@ -353,23 +394,26 @@ void createHandle (int index, boolean fixed, boolean scrolled) {
 		} else {
 			socketHandle = GTK.gtk_socket_new ();
 			if (socketHandle == 0) error (SWT.ERROR_NO_HANDLES);
-			GTK.gtk_container_add (handle, socketHandle);
+			GTK3.gtk_container_add (handle, socketHandle);
 		}
 	}
-	if ((style & SWT.NO_REDRAW_RESIZE) != 0 && (style & SWT.RIGHT_TO_LEFT) == 0) {
-		GTK.gtk_widget_set_redraw_on_allocate (handle, false);
-	}
-	/*
-	* Bug in GTK.  When a widget is double buffered and the back
-	* pixmap is null, the double buffer pixmap is filled with the
-	* background of the widget rather than the current contents of
-	* the screen.  If nothing is drawn during an expose event,
-	* the pixels are altered.  The fix is to clear double buffering
-	* when NO_BACKGROUND is set and DOUBLE_BUFFERED
-	* is not explicitly set.
-	*/
-	if ((style & SWT.DOUBLE_BUFFERED) == 0 && (style & SWT.NO_BACKGROUND) != 0) {
-		GTK.gtk_widget_set_double_buffered (handle, false);
+
+	if (!GTK.GTK4) {
+		if ((style & SWT.NO_REDRAW_RESIZE) != 0 && (style & SWT.RIGHT_TO_LEFT) == 0) {
+			GTK3.gtk_widget_set_redraw_on_allocate (handle, false);
+		}
+		/*
+		* Bug in GTK.  When a widget is double buffered and the back
+		* pixmap is null, the double buffer pixmap is filled with the
+		* background of the widget rather than the current contents of
+		* the screen.  If nothing is drawn during an expose event,
+		* the pixels are altered.  The fix is to clear double buffering
+		* when NO_BACKGROUND is set and DOUBLE_BUFFERED
+		* is not explicitly set.
+		*/
+		if ((style & SWT.DOUBLE_BUFFERED) == 0 && (style & SWT.NO_BACKGROUND) != 0) {
+			GTK3.gtk_widget_set_double_buffered (handle, false);
+		}
 	}
 }
 
@@ -437,21 +481,17 @@ void adjustChildClipping (long widget) {
 
 @Override
 long gtk_draw (long widget, long cairo) {
-	if (GTK.GTK_VERSION >= OS.VERSION(3, 14, 0)) {
-		long context = GTK.gtk_widget_get_style_context(widget);
-		GtkAllocation allocation = new GtkAllocation();
-		GTK.gtk_widget_get_allocation (widget, allocation);
-		int width = (state & ZERO_WIDTH) != 0 ? 0 : allocation.width;
-		int height = (state & ZERO_HEIGHT) != 0 ? 0 : allocation.height;
-		// We specify a 0 value for x & y as we want the whole widget to be
-		// colored, not some portion of it.
-		GTK.gtk_render_background(context, cairo, 0, 0, width, height);
-		if (GTK.GTK_VERSION >= OS.VERSION(3, 20, 0)) {
-			// If fixClipHandle is set: iterate through the children of widget
-			// and set their clips to be that of their allocation
-			if (widget == fixClipHandle) fixClippings();
-		}
-	}
+	long context = GTK.gtk_widget_get_style_context(widget);
+	GtkAllocation allocation = new GtkAllocation();
+	GTK.gtk_widget_get_allocation (widget, allocation);
+	int width = (state & ZERO_WIDTH) != 0 ? 0 : allocation.width;
+	int height = (state & ZERO_HEIGHT) != 0 ? 0 : allocation.height;
+	// We specify a 0 value for x & y as we want the whole widget to be
+	// colored, not some portion of it.
+	GTK.gtk_render_background(context, cairo, 0, 0, width, height);
+	// If fixClipHandle is set: iterate through the children of widget
+	// and set their clips to be that of their allocation
+	if (widget == fixClipHandle) fixClippings();
 	return super.gtk_draw(widget, cairo);
 }
 
@@ -634,24 +674,10 @@ void fixZOrder () {
 	if ((state & CANVAS) != 0) return;
 	long parentHandle = parentingHandle ();
 	if (GTK.GTK4) {
-		long parentSurface = gtk_widget_get_surface (parentHandle);
-		if (parentSurface == 0) return;
-		long [] userData = new long [1];
-		long surfaceList = GDK.gdk_surface_get_children (parentSurface);
-		if (surfaceList != 0) {
-			long surfaces = surfaceList;
-			while (surfaces != 0) {
-				long surface = OS.g_list_data (surfaces);
-				if (surface != redrawSurface) {
-					GDK.gdk_surface_get_user_data (surface, userData);
-					if (userData [0] == 0 || OS.G_OBJECT_TYPE (userData [0]) != display.gtk_fixed_get_type ()) {
-						GDK.gdk_surface_lower (surface);
-					}
-				}
-				surfaces = OS.g_list_next (surfaces);
-			}
-			OS.g_list_free (surfaceList);
-		}
+		/* TODO: GTK4 parent does not hold the list of children it has created (parent-children relationship can only be done
+		 * with GdkPopup) Will need to consider if we can get children some other way or not have to do fixZOrder at all
+		 * and use GdkPopup autohide feature.
+		 */
 	} else {
 		long parentWindow = gtk_widget_get_window (parentHandle);
 		if (parentWindow == 0) return;
@@ -738,14 +764,24 @@ public Control [] getChildren () {
 }
 
 int getChildrenCount () {
-	/*
-	* NOTE: The current implementation will count
-	* non-registered children.
-	*/
-	long list = GTK.gtk_container_get_children (handle);
-	if (list == 0) return 0;
-	int count = OS.g_list_length (list);
-	OS.g_list_free (list);
+	int count = 0;
+
+	if (GTK.GTK4) {
+		for (long child = GTK4.gtk_widget_get_first_child(handle); child != 0; child = GTK4.gtk_widget_get_next_sibling(child)) {
+			count++;
+		}
+	} else {
+		/*
+		* NOTE: The current implementation will count
+		* non-registered children.
+		*/
+		long list = GTK3.gtk_container_get_children(handle);
+		if (list != 0) {
+			count = OS.g_list_length(list);
+			OS.g_list_free(list);
+		}
+	}
+
 	return count;
 }
 
@@ -756,7 +792,9 @@ Rectangle getClientAreaInPixels () {
 		if ((state & ZERO_WIDTH) != 0 && (state & ZERO_HEIGHT) != 0) {
 			return new Rectangle (0, 0, 0, 0);
 		}
-		forceResize ();
+		if(RESIZE_ON_GETCLIENTAREA) {
+			forceResize ();
+		}
 		long clientHandle = clientHandle ();
 		GtkAllocation allocation = new GtkAllocation();
 		GTK.gtk_widget_get_allocation (clientHandle, allocation);
@@ -844,6 +882,7 @@ long gtk_button_press_event (long widget, long event) {
 		if ((style & SWT.NO_FOCUS) == 0 && hooksKeys ()) {
 			int [] eventButton = new int [1];
 			GDK.gdk_event_get_button(event, eventButton);
+
 			if (eventButton[0] == 1) {
 				if (getChildrenCount () == 0) setFocus ();
 			}
@@ -864,7 +903,12 @@ long gtk_key_press_event (long widget, long event) {
 	*/
 	if ((state & CANVAS) != 0 && socketHandle == 0) {
 		int [] eventKeyval = new int [1];
-		GDK.gdk_event_get_keyval(event, eventKeyval);
+		if (GTK.GTK4) {
+			eventKeyval[0] = GDK.gdk_key_event_get_keyval(event);
+		} else {
+			GDK.gdk_event_get_keyval(event, eventKeyval);
+		}
+
 		switch (eventKeyval[0]) {
 			case GDK.GDK_Return:
 			case GDK.GDK_KP_Enter: return 1;
@@ -944,7 +988,7 @@ boolean hasBorder () {
 void hookEvents () {
 	super.hookEvents ();
 	if ((state & CANVAS) != 0) {
-		if (!GTK.GTK4) GTK.gtk_widget_add_events (handle, GDK.GDK_POINTER_MOTION_HINT_MASK);
+		if (!GTK.GTK4) GTK3.gtk_widget_add_events (handle, GDK.GDK_POINTER_MOTION_HINT_MASK);
 		if (scrolledHandle != 0) {
 			OS.g_signal_connect_closure (scrolledHandle, OS.scroll_child, display.getClosure (SCROLL_CHILD), false);
 		}
@@ -1252,7 +1296,7 @@ public void layout (Control [] changed, int flags) {
 				composite = child.parent;
 			}
 		}
-		if ((flags & SWT.DEFER) != 0) {
+		if (!display.externalEventLoop && (flags & SWT.DEFER) != 0) {
 			setLayoutDeferred (true);
 			display.addLayoutDeferred (this);
 		}
@@ -1262,7 +1306,7 @@ public void layout (Control [] changed, int flags) {
 	} else {
 		if (layout == null && (flags & SWT.ALL) == 0) return;
 		markLayout ((flags & SWT.CHANGED) != 0, (flags & SWT.ALL) != 0);
-		if ((flags & SWT.DEFER) != 0) {
+		if (!display.externalEventLoop && (flags & SWT.DEFER) != 0) {
 			setLayoutDeferred (true);
 			display.addLayoutDeferred (this);
 		}
@@ -1316,11 +1360,7 @@ void moveChildren(int oldWidth) {
 		if (oldWidth > 0) x = oldWidth - controlWidth - x;
 		int clientWidth = getClientWidth ();
 		x = clientWidth - controlWidth - x;
-		if (GTK.GTK4) {
-			if (child.enableSurface != 0) {
-				GDK.gdk_surface_move (child.enableSurface, x, y);
-			}
-		} else {
+		if (!GTK.GTK4) {
 			if (child.enableWindow != 0) {
 				GDK.gdk_window_move (child.enableWindow, x, y);
 			}
@@ -1405,7 +1445,7 @@ void printWidget (GC gc, long drawable, int depth, int x, int y) {
 void connectFixedHandleDraw () {
 	long paintHandle = fixedHandle;
 	int paintMask = GDK.GDK_EXPOSURE_MASK;
-	GTK.gtk_widget_add_events (paintHandle, paintMask);
+	GTK3.gtk_widget_add_events (paintHandle, paintMask);
 
 	OS.g_signal_connect_closure_by_id (paintHandle, display.signalIds [DRAW], 0, display.getClosure (DRAW), true);
 }
@@ -1426,53 +1466,49 @@ void connectFixedHandleDraw () {
  */
 void propagateDraw (long container, long cairo) {
 	if (container == fixedHandle) {
-		long list = GTK.gtk_container_get_children (container);
-		long temp = list;
-		while (temp != 0) {
-			long child = OS.g_list_data (temp);
-			if (child != 0) {
-				Widget widget = display.getWidget (child);
-				if (widget != this) {
-					if (noChildDrawing) {
-						Boolean childLowered = childrenLowered.get(widget);
-						if (childLowered == null) {
-							childrenLowered.put(widget, false);
-							childLowered = false;
-						}
-						GtkAllocation allocation = new GtkAllocation ();
-						GTK.gtk_widget_get_allocation(child, allocation);
-						if (allocation.y < 0) {
-							if (!childLowered) {
-								if (GTK.GTK4) {
-									long surface = gtk_widget_get_surface(child);
-									GDK.gdk_surface_lower(surface);
-								} else {
+		if (GTK.GTK4) {
+			for (long child = GTK4.gtk_widget_get_first_child(container); child != 0; child = GTK4.gtk_widget_get_next_sibling(child)) {
+				//TODO: GTK4 no gtk_container_propagate_draw. Possibly not required at all.
+			}
+		} else {
+			long list = GTK3.gtk_container_get_children (container);
+			long temp = list;
+			while (temp != 0) {
+				long child = OS.g_list_data (temp);
+				if (child != 0) {
+					Widget widget = display.getWidget (child);
+					if (widget != this) {
+						if (noChildDrawing) {
+							Boolean childLowered = childrenLowered.get(widget);
+							if (childLowered == null) {
+								childrenLowered.put(widget, false);
+								childLowered = false;
+							}
+							GtkAllocation allocation = new GtkAllocation ();
+							GTK.gtk_widget_get_allocation(child, allocation);
+							if (allocation.y < 0) {
+								if (!childLowered) {
 									long window = gtk_widget_get_window(child);
 									GDK.gdk_window_lower(window);
+									childrenLowered.put(widget, true);
 								}
-								childrenLowered.put(widget, true);
-							}
-						} else {
-							if (childLowered) {
-								if (GTK.GTK4) {
-									long surface = gtk_widget_get_surface(child);
-									GDK.gdk_surface_raise(surface);
-								} else {
+							} else {
+								if (childLowered) {
 									long window = gtk_widget_get_window(child);
 									GDK.gdk_window_raise(window);
+									childrenLowered.put(widget, false);
 								}
-								childrenLowered.put(widget, false);
+								GTK3.gtk_container_propagate_draw(container, child, cairo);
 							}
-							GTK.gtk_container_propagate_draw(container, child, cairo);
+						} else {
+							GTK3.gtk_container_propagate_draw(container, child, cairo);
 						}
-					} else {
-						GTK.gtk_container_propagate_draw(container, child, cairo);
 					}
 				}
+				temp = OS.g_list_next (temp);
 			}
-			temp = OS.g_list_next (temp);
+			OS.g_list_free (list);
 		}
-		OS.g_list_free (list);
 	}
 }
 
@@ -1702,7 +1738,11 @@ boolean setTabGroupFocus (boolean next) {
 	Control [] children = _getChildren ();
 	for (int i=0; i<children.length; i++) {
 		Control child = children [i];
-		if (child.isTabItem () && child.setTabItemFocus (next)) return true;
+		/*
+		 * It is unlikely but possible that a child is disposed at this point, for more
+		 * details refer bug 381668.
+		 */
+		if (!child.isDisposed() && child.isTabItem () && child.setTabItemFocus (next)) return true;
 	}
 	return false;
 }
