@@ -19,6 +19,8 @@ import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.gtk3.*;
+import org.eclipse.swt.internal.gtk4.*;
 
 /**
  * Instances of this class represent a selectable user interface object
@@ -59,6 +61,11 @@ public class MenuItem extends Item {
 	int accelerator, userId;
 	String toolTipText;
 
+	/** GTK4 only fields */
+	long modelHandle, actionHandle, shortcutHandle;
+	int parentMenuPosition;
+	String actionName;
+
 /**
  * Constructs a new instance of this class given its parent
  * (which must be a <code>Menu</code>) and a style value
@@ -150,8 +157,8 @@ void addAccelerator (long accelGroup) {
 }
 
 void addAccelerators (long accelGroup) {
-	addAccelerator (accelGroup);
-	if (menu != null) menu.addAccelerators (accelGroup);
+	addAccelerator(accelGroup);
+	if (menu != null) menu.addAccelerators(accelGroup);
 }
 
 /**
@@ -260,15 +267,58 @@ void createHandle (int index) {
 	int bits = SWT.CHECK | SWT.RADIO | SWT.PUSH | SWT.SEPARATOR | SWT.CASCADE;
 
 	if (GTK.GTK4) {
-		/* TODO: Need to specify the correct GAction in order to get the different styles
-		 * SWT.SEPARATOR is not explicitly represented in the menu model, instead inserted between any two
-		 * non-empty sections of the menu
-		 * REF: https://developer.gnome.org/gio/stable/GMenuModel.html#GMenuModel-struct */
-		OS.g_menu_insert(parent.modelHandle, index, 0, 0);
+		modelHandle = OS.g_menu_new();
+
+		switch (style & bits) {
+			case SWT.SEPARATOR:
+				handle = OS.g_menu_item_new_section(null, modelHandle);
+				parent.sectionModelHandle = modelHandle;
+				break;
+			case SWT.RADIO:
+				Menu popoverMenuWidget = parent;
+				while (popoverMenuWidget.actionGroup == 0) {
+					popoverMenuWidget = popoverMenuWidget.cascade.parent;
+				}
+
+				long stringVariantType = OS.g_variant_type_new(OS.G_VARIANT_TYPE_STRING);
+				actionHandle = OS.g_simple_action_new_stateful(
+						Converter.javaStringToCString(String.valueOf(this.hashCode())),
+						stringVariantType,
+						OS.g_variant_new_string(Converter.javaStringToCString("untoggled")));
+				OS.g_action_map_add_action(popoverMenuWidget.actionGroup, actionHandle);
+				actionName = String.valueOf(popoverMenuWidget.hashCode()) + "." + String.valueOf(this.hashCode()) + "::toggled";
+				handle = OS.g_menu_item_new(null, Converter.javaStringToCString(actionName));
+				OS.g_variant_type_free(stringVariantType);
+				break;
+			case SWT.CASCADE:
+				handle = OS.g_menu_item_new_submenu(null, modelHandle);
+				break;
+			case SWT.PUSH:
+			default:
+				popoverMenuWidget = parent;
+				while (popoverMenuWidget.actionGroup == 0) {
+					popoverMenuWidget = popoverMenuWidget.cascade.parent;
+				}
+
+				actionHandle = OS.g_simple_action_new(Converter.javaStringToCString(String.valueOf(this.hashCode())), 0);
+				OS.g_action_map_add_action(popoverMenuWidget.actionGroup, actionHandle);
+				actionName = String.valueOf(popoverMenuWidget.hashCode()) + "." + String.valueOf(this.hashCode());
+				handle = OS.g_menu_item_new(null, Converter.javaStringToCString(actionName));
+				break;
+		}
+
+		if ((style & SWT.SEPARATOR) != 0) {
+			OS.g_menu_insert_item(parent.modelHandle, index, handle);
+		} else {
+			OS.g_menu_insert_item(parent.sectionModelHandle, index, handle);
+		}
+
+		parentMenuPosition = index;
+		parent.items.add(this);
 	} else {
 		switch (style & bits) {
 			case SWT.SEPARATOR:
-				handle = GTK.gtk_separator_menu_item_new ();
+				handle = GTK3.gtk_separator_menu_item_new ();
 				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 				break;
 			case SWT.RADIO:
@@ -283,14 +333,14 @@ void createHandle (int index) {
 				* to the same group.  This allows the visible button to be
 				* unselected.
 				*/
-				groupHandle = GTK.gtk_radio_menu_item_new (0);
+				groupHandle = GTK3.gtk_radio_menu_item_new (0);
 				if (groupHandle == 0) error (SWT.ERROR_NO_HANDLES);
 				OS.g_object_ref_sink (groupHandle);
-				long group = GTK.gtk_radio_menu_item_get_group (groupHandle);
-				handle = GTK.gtk_radio_menu_item_new (group);
+				long group = GTK3.gtk_radio_menu_item_get_group (groupHandle);
+				handle = GTK3.gtk_radio_menu_item_new (group);
 				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				labelHandle = GTK.gtk_accel_label_new (buffer);
+				labelHandle = GTK3.gtk_accel_label_new (buffer);
 				if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
 				boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
@@ -302,10 +352,10 @@ void createHandle (int index) {
 				}
 				break;
 			case SWT.CHECK:
-				handle = GTK.gtk_check_menu_item_new ();
+				handle = GTK3.gtk_check_menu_item_new ();
 				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				labelHandle = GTK.gtk_accel_label_new (buffer);
+				labelHandle = GTK3.gtk_accel_label_new (buffer);
 				if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
 				boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
@@ -320,10 +370,10 @@ void createHandle (int index) {
 			// menus are used, the "head" menu item (such as File, Edit, Help, etc.) should
 			// not be padded. We only care about this in Gtk3.
 			case SWT.CASCADE:
-				handle = GTK.gtk_menu_item_new ();
+				handle = GTK3.gtk_menu_item_new ();
 				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				labelHandle = GTK.gtk_accel_label_new (buffer);
+				labelHandle = GTK3.gtk_accel_label_new (buffer);
 				if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
 				boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
@@ -338,10 +388,10 @@ void createHandle (int index) {
 				break;
 			case SWT.PUSH:
 			default:
-				handle = GTK.gtk_menu_item_new ();
+				handle = GTK3.gtk_menu_item_new ();
 				if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 
-				labelHandle = GTK.gtk_accel_label_new (buffer);
+				labelHandle = GTK3.gtk_accel_label_new (buffer);
 				if (labelHandle == 0) error (SWT.ERROR_NO_HANDLES);
 
 				boxHandle = gtk_box_new (GTK.GTK_ORIENTATION_HORIZONTAL, false, 6);
@@ -357,8 +407,12 @@ void createHandle (int index) {
 			if (OS.SWT_PADDED_MENU_ITEMS) {
 				GTK.gtk_image_set_pixel_size (imageHandle, 16);
 			}
-			GTK.gtk_container_add (boxHandle, imageHandle);
-			GTK.gtk_widget_show (imageHandle);
+			if (GTK.GTK4) {
+				GTK4.gtk_box_append(boxHandle, imageHandle);
+			} else {
+				GTK3.gtk_container_add (boxHandle, imageHandle);
+				GTK.gtk_widget_show (imageHandle);
+			}
 		}
 		if (labelHandle != 0) {
 			GTK.gtk_label_set_xalign (labelHandle, 0);
@@ -367,19 +421,23 @@ void createHandle (int index) {
 			GTK.gtk_widget_show (labelHandle);
 		}
 		if (boxHandle != 0) {
-			GTK.gtk_container_add (handle, boxHandle);
-			GTK.gtk_widget_show (boxHandle);
+			if (GTK.GTK4) {
+				// TODO: need to implement how menu items will be populated, currently no value for handle
+			} else {
+				GTK3.gtk_container_add (handle, boxHandle);
+				GTK.gtk_widget_show (boxHandle);
+			}
 		}
 		if ((style & SWT.SEPARATOR) == 0) {
 			if (boxHandle == 0) {
-				labelHandle = GTK.gtk_bin_get_child (handle);
+				labelHandle = GTK3.gtk_bin_get_child (handle);
 			}
-			GTK.gtk_accel_label_set_accel_widget (labelHandle, 0);
+			GTK3.gtk_accel_label_set_accel_widget (labelHandle, 0);
 		}
 		long parentHandle = parent.handle;
 		boolean enabled = GTK.gtk_widget_get_sensitive (parentHandle);
 		if (!enabled) GTK.gtk_widget_set_sensitive (parentHandle, true);
-		GTK.gtk_menu_shell_insert (parentHandle, handle, index);
+		GTK3.gtk_menu_shell_insert (parentHandle, handle, index);
 		if (!enabled) GTK.gtk_widget_set_sensitive (parentHandle, false);
 		GTK.gtk_widget_show (handle);
 	}
@@ -450,7 +508,12 @@ long getAccelGroup () {
  */
 public boolean getEnabled () {
 	checkWidget();
-	return GTK.gtk_widget_get_sensitive (handle);
+
+	if (GTK.GTK4) {
+		return OS.g_action_get_enabled(actionHandle);
+	} else {
+		return GTK.gtk_widget_get_sensitive(handle);
+	}
 }
 
 /**
@@ -527,7 +590,14 @@ public Menu getParent () {
 public boolean getSelection () {
 	checkWidget();
 	if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return false;
-	return GTK.gtk_check_menu_item_get_active(handle);
+
+	if (GTK.GTK4) {
+		long gVariantState = OS.g_action_get_state(actionHandle);
+		String stateString = Converter.cCharPtrToJavaString(OS.g_variant_get_string(gVariantState, null), false);
+		return stateString.equals("toggled");
+	} else {
+		return GTK3.gtk_check_menu_item_get_active(handle);
+	}
 }
 
 /**
@@ -550,19 +620,24 @@ public String getToolTipText () {
 @Override
 long gtk_activate (long widget) {
 	if ((style & SWT.CASCADE) != 0 && menu != null) return 0;
-	/*
-	* Bug in GTK.  When an ancestor menu is disabled and
-	* the user types an accelerator key, GTK delivers the
-	* the activate signal even though the menu item cannot
-	* be invoked using the mouse.  The fix is to ignore
-	* activate signals when an ancestor menu is disabled.
-	*/
-	if (!isEnabled ()) return 0;
+
+	if (!GTK.GTK4) {
+		/*
+		* Bug in GTK.  When an ancestor menu is disabled and
+		* the user types an accelerator key, GTK delivers the
+		* the activate signal even though the menu item cannot
+		* be invoked using the mouse.  The fix is to ignore
+		* activate signals when an ancestor menu is disabled.
+		*/
+		if (!isEnabled()) return 0;
+	}
+
 	if ((style & SWT.RADIO) != 0) {
 		if ((parent.getStyle () & SWT.NO_RADIO_GROUP) == 0) {
 			selectRadio ();
 		}
 	}
+
 	sendSelectionEvent (SWT.Selection);
 	return 0;
 }
@@ -584,7 +659,7 @@ long gtk_show_help (long widget, long helpType) {
 	}
 	if (handled) {
 		// TODO: GTK4 there is no idea of a menu shell
-		GTK.gtk_menu_shell_deactivate (parent.handle);
+		GTK3.gtk_menu_shell_deactivate (parent.handle);
 		return 1;
 	}
 	return 0;
@@ -593,9 +668,17 @@ long gtk_show_help (long widget, long helpType) {
 @Override
 void hookEvents () {
 	super.hookEvents ();
-	OS.g_signal_connect_closure (handle, OS.activate, display.getClosure (ACTIVATE), false);
-	OS.g_signal_connect_closure (handle, OS.select, display.getClosure (SELECT), false);
-	OS.g_signal_connect_closure_by_id (handle, display.signalIds [SHOW_HELP], 0, display.getClosure (SHOW_HELP), false);
+
+	if (GTK.GTK4) {
+		// Bind activate signal only for menu items with actions (SWT.CHECK and SWT.RADIO)
+		if (actionHandle != 0) {
+			OS.g_signal_connect(actionHandle, OS.activate, display.activateProc, handle);
+		}
+	} else {
+		OS.g_signal_connect_closure(handle, OS.activate, display.getClosure (ACTIVATE), false);
+		OS.g_signal_connect_closure (handle, OS.select, display.getClosure (SELECT), false);
+		OS.g_signal_connect_closure_by_id (handle, display.signalIds [SHOW_HELP], 0, display.getClosure (SHOW_HELP), false);
+	}
 }
 
 /**
@@ -763,10 +846,50 @@ void selectRadio () {
 public void setAccelerator (int accelerator) {
 	checkWidget();
 	if (this.accelerator == accelerator) return;
-	long accelGroup = getAccelGroup ();
-	if (accelGroup != 0) removeAccelerator (accelGroup);
-	this.accelerator = accelerator;
-	if (accelGroup != 0) addAccelerator (accelGroup);
+
+	if (GTK.GTK4) {
+		if (shortcutHandle != 0) {
+			GTK4.gtk_shortcut_controller_remove_shortcut(parent.shortcutController, shortcutHandle);
+		}
+
+		this.accelerator = accelerator;
+		addShortcut(accelerator);
+	} else {
+		long accelGroup = getAccelGroup();
+		if (accelGroup != 0) removeAccelerator(accelGroup);
+		this.accelerator = accelerator;
+		if (accelGroup != 0) addAccelerator(accelGroup);
+	}
+}
+
+void addShortcut(int accelerator) {
+	if (accelerator == 0 || !getEnabled()) return;
+	if ((accelerator & SWT.COMMAND) != 0) return;
+
+	int mask = 0;
+	if ((accelerator & SWT.ALT) != 0) mask |= GDK.GDK_MOD1_MASK;
+	if ((accelerator & SWT.SHIFT) != 0) mask |= GDK.GDK_SHIFT_MASK;
+	if ((accelerator & SWT.CONTROL) != 0) mask |= GDK.GDK_CONTROL_MASK;
+
+	int keyval = accelerator & SWT.KEY_MASK;
+	int newKey = Display.untranslateKey (keyval);
+	if (newKey != 0) {
+		keyval = newKey;
+	} else {
+		switch (keyval) {
+			case '\r': keyval = GDK.GDK_Return; break;
+			default: keyval = Converter.wcsToMbcs ((char) keyval);
+		}
+	}
+
+	if (keyval != 0) {
+		shortcutHandle = GTK4.gtk_shortcut_new(
+				GTK4.gtk_keyval_trigger_new(keyval, mask),
+				GTK4.gtk_named_action_new(Converter.javaStringToCString(actionName))
+			);
+		if (shortcutHandle == 0) error(SWT.ERROR_NO_HANDLES);
+		GTK4.gtk_shortcut_controller_add_shortcut(parent.shortcutController, shortcutHandle);
+	}
 }
 
 /**
@@ -784,11 +907,16 @@ public void setAccelerator (int accelerator) {
  */
 public void setEnabled (boolean enabled) {
 	checkWidget();
-	if (GTK.gtk_widget_get_sensitive (handle) == enabled) return;
-	long accelGroup = getAccelGroup ();
-	if (accelGroup != 0) removeAccelerator (accelGroup);
-	GTK.gtk_widget_set_sensitive (handle, enabled);
-	if (accelGroup != 0) addAccelerator (accelGroup);
+
+	if (GTK.GTK4) {
+		OS.g_simple_action_set_enabled(actionHandle, enabled);
+	} else {
+		if (GTK.gtk_widget_get_sensitive(handle) == enabled) return;
+		long accelGroup = getAccelGroup();
+		if (accelGroup != 0) removeAccelerator(accelGroup);
+		GTK.gtk_widget_set_sensitive(handle, enabled);
+		if (accelGroup != 0) addAccelerator(accelGroup);
+	}
 }
 
 /**
@@ -831,64 +959,52 @@ public void setID (int id) {
  */
 @Override
 public void setImage (Image image) {
+	//TODO: GTK4 Menu images with text are no longer supported
+	if (GTK.GTK4) return;
+
 	checkWidget();
 	if ((style & SWT.SEPARATOR) != 0) return;
 	super.setImage (image);
+
 	if (image != null) {
 		ImageList imageList = parent.imageList;
 		if (imageList == null) imageList = parent.imageList = new ImageList ();
 		int imageIndex = imageList.indexOf (image);
-		long pixbuf = 0;
+		long surface = 0;
 		if (imageIndex == -1) {
 			imageIndex = imageList.add (image);
-			pixbuf = imageList.getPixbuf (imageIndex);
-			/*
-			 * Bug in GTK the default renderer does scale again on pixbuf.
-			 * Need to scaledown here and no need to scaledown id device scale is 1
-			 *
-			 * We should resize pixbuf and update pixbuf array in the imagelist only if
-			 * the image is added to the imagelist in this call. If the image is already
-			 * add imageList.getPixbuf returns resized pixbuf.
-			 */
-			if (DPIUtil.useCairoAutoScale()) {
-				Rectangle imgSize = image.getBounds();
-				long scaledPixbuf = GDK.gdk_pixbuf_scale_simple(pixbuf, imgSize.width, imgSize.height, GDK.GDK_INTERP_BILINEAR);
-				if (scaledPixbuf !=0) {
-					pixbuf = scaledPixbuf;
-				}
-				imageList.replacePixbuf(imageIndex, pixbuf);
-			}
+			surface = imageList.getSurface (imageIndex);
 		} else {
 			imageList.put (imageIndex, image);
-			pixbuf = imageList.getPixbuf (imageIndex);
+			surface = imageList.getSurface (imageIndex);
 		}
 
-		if (!GTK.GTK4 && !GTK.GTK_IS_MENU_ITEM (handle)) return;
+		if (!GTK3.GTK_IS_MENU_ITEM (handle)) return;
 		if (OS.SWT_PADDED_MENU_ITEMS && imageHandle != 0) {
-			GTK.gtk_image_set_from_surface(imageHandle, image.surface);
+			GTK3.gtk_image_set_from_surface(imageHandle, surface);
 		} else {
-			if (imageHandle == 0 && boxHandle != 0) {
-				imageHandle = GTK.gtk_image_new_from_pixbuf (pixbuf);
-				GTK.gtk_container_add (boxHandle, imageHandle);
-				GTK.gtk_box_reorder_child (boxHandle, imageHandle, 0);
+			if (imageHandle == 0) {
+				imageHandle = GTK3.gtk_image_new_from_surface(surface);
+				if (imageHandle == 0) error(SWT.ERROR_NO_HANDLES);
+
+				GTK3.gtk_container_add(boxHandle, imageHandle);
+				GTK3.gtk_box_reorder_child(boxHandle, imageHandle, 0);
 			} else {
-				GTK.gtk_image_set_from_surface(imageHandle, image.surface);
+				GTK3.gtk_image_set_from_surface(imageHandle, surface);
 			}
-			if (boxHandle == 0) error (SWT.ERROR_NO_HANDLES);
 		}
-		if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
-		GTK.gtk_widget_show (imageHandle);
+		GTK.gtk_widget_show(imageHandle);
 	} else {
-		if (imageHandle != 0 && boxHandle != 0) {
+		if (imageHandle != 0) {
 			if (OS.SWT_PADDED_MENU_ITEMS) {
-				GTK.gtk_container_remove(boxHandle, imageHandle);
+				GTK3.gtk_container_remove(boxHandle, imageHandle);
 				imageHandle = GTK.gtk_image_new ();
 				if (imageHandle == 0) error (SWT.ERROR_NO_HANDLES);
 				GTK.gtk_image_set_pixel_size (imageHandle, 16);
-				GTK.gtk_container_add (boxHandle, imageHandle);
+				GTK3.gtk_container_add (boxHandle, imageHandle);
 				GTK.gtk_widget_show (imageHandle);
 			} else {
-				GTK.gtk_container_remove(boxHandle, imageHandle);
+				GTK3.gtk_container_remove(boxHandle, imageHandle);
 				imageHandle = 0;
 			}
 		}
@@ -939,31 +1055,37 @@ public void setMenu (Menu menu) {
 	/* Assign the new menu */
 	Menu oldMenu = this.menu;
 	if (oldMenu == menu) return;
-	long accelGroup = getAccelGroup ();
-	if (accelGroup != 0) removeAccelerators (accelGroup);
-	if (oldMenu != null) {
-		oldMenu.cascade = null;
-		/*
-		* Add a reference to the menu we are about
-		* to replace or GTK will destroy it.
-		*/
-		OS.g_object_ref (oldMenu.handle);
 
-		if (GTK.GTK4) {
-			OS.g_menu_item_set_submenu(handle, 0);
-		} else {
-			GTK.gtk_menu_item_set_submenu (handle, 0);
-		}
-	}
-	if ((this.menu = menu) != null) {
-		menu.cascade = this;
-		if (GTK.GTK4) {
+	if (GTK.GTK4) {
+		this.menu = menu;
+		if (menu != null) {
+			menu.cascade = this;
 			OS.g_menu_item_set_submenu(handle, menu.modelHandle);
 		} else {
-			GTK.gtk_menu_item_set_submenu (handle, menu.handle);
+			oldMenu.cascade = null;
+			OS.g_menu_item_set_submenu(handle, 0);
 		}
+
+		OS.g_menu_remove(parent.sectionModelHandle, parentMenuPosition);
+		OS.g_menu_insert_item(parent.sectionModelHandle, parentMenuPosition, handle);
+	} else {
+		long accelGroup = getAccelGroup ();
+		if (accelGroup != 0) removeAccelerators (accelGroup);
+		if (oldMenu != null) {
+			oldMenu.cascade = null;
+			/*
+			* Add a reference to the menu we are about
+			* to replace or GTK will destroy it.
+			*/
+			OS.g_object_ref (oldMenu.handle);
+			GTK3.gtk_menu_item_set_submenu (handle, 0);
+		}
+		if ((this.menu = menu) != null) {
+			menu.cascade = this;
+			GTK3.gtk_menu_item_set_submenu (handle, menu.handle);
+		}
+		if (accelGroup != 0) addAccelerators (accelGroup);
 	}
-	if (accelGroup != 0) addAccelerators (accelGroup);
 }
 
 @Override
@@ -972,7 +1094,7 @@ void setOrientation (boolean create) {
 	if ((parent.style & SWT.RIGHT_TO_LEFT) != 0 || !create) {
 		int dir = (parent.style & SWT.RIGHT_TO_LEFT) != 0 ? GTK.GTK_TEXT_DIR_RTL : GTK.GTK_TEXT_DIR_LTR;
 		GTK.gtk_widget_set_direction (handle, dir);
-		GTK.gtk_container_forall (handle, display.setDirectionProc, dir);
+		GTK3.gtk_container_forall (handle, display.setDirectionProc, dir);
 		if (menu != null) menu._setOrientation (parent.style & (SWT.RIGHT_TO_LEFT | SWT.LEFT_TO_RIGHT));
 	}
 }
@@ -1002,10 +1124,15 @@ boolean setRadioSelection (boolean value) {
 public void setSelection (boolean selected) {
 	checkWidget();
 	if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return;
-	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, ACTIVATE);
-	GTK.gtk_check_menu_item_set_active (handle, selected);
-	if ((style & SWT.RADIO) != 0) GTK.gtk_check_menu_item_set_active (groupHandle, !selected);
-	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, ACTIVATE);
+
+	if (GTK.GTK4) {
+		OS.g_simple_action_set_state(actionHandle, OS.g_variant_new_string(Converter.javaStringToCString(selected ? "toggled" : "untoggled")));
+	} else {
+		OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, ACTIVATE);
+		GTK3.gtk_check_menu_item_set_active (handle, selected);
+		if ((style & SWT.RADIO) != 0) GTK3.gtk_check_menu_item_set_active (groupHandle, !selected);
+		OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, ACTIVATE);
+	}
 }
 
 /**
@@ -1048,27 +1175,43 @@ public void setSelection (boolean selected) {
 @Override
 public void setText (String string) {
 	checkWidget();
-	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (string == null) error(SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
-	if (text.equals (string)) return;
-	super.setText (string);
-	int index = string.indexOf ('\t');
+	if (text.equals(string)) return;
+	super.setText(string);
+
+	int index = string.indexOf('\t');
 	if (index != -1) {
-		string = string.substring (0, index);
+		string = string.substring(0, index);
 	}
-	char [] chars = fixMnemonic (string);
-	byte [] buffer = Converter.wcsToMbcs (chars, true);
-	if (labelHandle != 0 && GTK.GTK_IS_LABEL (labelHandle)) {
-		GTK.gtk_label_set_text_with_mnemonic (labelHandle, buffer);
-		if (GTK.GTK_IS_ACCEL_LABEL (labelHandle)) {
-			MaskKeysym maskKeysym = getMaskKeysym();
-			if (maskKeysym != null) {
-				GTK.gtk_accel_label_set_accel_widget (labelHandle, handle);
-				GTK.gtk_accel_label_set_accel (labelHandle,
-						maskKeysym.keysym, maskKeysym.mask);
+	char[] chars = fixMnemonic(string);
+	byte[] buffer = Converter.wcsToMbcs(chars, true);
+
+	if (GTK.GTK4) {
+		OS.g_menu_item_set_label(handle, buffer);
+		MaskKeysym maskKeysym = getMaskKeysym();
+		if (maskKeysym != null) {
+			OS.g_menu_item_set_attribute(handle,
+					Converter.javaStringToCString("accel"),
+					Converter.javaStringToCString("s"),
+					GTK.gtk_accelerator_name(maskKeysym.keysym, maskKeysym.mask)
+				);
+		}
+		OS.g_menu_remove(parent.sectionModelHandle, parentMenuPosition);
+		OS.g_menu_insert_item(parent.sectionModelHandle, parentMenuPosition, handle);
+	} else {
+		if (labelHandle != 0 && GTK.GTK_IS_LABEL (labelHandle)) {
+			GTK.gtk_label_set_text_with_mnemonic (labelHandle, buffer);
+			if (GTK.GTK_IS_ACCEL_LABEL (labelHandle)) {
+				MaskKeysym maskKeysym = getMaskKeysym();
+				if (maskKeysym != null) {
+					GTK3.gtk_accel_label_set_accel_widget (labelHandle, handle);
+					GTK3.gtk_accel_label_set_accel (labelHandle,
+							maskKeysym.keysym, maskKeysym.mask);
+				}
+				// A workaround for Ubuntu Unity global menu
+				OS.g_signal_emit_by_name(handle, OS.accel_closures_changed);
 			}
-			// A workaround for Ubuntu Unity global menu
-			OS.g_signal_emit_by_name(handle, OS.accel_closures_changed);
 		}
 	}
 }
@@ -1130,9 +1273,9 @@ void updateAccelerator (long accelGroup, boolean add) {
 	/* When accel_key is zero, it causes GTK warnings */
 	if (keysym != 0) {
 		if (add) {
-			GTK.gtk_widget_add_accelerator (handle, OS.activate, accelGroup, keysym, mask, GTK.GTK_ACCEL_VISIBLE);
+			GTK3.gtk_widget_add_accelerator(handle, OS.activate, accelGroup, keysym, mask, GTK.GTK_ACCEL_VISIBLE);
 		} else {
-			GTK.gtk_widget_remove_accelerator (handle, accelGroup, keysym, mask);
+			GTK3.gtk_widget_remove_accelerator(handle, accelGroup, keysym, mask);
 		}
 	}
 }
@@ -1205,9 +1348,9 @@ boolean updateAcceleratorText (boolean show) {
 	if (maskKeysym.keysym != 0) {
 		long accelGroup = getAccelGroup ();
 		if (show) {
-			GTK.gtk_widget_add_accelerator (handle, OS.activate, accelGroup, maskKeysym.keysym, maskKeysym.mask, GTK.GTK_ACCEL_VISIBLE);
+			GTK3.gtk_widget_add_accelerator (handle, OS.activate, accelGroup, maskKeysym.keysym, maskKeysym.mask, GTK.GTK_ACCEL_VISIBLE);
 		} else {
-			GTK.gtk_widget_remove_accelerator (handle, accelGroup, maskKeysym.keysym, maskKeysym.mask);
+			GTK3.gtk_widget_remove_accelerator (handle, accelGroup, maskKeysym.keysym, maskKeysym.mask);
 		}
 	}
 	return maskKeysym.keysym != 0;
