@@ -19,6 +19,7 @@ import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.gtk3.*;
 
 /**
  * Instances of the receiver represent a selectable user interface object
@@ -141,10 +142,12 @@ Point computeSizeInPixels (int wHint, int hHint, boolean changed) {
 @Override
 void createHandle (int index) {
 	state |= HANDLE | THEME_BACKGROUND;
-	handle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
-	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
-	gtk_widget_set_has_surface_or_window (handle, true);
-	GTK.gtk_widget_set_can_focus (handle, true);
+
+	handle = OS.g_object_new(display.gtk_fixed_get_type(), 0);
+	if (handle == 0) error(SWT.ERROR_NO_HANDLES);
+	if (!GTK.GTK4) GTK3.gtk_widget_set_has_window(handle, true);
+	GTK.gtk_widget_set_can_focus(handle, true);
+
 	if ((style & SWT.VERTICAL) != 0) {
 		if (GTK.GTK4) {
 			defaultCursor = GDK.gdk_cursor_new_from_name("sb_h_double_arrow", 0);
@@ -408,14 +411,14 @@ long gtk_motion_notify_event (long widget, long eventPtr) {
 	}
 	if (isHint) {
 		int [] pointer_x = new int [1], pointer_y = new int [1], mask = new int [1];
-		display.gdk_window_get_device_position (gdkResource, pointer_x, pointer_y, mask);
+		display.getWindowPointerPosition (gdkResource, pointer_x, pointer_y, mask);
 		eventX = pointer_x [0];
 		eventY = pointer_y [0];
 		eventState = mask [0];
 	} else {
 		int [] origin_x = new int [1], origin_y = new int [1];
 		if (GTK.GTK4) {
-			GDK.gdk_surface_get_origin(gdkResource, origin_x, origin_y);
+			//TODO: GTK4 no gdk_surface_get_origin
 		} else {
 			GDK.gdk_window_get_origin (gdkResource, origin_x, origin_y);
 		}
@@ -481,7 +484,7 @@ long gtk_realize (long widget) {
 @Override
 void hookEvents () {
 	super.hookEvents ();
-	GTK.gtk_widget_add_events (handle, GDK.GDK_POINTER_MOTION_HINT_MASK);
+	if (!GTK.GTK4) GTK3.gtk_widget_add_events(handle, GDK.GDK_POINTER_MOTION_HINT_MASK);
 }
 
 @Override
