@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2019 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -1179,10 +1179,14 @@ boolean setTabGroupFocus () {
 	if (takeFocus && setTabItemFocus ()) return true;
 	Control [] children = _getChildren ();
 	for (Control child : children) {
-		if (child.isTabItem () && child.setRadioFocus (true)) return true;
+		/*
+		 * It is unlikely but possible that a child is disposed at this point, for more
+		 * details refer bug 381668.
+		 */
+		if (!child.isDisposed() && child.isTabItem() && child.setRadioFocus (true)) return true;
 	}
 	for (Control child : children) {
-		if (child.isTabItem () && !child.isTabGroup () && child.setTabItemFocus ()) {
+		if (!child.isDisposed() && child.isTabItem () && !child.isTabGroup () && child.setTabItemFocus ()) {
 			return true;
 		}
 	}
