@@ -236,21 +236,6 @@ static int checkStyle (int style) {
 	*/
 	if ((style & SWT.CLOSE) != 0) style |= SWT.TITLE;
 
-	/*
-	* Bug in Windows.  The WS_CAPTION style must be
-	* set when the window is resizable or it does not
-	* draw properly.
-	*/
-	/*
-	* This code is intentionally commented.  It seems
-	* that this problem originally in Windows 3.11,
-	* has been fixed in later versions.  Because the
-	* exact nature of the drawing problem is unknown,
-	* keep the commented code around in case it comes
-	* back.
-	*/
-//	if ((style & SWT.RESIZE) != 0) style |= SWT.TITLE;
-
 	return style;
 }
 
@@ -354,8 +339,7 @@ void createAccelerators () {
 	ACCEL accel = new ACCEL ();
 	byte [] buffer1 = new byte [ACCEL.sizeof];
 	byte [] buffer2 = new byte [items.length * ACCEL.sizeof];
-	for (int i=0; i<items.length; i++) {
-		MenuItem item = items [i];
+	for (MenuItem item : items) {
 		if (item != null && item.accelerator != 0) {
 			Menu menu = item.parent;
 			if (menu.parent == this) {
@@ -410,8 +394,7 @@ public void dispose () {
 
 Menu findMenu (long hMenu) {
 	if (menus == null) return null;
-	for (int i=0; i<menus.length; i++) {
-		Menu menu = menus [i];
+	for (Menu menu : menus) {
 		if (menu != null && hMenu == menu.handle) return menu;
 	}
 	return null;
@@ -729,8 +712,7 @@ void releaseChildren (boolean destroy) {
 	}
 	super.releaseChildren (destroy);
 	if (menus != null) {
-		for (int i=0; i<menus.length; i++) {
-			Menu menu = menus [i];
+		for (Menu menu : menus) {
 			if (menu != null && !menu.isDisposed ()) {
 				menu.dispose ();
 			}
@@ -766,8 +748,7 @@ void removeMenu (Menu menu) {
 void reskinChildren (int flags) {
 	if (menuBar != null) menuBar.reskin (flags);
 	if (menus != null) {
-		for (int i=0; i<menus.length; i++) {
-			Menu menu = menus [i];
+		for (Menu menu : menus) {
 			if (menu != null) menu.reskin (flags);
 		}
 	}
@@ -993,8 +974,8 @@ void setImages (Image image, Image [] images) {
 public void setImages (Image [] images) {
 	checkWidget ();
 	if (images == null) error (SWT.ERROR_INVALID_ARGUMENT);
-	for (int i = 0; i < images.length; i++) {
-		if (images [i] == null || images [i].isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+	for (Image image : images) {
+		if (image == null || image.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
 	}
 	this.images = images;
 	setImages (null, images);
@@ -1073,7 +1054,7 @@ public void setMenuBar (Menu menu) {
  * always. This should be avoided if possible.
  * </p>
  *
- * @param minimized the new maximized state
+ * @param minimized the new minimized state
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -1092,8 +1073,7 @@ public void setMinimized (boolean minimized) {
 public void setOrientation (int orientation) {
 	super.setOrientation (orientation);
 	if (menus != null) {
-		for (int i=0; i<menus.length; i++) {
-			Menu menu = menus [i];
+		for (Menu menu : menus) {
 			if (menu != null && !menu.isDisposed () && (menu.getStyle () & SWT.POP_UP) != 0) {
 				menu._setOrientation (menu.getOrientation ());
 			}
@@ -1240,7 +1220,6 @@ void setSystemMenu () {
 public void setText (String string) {
 	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
-	/* Use the character encoding for the default locale */
 	TCHAR buffer = new TCHAR (0, string, true);
 	/* Ensure that the title appears in the task bar.*/
 	if ((state & FOREIGN_HANDLE) != 0) {
