@@ -152,7 +152,7 @@ void destroyWidget () {
 	releaseHandle ();
 }
 
-long /*int*/ fontHandle (int index) {
+long fontHandle (int index) {
 	if (cellFont != null && cellFont [index] != null) return cellFont [index].handle;
 	if (font != null) return font.handle;
 	return -1;
@@ -256,7 +256,7 @@ RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean
 	return getBounds (row, column, getText, getImage, fullText, false, 0);
 }
 
-RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean fullText, boolean fullImage, long /*int*/ hDC) {
+RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean fullText, boolean fullImage, long hDC) {
 	if (!getText && !getImage) return new RECT ();
 	int columnCount = parent.getColumnCount ();
 	if (!(0 <= column && column < Math.max (1, columnCount))) {
@@ -264,31 +264,31 @@ RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean
 	}
 	if (parent.fixScrollWidth) parent.setScrollWidth (null, true);
 	RECT rect = new RECT ();
-	long /*int*/ hwnd = parent.handle;
-	int bits = (int)/*64*/OS.SendMessage (hwnd, OS.LVM_GETEXTENDEDLISTVIEWSTYLE, 0, 0);
+	long hwnd = parent.handle;
+	int bits = (int)OS.SendMessage (hwnd, OS.LVM_GETEXTENDEDLISTVIEWSTYLE, 0, 0);
 	if (column == 0 && (bits & OS.LVS_EX_FULLROWSELECT) == 0) {
 		if (parent.explorerTheme) {
 			rect.left = OS.LVIR_ICON;
 			parent.ignoreCustomDraw = true;
-			long /*int*/ code = OS.SendMessage (hwnd, OS. LVM_GETITEMRECT, row, rect);
+			long code = OS.SendMessage (hwnd, OS. LVM_GETITEMRECT, row, rect);
 			parent.ignoreCustomDraw = false;
 			if (code == 0) return new RECT ();
 			if (getText) {
 				int width = 0;
-				long /*int*/ hFont = fontHandle (column);
+				long hFont = fontHandle (column);
 				if (hFont == -1 && hDC == 0) {
 					TCHAR buffer = new TCHAR (parent.getCodePage (), text, true);
-					width = (int)/*64*/OS.SendMessage (hwnd, OS.LVM_GETSTRINGWIDTH, 0, buffer);
+					width = (int)OS.SendMessage (hwnd, OS.LVM_GETSTRINGWIDTH, 0, buffer);
 				} else {
-					TCHAR buffer = new TCHAR (parent.getCodePage (), text, false);
-					long /*int*/ textDC = hDC != 0 ? hDC : OS.GetDC (hwnd), oldFont = -1;
+					char [] buffer = text.toCharArray ();
+					long textDC = hDC != 0 ? hDC : OS.GetDC (hwnd), oldFont = -1;
 					if (hDC == 0) {
 						if (hFont == -1) hFont = OS.SendMessage (hwnd, OS.WM_GETFONT, 0, 0);
 						oldFont = OS.SelectObject (textDC, hFont);
 					}
 					RECT textRect = new RECT ();
 					int flags = OS.DT_NOPREFIX | OS.DT_SINGLELINE | OS.DT_CALCRECT;
-					OS.DrawText (textDC, buffer, buffer.length (), textRect, flags);
+					OS.DrawText (textDC, buffer, buffer.length, textRect, flags);
 					width = textRect.right - textRect.left;
 					if (hDC == 0) {
 						if (oldFont != -1) OS.SelectObject (textDC, oldFont);
@@ -302,7 +302,7 @@ RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean
 			if (getText) {
 				rect.left = OS.LVIR_SELECTBOUNDS;
 				parent.ignoreCustomDraw = true;
-				long /*int*/ code = OS.SendMessage (hwnd, OS.LVM_GETITEMRECT, row, rect);
+				long code = OS.SendMessage (hwnd, OS.LVM_GETITEMRECT, row, rect);
 				parent.ignoreCustomDraw = false;
 				if (code == 0) return new RECT ();
 				if (!getImage) {
@@ -316,14 +316,14 @@ RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean
 			} else {
 				rect.left = OS.LVIR_ICON;
 				parent.ignoreCustomDraw = true;
-				long /*int*/ code = OS.SendMessage (hwnd, OS.LVM_GETITEMRECT, row, rect);
+				long code = OS.SendMessage (hwnd, OS.LVM_GETITEMRECT, row, rect);
 				parent.ignoreCustomDraw = false;
 				if (code == 0) return new RECT ();
 			}
 		}
 		if (fullText || fullImage) {
 			RECT headerRect = new RECT ();
-			long /*int*/ hwndHeader = OS.SendMessage (hwnd, OS.LVM_GETHEADER, 0, 0);
+			long hwndHeader = OS.SendMessage (hwnd, OS.LVM_GETHEADER, 0, 0);
 			OS.SendMessage (hwndHeader, OS.HDM_GETITEMRECT, 0, headerRect);
 			OS.MapWindowPoints (hwndHeader, hwnd, headerRect, 2);
 			if (getText && fullText) rect.right = headerRect.right;
@@ -347,7 +347,7 @@ RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean
 			*/
 			rect.left = getText ? OS.LVIR_LABEL : OS.LVIR_ICON;
 			parent.ignoreCustomDraw = true;
-			long /*int*/ code = OS.SendMessage (hwnd, OS. LVM_GETSUBITEMRECT, row, rect);
+			long code = OS.SendMessage (hwnd, OS. LVM_GETSUBITEMRECT, row, rect);
 			parent.ignoreCustomDraw = false;
 			if (code == 0) return new RECT ();
 			/*
@@ -381,7 +381,7 @@ RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean
 			}
 			if (column == 0 && fullImage) {
 				RECT headerRect = new RECT ();
-				long /*int*/ hwndHeader = OS.SendMessage (hwnd, OS.LVM_GETHEADER, 0, 0);
+				long hwndHeader = OS.SendMessage (hwnd, OS.LVM_GETHEADER, 0, 0);
 				OS.SendMessage (hwndHeader, OS.HDM_GETITEMRECT, 0, headerRect);
 				OS.MapWindowPoints (hwndHeader, hwnd, headerRect, 2);
 				rect.left = headerRect.left;
@@ -389,7 +389,7 @@ RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean
 		} else {
 			rect.left = OS.LVIR_ICON;
 			parent.ignoreCustomDraw = true;
-			long /*int*/ code = OS.SendMessage (hwnd, OS. LVM_GETSUBITEMRECT, row, rect);
+			long code = OS.SendMessage (hwnd, OS. LVM_GETSUBITEMRECT, row, rect);
 			parent.ignoreCustomDraw = false;
 			if (code == 0) return new RECT ();
 			if (!hasImage) rect.right = rect.left;
@@ -397,9 +397,9 @@ RECT getBounds (int row, int column, boolean getText, boolean getImage, boolean
 				String string = column == 0 ? text : strings != null ? strings [column] : null;
 				if (string != null) {
 					RECT textRect = new RECT ();
-					TCHAR buffer = new TCHAR (parent.getCodePage (), string, false);
+					char [] buffer = string.toCharArray ();
 					int flags = OS.DT_NOPREFIX | OS.DT_SINGLELINE | OS.DT_CALCRECT;
-					OS.DrawText (hDC, buffer, buffer.length (), textRect, flags);
+					OS.DrawText (hDC, buffer, buffer.length, textRect, flags);
 					rect.right += textRect.right - textRect.left + Table.INSET * 3 + 2;
 				}
 			}
@@ -706,7 +706,7 @@ Rectangle getTextBoundsInPixels (int index) {
 
 void redraw () {
 	if (parent.currentItem == this || !parent.getDrawing ()) return;
-	long /*int*/ hwnd = parent.handle;
+	long hwnd = parent.handle;
 	if (!OS.IsWindowVisible (hwnd)) return;
 	int index = parent.indexOf (this);
 	if (index == -1) return;
@@ -715,7 +715,7 @@ void redraw () {
 
 void redraw (int column, boolean drawText, boolean drawImage) {
 	if (parent.currentItem == this || !parent.getDrawing ()) return;
-	long /*int*/ hwnd = parent.handle;
+	long hwnd = parent.handle;
 	if (!OS.IsWindowVisible (hwnd)) return;
 	int index = parent.indexOf (this);
 	if (index == -1) return;
@@ -885,7 +885,7 @@ public void setFont (Font font){
 	if ((parent.style & SWT.VIRTUAL) == 0 && cached) {
 		int itemIndex = parent.indexOf (this);
 		if (itemIndex != -1) {
-			long /*int*/ hwnd = parent.handle;
+			long hwnd = parent.handle;
 			LVITEM lvItem = new LVITEM ();
 			lvItem.mask = OS.LVIF_TEXT;
 			lvItem.iItem = itemIndex;
@@ -949,7 +949,7 @@ public void setFont (int index, Font font) {
 		if ((parent.style & SWT.VIRTUAL) == 0 && cached) {
 			int itemIndex = parent.indexOf (this);
 			if (itemIndex != -1) {
-				long /*int*/ hwnd = parent.handle;
+				long hwnd = parent.handle;
 				LVITEM lvItem = new LVITEM ();
 				lvItem.mask = OS.LVIF_TEXT;
 				lvItem.iItem = itemIndex;
@@ -1141,7 +1141,6 @@ public void setImage (Image image) {
  *
  * @param indent the new indent
  *
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -1160,7 +1159,7 @@ public void setImageIndent (int indent) {
 	} else {
 		int index = parent.indexOf (this);
 		if (index != -1) {
-			long /*int*/ hwnd = parent.handle;
+			long hwnd = parent.handle;
 			LVITEM lvItem = new LVITEM ();
 			lvItem.mask = OS.LVIF_INDENT;
 			lvItem.iItem = index;
@@ -1247,7 +1246,7 @@ public void setText (int index, String string) {
 		if ((parent.style & SWT.VIRTUAL) == 0 && cached) {
 			int itemIndex = parent.indexOf (this);
 			if (itemIndex != -1) {
-				long /*int*/ hwnd = parent.handle;
+				long hwnd = parent.handle;
 				LVITEM lvItem = new LVITEM ();
 				lvItem.mask = OS.LVIF_TEXT;
 				lvItem.iItem = itemIndex;
