@@ -330,6 +330,22 @@ public Image getHotImage () {
 	return hotImage;
 }
 
+/**
+ * Returns the receiver's enabled image if it has one, or null
+ * if it does not.
+ *
+ * @return the receiver's enabled image
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ */
+@Override
+public Image getImage () {
+	return super.getImage();
+}
+
 /**
  * Returns the receiver's parent, which must be a <code>ToolBar</code>.
  *
@@ -431,8 +447,8 @@ public boolean isEnabled () {
 boolean isTabGroup () {
 	ToolItem [] tabList = parent._getTabItemList ();
 	if (tabList != null) {
-		for (int i=0; i<tabList.length; i++) {
-			if (tabList [i] == this) return true;
+		for (ToolItem item : tabList) {
+			if (item == this) return true;
 		}
 	}
 	if ((style & SWT.SEPARATOR) != 0) return true;
