@@ -22,14 +22,13 @@ import org.eclipse.swt.internal.win32.*;
 /**
  * Instances of this class represent icons that can be placed on the
  * system tray or task bar status area.
- * <p>
  * <dl>
  * <dt><b>Styles:</b></dt>
  * <dd>(none)</dd>
  * <dt><b>Events:</b></dt>
  * <dd>DefaultSelection, MenuDetect, Selection</dd>
  * </dl>
- * </p><p>
+ * <p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
  *
@@ -263,7 +262,7 @@ public boolean getVisible () {
 	return visible;
 }
 
-long /*int*/ messageProc (long /*int*/ hwnd, int msg, long /*int*/ wParam, long /*int*/ lParam) {
+long messageProc (long hwnd, int msg, long wParam, long lParam) {
 	/*
 	* Feature in Windows.  When the user clicks on the tray
 	* icon, another application may be the foreground window.
@@ -275,7 +274,7 @@ long /*int*/ messageProc (long /*int*/ hwnd, int msg, long /*int*/ wParam, long
 	* menu is not hidden.  The fix is to force the tray icon
 	* message window to the foreground when sending an event.
 	*/
-	switch ((int)/*64*/lParam) {
+	switch ((int)lParam) {
 		case OS.WM_LBUTTONDOWN:
 			if (hooks (SWT.Selection)) {
 				OS.SetForegroundWindow (hwnd);
@@ -460,7 +459,7 @@ public void setImage (Image image) {
 	super.setImage (image);
 	if (image2 != null) image2.dispose ();
 	image2 = null;
-	long /*int*/ hIcon = 0;
+	long hIcon = 0;
 	Image icon = image;
 	if (icon != null) {
 		switch (icon.type) {
@@ -517,7 +516,7 @@ public void setToolTip (ToolTip toolTip) {
  * </p>
  * <p>
  * NOTE: This operation is a hint and behavior is platform specific, on Windows
- * for CJK-style mnemonics of the form " (&C)" at the end of the tooltip text
+ * for CJK-style mnemonics of the form " (&amp;C)" at the end of the tooltip text
  * are not shown in tooltip.
  * </p>
  *
