@@ -344,11 +344,6 @@ public class PaintExample {
 	 */
 	public void dispose() {
 		if (paintSurface != null) paintSurface.dispose();
-		if (paintColors != null) {
-			for (final Color color : paintColors) {
-				if (color != null) color.dispose();
-			}
-		}
 		paintDefaultFont = null;
 		paintColors = null;
 		paintSurface = null;
@@ -411,8 +406,8 @@ public class PaintExample {
 	private void init() {
 		Display display = mainComposite.getDisplay();
 
-		paintColorWhite = new Color(display, 255, 255, 255);
-		paintColorBlack = new Color(display, 0, 0, 0);
+		paintColorWhite = new Color(255, 255, 255);
+		paintColorBlack = new Color(0, 0, 0);
 
 		paintDefaultFont = display.getSystemFont();
 
@@ -420,8 +415,8 @@ public class PaintExample {
 		paintColors[0] = paintColorBlack;
 		paintColors[1] = paintColorWhite;
 		for (int i = 2; i < paintColors.length; i++) {
-			paintColors[i] = new Color(display,
-				((i*7)%255),((i*23)%255), ((i*51)%255));
+			paintColors[i] = new Color(((i*7)%255),
+				((i*23)%255),((i*51)%255));
 		}
 
 		toolSettings = new ToolSettings();
