@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -18,23 +18,20 @@ package org.eclipse.swt.snippets;
  *
  * For a list of all SWT example snippets see
  * http://www.eclipse.org/swt/snippets/
- * 
+ *
  * @since 3.2
  */
 import org.eclipse.swt.*;
-import org.eclipse.swt.layout.*;
-import org.eclipse.swt.widgets.*;
 import org.eclipse.swt.graphics.*;
-import org.eclipse.swt.opengl.GLCanvas;
-import org.eclipse.swt.opengl.GLData;
+import org.eclipse.swt.layout.*;
+import org.eclipse.swt.opengl.*;
+import org.eclipse.swt.widgets.*;
 
-import javax.media.opengl.GL;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLDrawableFactory;
-import javax.media.opengl.glu.GLU;
+import com.jogamp.opengl.*;
+import com.jogamp.opengl.glu.*;
 
 public class Snippet209 {
-	static void drawTorus(GL gl, float r, float R, int nsides, int rings) {
+	static void drawTorus(GL2 gl, float r, float R, int nsides, int rings) {
 		float ringDelta = 2.0f * (float) Math.PI / rings;
 		float sideDelta = 2.0f * (float) Math.PI / nsides;
 		float theta = 0.0f, cosTheta = 1.0f, sinTheta = 0.0f;
@@ -42,7 +39,7 @@ public class Snippet209 {
 			float theta1 = theta + ringDelta;
 			float cosTheta1 = (float) Math.cos(theta1);
 			float sinTheta1 = (float) Math.sin(theta1);
-			gl.glBegin(GL.GL_QUAD_STRIP);
+			gl.glBegin(GL2.GL_QUAD_STRIP);
 			float phi = 0.0f;
 			for (int j = nsides; j >= 0; j--) {
 				phi += sideDelta;
@@ -72,7 +69,7 @@ public class Snippet209 {
 		final GLCanvas canvas = new GLCanvas(comp, SWT.NONE, data);
 
 		canvas.setCurrent();
-		final GLContext context = GLDrawableFactory.getFactory().createExternalGLContext();
+		final GLContext context = GLDrawableFactory.getFactory(GLProfile.getGL2GL3()).createExternalGLContext();;
 
 		canvas.addListener(SWT.Resize, new Listener() {
 			@Override
@@ -81,23 +78,23 @@ public class Snippet209 {
 				float fAspect = (float) bounds.width / (float) bounds.height;
 				canvas.setCurrent();
 				context.makeCurrent();
-				GL gl = context.getGL ();
+				GL2 gl = (GL2) context.getGL ();
 				gl.glViewport(0, 0, bounds.width, bounds.height);
-				gl.glMatrixMode(GL.GL_PROJECTION);
+				gl.glMatrixMode(GL2.GL_PROJECTION);
 				gl.glLoadIdentity();
 				GLU glu = new GLU();
 				glu.gluPerspective(45.0f, fAspect, 0.5f, 400.0f);
-				gl.glMatrixMode(GL.GL_MODELVIEW);
+				gl.glMatrixMode(GL2.GL_MODELVIEW);
 				gl.glLoadIdentity();
 				context.release();
 			}
 		});
 
 		context.makeCurrent();
-		GL gl = context.getGL ();
+		GL2 gl = (GL2) context.getGL ();
 		gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
 		gl.glColor3f(1.0f, 0.0f, 0.0f);
-		gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
+		gl.glHint(GL2.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
 		gl.glClearDepth(1.0);
 		gl.glLineWidth(2);
 		gl.glEnable(GL.GL_DEPTH_TEST);
@@ -114,7 +111,7 @@ public class Snippet209 {
 				if (!canvas.isDisposed()) {
 					canvas.setCurrent();
 					context.makeCurrent();
-					GL gl = context.getGL ();
+					GL2 gl = (GL2) context.getGL ();
 					gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
 					gl.glClearColor(.3f, .5f, .8f, 1.0f);
 					gl.glLoadIdentity();
@@ -123,7 +120,7 @@ public class Snippet209 {
 					gl.glRotatef(0.15f * rot, 2.0f * frot, 10.0f * frot, 1.0f);
 					gl.glRotatef(0.3f * rot, 3.0f * frot, 1.0f * frot, 1.0f);
 					rot++;
-					gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_LINE);
+					gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL2.GL_LINE);
 					gl.glColor3f(0.9f, 0.9f, 0.9f);
 					drawTorus(gl, 1, 1.9f + ((float) Math.sin((0.004f * frot))), 15, 15);
 					canvas.swapBuffers();
