@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
+ * Copyright (c) 2000, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -14,6 +14,7 @@
 package org.eclipse.swt.tests.junit;
 
 
+import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
@@ -62,6 +63,54 @@ public void test_getToolTipText() {
 public void test_setImageLorg_eclipse_swt_graphics_Image() {
 }
 
+@Test
+public void test_setDisabledImage() {
+		toolItem.setImage(images[0]);
+		toolItem.setDisabledImage(images[1]);
+		toolItem.setEnabled(false);
+		assertEquals(images[0], toolItem.getImage());
+
+		toolItem.setEnabled(true);
+		assertEquals(images[0], toolItem.getImage());
+
+		toolItem.setDisabledImage(images[0]);
+		assertEquals(images[0], toolItem.getImage());
+
+		toolItem.setEnabled(false);
+		assertEquals(images[0], toolItem.getImage());
+
+		toolItem.setImage(images[0]);
+		toolItem.setEnabled(true);
+		assertEquals(images[0], toolItem.getImage());
+
+		toolItem.setDisabledImage(images[2]);
+		toolItem.setEnabled(false);
+		assertEquals(images[0], toolItem.getImage());
+
+		toolItem.setEnabled(true);
+		toolItem.setDisabledImage(null);
+		toolItem.setEnabled(false);
+		assertEquals(images[0], toolItem.getImage());
+
+		toolItem.setEnabled(true);
+		toolItem.setDisabledImage(null);
+		assertEquals(images[0], toolItem.getImage());
+
+		toolItem.setEnabled(false);
+		toolItem.setDisabledImage(null);
+		assertEquals(images[0], toolItem.getImage());
+
+		toolItem.setImage(null);
+		assertEquals(null, toolItem.getImage());
+		toolItem.setEnabled(false);
+		toolItem.setDisabledImage(images[1]);
+		toolItem.setImage(images[1]);
+		assertEquals(images[1], toolItem.getImage());
+		toolItem.setImage(null);
+		assertEquals(null, toolItem.getImage());
+		assertEquals(images[1], toolItem.getDisabledImage());
+}
+
 @Override
 @Test
 public void test_setTextLjava_lang_String() {
