@@ -745,7 +745,7 @@ public void test_setBackgroundILorg_eclipse_swt_graphics_Color() {
 	assertEquals(tree.getBackground(),treeItem.getBackground(0));
 
 	try {
-		Color color = new Color(display, 255, 0, 0);
+		Color color = new Color(255, 0, 0);
 		color.dispose();
 		treeItem.setBackground(color);
 		fail("No exception thrown for color disposed");
@@ -755,7 +755,7 @@ public void test_setBackgroundILorg_eclipse_swt_graphics_Color() {
 
 @Test
 public void test_setBackgroundLorg_eclipse_swt_graphics_Color() {
-	Color color = new Color(treeItem.getDisplay(), 255, 0, 0);
+	Color color = new Color(255, 0, 0);
 	treeItem.setBackground(color);
 	assertEquals(color, treeItem.getBackground());
 	treeItem.setBackground(null);
@@ -919,7 +919,7 @@ public void test_setForegroundILorg_eclipse_swt_graphics_Color() {
 	assertEquals(tree.getForeground(),treeItem.getForeground(0));
 
 	try {
-		Color color = new Color(display, 255, 0, 0);
+		Color color = new Color(255, 0, 0);
 		color.dispose();
 		treeItem.setForeground(color);
 		fail("No exception thrown for color disposed");
@@ -929,7 +929,7 @@ public void test_setForegroundILorg_eclipse_swt_graphics_Color() {
 
 @Test
 public void test_setForegroundLorg_eclipse_swt_graphics_Color() {
-	Color color = new Color(treeItem.getDisplay(), 255, 0, 0);
+	Color color = new Color(255, 0, 0);
 	treeItem.setForeground(color);
 	assertEquals(color, treeItem.getForeground());
 	treeItem.setForeground(null);
